#pragma once

#include <vector>

#include "types.h"

class CVBody;
class CVirtualBodiesList;

namespace nV
{

	class Shape;
	class Primitive;
	class BodyMngr;

	class Body
	{
		friend class JointMngr;
		friend class BodyMngr;

		CVBody * pBody;
		BodyMngr* pManager;

	protected:
		Body(CVBody*, BodyMngr*);
		~Body(void);

	public:

		std::vector<nV::Primitive*> Primitives;

		Primitive* AddPlane(nvReal nx,nvReal ny,nvReal nz,nvReal d);
		Primitive* AddPlane(nvReal xPointOnPlane,nvReal yPointOnPlane,nvReal zPointOnPlane,nvReal xNormalOnPlane,nvReal yNormalOnPlane,nvReal zNormalOnPlane);
		Primitive* AddBox(nvReal mass,nvReal xLength,nvReal yLength,nvReal zLength);
		Primitive* AddSphere(nvReal mass,nvReal radius);
		Primitive* AddCylinder(nvReal mass,nvReal radius,nvReal height);
		Primitive* AddConvex(nvReal mass);
		Primitive* AddTriMesh(nvReal mass);

		Primitive* AddNewPrimitive(nV::Shape* pShap,nvReal mass,nvReal* pWorldPosition=0,nvReal* pWorldRotation=0);

		
		void RotateAxisAngle(	nvReal xAxis,nvReal yAxis,nvReal zAxis,nvReal angle);
		void Move( nvReal deltaX,nvReal deltaY,nvReal deltaZ);
		void AddPermanentForce(	nvReal xForceOnCenterOfMass,nvReal yForceOnCenterOfMass,nvReal zForceOnCenterOfMass);
		void AddPermanentForce(	nvReal xBodySpaceForce,nvReal yBodySpaceForce,nvReal zBodySpaceForce,
								nvReal xBodySpacePointOfAct,nvReal yBodySpacePointOfAct,nvReal zBodySpacePointOfAct);
		void AddOneFrameForce(	nvReal xForceOnCenterOfMass,nvReal yForceOnCenterOfMass,nvReal zForceOnCenterOfMass);
		void AddOneFrameForce(	nvReal xBodySpaceForce,nvReal yBodySpaceForce,nvReal zBodySpaceForce,
								nvReal xBodySpacePointOfAct,nvReal yBodySpacePointOfAct,nvReal zBodySpacePointOfAct);

		void AddOneFrameTorque( nvReal xTorque,nvReal yTorque,nvReal zTorque);
		void AddPermanentTorque( nvReal xTorque,nvReal yTorque,nvReal zTorque);

		/* Setter & Getter */
		nvReal* GetRotationMatrix();
		void GetPositon(nvReal &xWorldPosition,nvReal &yWorldPosition,nvReal &zWorldPosition);
		void SetPosition(nvReal xWorldPosition,nvReal yWorldPosition,nvReal zWorldPosition);
		void GetAngularVelocity(nvReal &xAngularVelocity,nvReal &yAngularVelocity,nvReal &zAngularVelocity);
		void SetAngularVelocity(nvReal xAngularVelocity,nvReal yAngularVelocity,nvReal zAngularVelocity);
		void GetLinearVelocity(nvReal &xLinearVelocity,nvReal &yLinearVelocity,nvReal &zLinearVelocity);
		void SetLinearVelocity(nvReal xLinearVelocity,nvReal yLinearVelocity,nvReal zLinearVelocity);
		void SetData(void* pUserData);
		void* GetData();
		nvReal GetMass();
		void GetCenterOfMass(nvReal &xWorldCenterOfMass,nvReal &yWorldCenterOfMass,nvReal &zWorldCenterOfMass);
		bool IsSleeping();
		bool IsStatic();

	};

}
