#pragma once
#include ".\shape.h"

class CConvexShape;

namespace nV
{
	class ConvexShape :
		public Shape
	{
		friend class ShapeMngr;
		friend class Body;

		CConvexShape* pShape;
	
	protected:
		ConvexShape(CConvexShape*);
		~ConvexShape();
	public:

		virtual nvReal GetBoundingRadius();
		virtual nvReal GetBoundingRadiusSqr();


		// get the type before u cast
		virtual Shape::eShapeType GetType();


		//////////////////////////////////////
		//ConvexShape Specifics:

		// a faster way to add vertices should be in this form
		// send a pointer to the first vertex
		// and the stride (in bytes) from one vertex to the next
		// data will be copied locally
		void AddVertices(void* pStartVertex,int stride,int vertexCount);

		// add a new edge, by putting 2 vertices, and one edge
		void AddEdge(nvReal* pVertexA,nvReal* pVertexB);

		// add a new face..
		// should create one face, and a number of edges= number of vertices, and add the vertices
		// we start with a triangle
		// the 3 vertices are in CCW order
		void AddTriangle(nvReal* pVertexA,nvReal* pVertexB,nvReal* pVertexC);

		// adds a list of triangles
		// using just 1 pointer
		// we take 3 x,y,z values by jumping by the stride vlues
		// each 3 vertices creates a new face
		void AddTriangles(void* pStartVertex,int stride,int numVerts);

		// adds a list of indexed triangles
		// using just 1 pointer to the vertices
		// and a pointer to the indices
		// we take 3 x,y,z values by jumping by the stride values
		// each 3 vertices creates a new face
		void AddIndexedTriangles(void* pStartVertex,int* pIndices,int stride,int indexCount);

		// add a triangles strip
		// same as the previous, but the vertices are added in a different way
		// each vertex takes the 2 previous ones, and inverts the order of them
		void AddTriangleStrip(void* pStartVertex,int stride,int vertexCount);

		// add an indexed triangles strip
		// same as the previous, but the vertices are added in a different way
		// each vertex takes the 2 previous ones, and inverts the order of them
		void AddIndexedTriangleStrip(void* pStartVertex,int* pIndices,int stride,int indexCount);

		// adds a triangle fan
		void AddTriangleFan(void* pStartVertex,int stride,int vertexCount);

		// adds an indexed triangle fan
		void AddIndexedTriangleFan(void* pStartVertex,int stride,int vertexCount);

		// these 2 function calls are essential 
		// vertices should only be added between a begin and an end
		void Begin();

		// call this when done with adding vertices
		void End();

	};
}