#pragma once
#include "types.h"
#include <list>

class CJointsMngr;

struct VECTOR3;


namespace nV
{
	class Body;
	class Primitive;
	class Joint;

	class JointMngr
	{
		friend class PhysicsWorld;

		CJointsMngr * pJointsMngr;
		std::list<nV::Joint*> Joints;
	public:
		JointMngr(void);
		~JointMngr(void);

		nV::Joint* CreateBallJoint(nV::Body *pBdy1, nV::Body *pBdy2,
			nvReal xGlobalPoint,nvReal yGlobalPoint,nvReal zGlobalPoint);

		nV::Joint* CreateBallJoint(nV::Body *pBdy1,nvReal r1x,nvReal r1y,nvReal r1z,
						nV::Body *pBdy2,nvReal r2x,nvReal r2y,nvReal r2z);

		nV::Joint* CreateCarWheelJoint(nV::Body *pBdy1, nV::Body *pBdy2, 
			nvReal xGlobalPoint,nvReal yGlobalPoint,nvReal zGlobalPoint,
			nvReal xAxis,nvReal yAxis,nvReal zAxis,
			nvReal xSteeringAxis,nvReal ySteeringAxis,nvReal zSteeringAxis, nvReal Stiffness=100.0f);

		nV::Joint* CreateContactJoint(nV::Primitive* pPrimitive1,nV::Primitive* pPrimitive2,VECTOR3 P,
			VECTOR3 B2toB1normal,nvReal overlappedDistance,bool isFirstPoint,bool IsOneFrameJoint);

		nV::Joint* CreateFixedJoint(Body* pBdy1,Body* pBdy2);

		nV::Joint* CreateFrictionJoint(const nV::Primitive* pPrim1,const nV::Primitive* pPrim2,const VECTOR3 &B2toB1normal,const VECTOR3 &NormalPerp1,const VECTOR3 &NormalPerp2,const VECTOR3 &P,const int NoOfFricitonPoints,const bool bIsFirst,const bool IsOneFrameJoint);

		nV::Joint* CreateHingeJoint(Body * pBdy1, Body *pBdy2, 
			nvReal xGlobalPoint,nvReal yGlobalPoint,nvReal zGlobalPoint,
			nvReal xAxis,nvReal yAxis,nvReal zAxis);

		nV::Joint* CreateDoubleHingeJoint(Body * pBdy1, Body *pBdy2, 
			nvReal xGlobalPoint,nvReal yGlobalPoint,nvReal zGlobalPoint,
			nvReal xAxis1,nvReal yAxis1,nvReal zAxis1,
			nvReal xAxis2,nvReal yAxis2,nvReal zAxis2);

		nV::Joint* CreateUniversalJoint(Body * pBdy1, Body *pBdy2, 
			nvReal xGlobalPoint,nvReal yGlobalPoint,nvReal zGlobalPoint,
			nvReal xAxis1,nvReal yAxis1,nvReal zAxis1,
			nvReal xAxis2,nvReal yAxis2,nvReal zAxis2);

		nV::Joint* CreateLimitedBallJoint(Body * pBdy1, Body *pBdy2, 
			nvReal xGlobalPoint,nvReal yGlobalPoint,nvReal zGlobalPoint,
			nvReal xAxis1,nvReal yAxis1,nvReal zAxis1,
			nvReal xAxis2,nvReal yAxis2,nvReal zAxis2);

		nV::Joint* CreateStiffSpringJoint(Body* pBdy1,
			nvReal xR1,nvReal yR1,nvReal zR1,
			Body* pBdy2,
			nvReal xR2,nvReal yR2,nvReal zR2,
			nvReal L, nvReal Stiffness=1.0f, nvReal Dampenning=0.9f);

		void DestroyJoint(nV::Joint* pJ);

		void DeleteAllBodyJoints(nV::Body* pBody);
	};

}