#pragma once
#include ".\types.h"

#include <list>

class CShapesMngr;

namespace nV
{

	class Shape;
	class PlaneShape; 
	class BoxShape;
	class SphereShape;
	class CylinderShape;
	class ConvexShape;
	class TriMeshShape;

	class ShapeMngr
	{
		friend class PhysicsWorld;

		::CShapesMngr* pShapeMngr;
		std::list<nV::Shape*> Shapes;
	protected:
		ShapeMngr(void);
		~ShapeMngr(void);

	public:

		// create different shapes
		PlaneShape* CreatePlane(nvReal nx,nvReal ny,nvReal nz,nvReal d);
		PlaneShape* CreatePlane(nvReal xPointOnPlane,nvReal yPointOnPlane,nvReal zPointOnPlane,nvReal xNormalOnPlane,nvReal yNormalOnPlane,nvReal zNormalOnPlane);
		BoxShape* CreateBox(nvReal xLength,nvReal yLength,nvReal zLength);
		SphereShape* CreateSphere(nvReal radius);
		CylinderShape* CreateCylinder(nvReal radius,nvReal height);
		ConvexShape* CreateConvex();
		TriMeshShape* CreateTriMesh();
	    
		// destroy a shape
		// rarely used
		void DestroyShape(Shape* pShape);

		// this destroy all the shapes, should be called at the destruction of the engine
		void DestroyAllShapes();
	};

}