#include "boxmesh.h"


struct sBoxVertex
{
    Vector3 Pos;
    Vector3 Normal;
    Vector2 TexCoord;
};


// this code will create the mesh of the box
// it is an indexed mesh, with texture coordinates and normals
void sBoxMesh::Create(float sizeX,float sizeY,float sizeZ,float texScaleX,float texScaleY,float texScaleZ)
{
    Clear();
    m_VertexFormat          =   MESH_VERTEXFORMAT_POSITION|MESH_VERTEXFORMAT_TEXCOORD0|MESH_VERTEXFORMAT_NORMAL;
    m_RenderMode            =   MESH_RENDERMODE_TRIANGLES;

    // this would be the new vertex
    sBoxVertex boxVertex;

    // start by the front face
    boxVertex.Normal.Assign(0.0f,0.0f,1.0f);
    boxVertex.Pos.z=sizeZ/2.0f;


    boxVertex.Pos.x=-sizeX/2.0f;
    boxVertex.Pos.y=-sizeY/2.0f;
    boxVertex.TexCoord.x=0.0f;
    boxVertex.TexCoord.y=0.0f;
    AddVertex(boxVertex.Pos.v);
    //m_Vertices.push_back(boxVertex);

    boxVertex.Pos.x=sizeX/2.0f;
    boxVertex.Pos.y=-sizeY/2.0f;
    boxVertex.TexCoord.x=texScaleX;
    boxVertex.TexCoord.y=0.0f;
    AddVertex(boxVertex.Pos.v);
    //m_Vertices.push_back(boxVertex);

    boxVertex.Pos.x=sizeX/2.0f;
    boxVertex.Pos.y=sizeY/2.0f;
    boxVertex.TexCoord.x=texScaleX;
    boxVertex.TexCoord.y=texScaleY;
    AddVertex(boxVertex.Pos.v);

    boxVertex.Pos.x=-sizeX/2.0f;
    boxVertex.Pos.y=sizeY/2.0f;
    boxVertex.TexCoord.x=0.0f;
    boxVertex.TexCoord.y=texScaleY;
    AddVertex(boxVertex.Pos.v);

    AddIndex(0);
    AddIndex(1);
    AddIndex(2);

    AddIndex(3);
    AddIndex(0);
    AddIndex(2);


    // back face
    boxVertex.Normal.Assign(0.0f,0.0f,-1.0f);
    boxVertex.Pos.z=-sizeZ/2.0f;

    boxVertex.Pos.x=sizeX/2.0f;
    boxVertex.Pos.y=sizeY/2.0f;
    boxVertex.TexCoord.x=texScaleX;
    boxVertex.TexCoord.y=texScaleY;
    AddVertex(boxVertex.Pos.v);

    boxVertex.Pos.x=sizeX/2.0f;
    boxVertex.Pos.y=-sizeY/2.0f;
    boxVertex.TexCoord.x=texScaleX;
    boxVertex.TexCoord.y=0.0f;
    AddVertex(boxVertex.Pos.v);

    boxVertex.Pos.x=-sizeX/2.0f;
    boxVertex.Pos.y=-sizeY/2.0f;
    boxVertex.TexCoord.x=0.0f;
    boxVertex.TexCoord.y=0.0f;
    AddVertex(boxVertex.Pos.v);

    boxVertex.Pos.x=-sizeX/2.0f;
    boxVertex.Pos.y=sizeY/2.0f;
    boxVertex.TexCoord.x=0.0f;
    boxVertex.TexCoord.y=texScaleY;
    AddVertex(boxVertex.Pos.v);


    AddIndex(4);
    AddIndex(5);
    AddIndex(6);

    AddIndex(7);
    AddIndex(4);
    AddIndex(6);


    // top face
    boxVertex.Normal.Assign(0.0f,1.0f,0.0f);
    boxVertex.Pos.y=sizeY/2.0f;

    boxVertex.Pos.x=sizeX/2.0f;
    boxVertex.Pos.z=sizeZ/2.0f;
    boxVertex.TexCoord.x=texScaleX;
    boxVertex.TexCoord.y=texScaleZ;
    AddVertex(boxVertex.Pos.v);


    boxVertex.Pos.x=sizeX/2.0f;
    boxVertex.Pos.z=-sizeZ/2.0f;
    boxVertex.TexCoord.x=texScaleX;
    boxVertex.TexCoord.y=0.0f;
    AddVertex(boxVertex.Pos.v);

    boxVertex.Pos.x=-sizeX/2.0f;
    boxVertex.Pos.z=-sizeZ/2.0f;
    boxVertex.TexCoord.x=0.0f;
    boxVertex.TexCoord.y=0.0f;
    AddVertex(boxVertex.Pos.v);

    boxVertex.Pos.x=-sizeX/2.0f;
    boxVertex.Pos.z=sizeZ/2.0f;
    boxVertex.TexCoord.x=0.0f;
    boxVertex.TexCoord.y=texScaleZ;
    AddVertex(boxVertex.Pos.v);

    AddIndex(8);
    AddIndex(9);
    AddIndex(10);

    AddIndex(11);
    AddIndex(8);
    AddIndex(10);



    // bottom face
    boxVertex.Normal.Assign(0.0f,-1.0f,0.0f);
    boxVertex.Pos.y=-sizeY/2.0f;

    boxVertex.Pos.x=-sizeX/2.0f;
    boxVertex.Pos.z=sizeZ/2.0f;
    boxVertex.TexCoord.x=0.0f;
    boxVertex.TexCoord.y=texScaleZ;
    AddVertex(boxVertex.Pos.v);

    boxVertex.Pos.x=-sizeX/2.0f;
    boxVertex.Pos.z=-sizeZ/2.0f;
    boxVertex.TexCoord.x=0.0f;
    boxVertex.TexCoord.y=0.0f;
    AddVertex(boxVertex.Pos.v);

    boxVertex.Pos.x=sizeX/2.0f;
    boxVertex.Pos.z=-sizeZ/2.0f;
    boxVertex.TexCoord.x=texScaleX;
    boxVertex.TexCoord.y=0.0f;
    AddVertex(boxVertex.Pos.v);

    boxVertex.Pos.x=sizeX/2.0f;
    boxVertex.Pos.z=sizeZ/2.0f;
    boxVertex.TexCoord.x=texScaleX;
    boxVertex.TexCoord.y=texScaleZ;
    AddVertex(boxVertex.Pos.v);

    AddIndex(12);
    AddIndex(13);
    AddIndex(14);

    AddIndex(15);
    AddIndex(12);
    AddIndex(14);



    // right face
    boxVertex.Normal.Assign(1.0f,0.0f,0.0f);
    boxVertex.Pos.x=-sizeX/2.0f;

    boxVertex.Pos.y=-sizeY/2.0f;
    boxVertex.Pos.z=sizeZ/2.0f;
    boxVertex.TexCoord.y=0.0f;
    boxVertex.TexCoord.x=texScaleZ;
    AddVertex(boxVertex.Pos.v);

    boxVertex.Pos.y=sizeY/2.0f;
    boxVertex.Pos.z=sizeZ/2.0f;
    boxVertex.TexCoord.y=texScaleY;
    boxVertex.TexCoord.x=texScaleZ;
    AddVertex(boxVertex.Pos.v);

    boxVertex.Pos.y=sizeY/2.0f;
    boxVertex.Pos.z=-sizeZ/2.0f;
    boxVertex.TexCoord.y=texScaleY;
    boxVertex.TexCoord.x=0.0f;
    AddVertex(boxVertex.Pos.v);


    boxVertex.Pos.y=-sizeY/2.0f;
    boxVertex.Pos.z=-sizeZ/2.0f;
    boxVertex.TexCoord.x=0.0f;
    boxVertex.TexCoord.y=0.0f;
    AddVertex(boxVertex.Pos.v);

    AddIndex(16);
    AddIndex(17);
    AddIndex(18);

    AddIndex(19);
    AddIndex(16);
    AddIndex(18);

    // left face
    boxVertex.Normal.Assign(1.0f,0.0f,0.0f);
    boxVertex.Pos.x=sizeX/2.0f;


    boxVertex.Pos.y=-sizeY/2.0f;
    boxVertex.Pos.z=-sizeZ/2.0f;
    boxVertex.TexCoord.y=0.0f;
    boxVertex.TexCoord.x=0.0f;
    AddVertex(boxVertex.Pos.v);
    

    boxVertex.Pos.y=sizeY/2.0f;
    boxVertex.Pos.z=-sizeZ/2.0f;
    boxVertex.TexCoord.y=texScaleY;
    boxVertex.TexCoord.x=0.0f;
    AddVertex(boxVertex.Pos.v);
    

    boxVertex.Pos.y=sizeY/2.0f;
    boxVertex.Pos.z=sizeZ/2.0f;
    boxVertex.TexCoord.y=texScaleY;
    boxVertex.TexCoord.x=texScaleZ;
    AddVertex(boxVertex.Pos.v);
    

    boxVertex.Pos.y=-sizeY/2.0f;
    boxVertex.Pos.z=sizeZ/2.0f;
    boxVertex.TexCoord.y=0.0f;
    boxVertex.TexCoord.x=texScaleZ;
    AddVertex(boxVertex.Pos.v);
    
    AddIndex(20);
    AddIndex(21);
    AddIndex(22);

    AddIndex(23);
    AddIndex(20);
    AddIndex(22);


    // this will make it ready for rendering
    Compile();
}



