#include "system/engine.h"
#include "3d/Graphics/graphicssubsystem.h"
#include "3d/3dtypes/3dtypes.h"
#include "util/util.h"
#include "audio/audio.h"
#include "raytracing.h"

#include "3d/text/text.h"
// this is for ODE
//#include "../physics/physics.h"

// and this is for TPE
#include "nvphysics.h"

// header implemented here
#include "tpe.h"
#include "bridge.h"
#include "boxmesh.h"




int     g_BridgeDemoNumPlanks=10;

float   g_BridgeDemoGravity=-9.8f;
float   g_BridgeDemoPlankDensity=0.01f;
float   g_BridgeDemoPlankFriction=0.1f;
float   g_BridgeDemoPlankElasticity=0.1f;
float   g_BridgeDemoPlankSpacing=0.5f;
float   g_BridgeDemoBridgeRadius=80.0f;
float   g_BridgeDemoBridgeStartAngle=250.0f;
float   g_BridgeDemoBridgeEndAngle=290.0f;
float   g_BridgeDemoBallsSpacing=5.0f;

Vector3 g_FirstHingeLoc;
Vector3 g_BridgeDemoPlankDims(5.0f,1.4f,35.0f);
Vector3 g_BridgeDemoBridgeCenter(0.0f,80.0f,0.0f);


float g_BridgeDemoBallRadius=4.0f;
float g_BridgeDemoBallDensity=10.1f;
float g_BridgeDemoAppliedTorque=1000.0f;
float g_BridgeDemoAppliedForce=100.0f;

cMusic* g_pBridgeMusic=NULL;
sTexture* g_pBridgeWoodtexture=NULL;
sBoxMesh g_BridgePlankMesh;
int g_BridgePlankIndex=-1;


cCamera g_BridgeCamera;
Vector3 g_BridgeClickPos;


void SetBridgeCamera(cCamera& camera)
{
    g_BridgeCamera=camera;
}



// the bridge point is stored here... just a point and an angle
struct sBridgePoint
{
    Vector3 Pos;
    float Angle;
};

// all bridge points are here
vector<sBridgePoint> g_BridgePoints;

// compute the points between which we will create the bride planks
void ComputeBridgePoints()
{
    float stepAngle=(float)(g_BridgeDemoBridgeEndAngle-g_BridgeDemoBridgeStartAngle)/g_BridgeDemoNumPlanks;
    sBridgePoint newBPoint;
    for(float currAngle=g_BridgeDemoBridgeStartAngle;currAngle<=g_BridgeDemoBridgeEndAngle;currAngle+=stepAngle)
    {
        newBPoint.Angle=currAngle*PI/180.0f;
        newBPoint.Pos=g_BridgeDemoBridgeCenter;
        newBPoint.Pos.x+=cosf(newBPoint.Angle)*g_BridgeDemoBridgeRadius;
        newBPoint.Pos.y+=sinf(newBPoint.Angle)*g_BridgeDemoBridgeRadius;
        g_BridgePoints.push_back(newBPoint);
    }
}

vector<nV::Body*> g_TPEPlankBodies;


// create the Bridge scene here
void CreateBridgeTPE()
{
    // the physics engine itself
    pTPEEngine=new PhysicsWorld();
    SetEngine(pTPEEngine);
    pTPEEngine->SetGravity(0,g_BridgeDemoGravity,0);

    float plankMass=g_BridgeDemoPlankDensity*g_BridgeDemoPlankDims.x*g_BridgeDemoPlankDims.y*g_BridgeDemoPlankDims.z;


    // compute the hanging points of the bridge
    ComputeBridgePoints();


    float totalBridgeLength=(g_BridgeDemoPlankSpacing+g_BridgeDemoPlankDims.x)*g_BridgeDemoNumPlanks;


    // a dummy ghost body
    nV::Body* pGhostBody=pTPEEngine->BodyMngr.Create();
    pGhostBody->AddSphere(-1,0.0001f);
    pGhostBody->SetPosition(0,20000,0);

    // create the planks here
    for(unsigned int i=0;i<g_BridgePoints.size()-1;i++)
    {
        Vector3& startLoc=-g_BridgePoints[i].Pos;
        Vector3& endLoc=-g_BridgePoints[i+1].Pos;
        Vector3 midPoint=(endLoc+startLoc)/2.0f;
        float startAngle=g_BridgePoints[i].Angle;
        float endAngle=g_BridgePoints[i+1].Angle;
        float midAngle=(startAngle+endAngle)/2.0f;

        nV::Body* pNewTPEBody=pTPEEngine->BodyMngr.Create();
        nV::Primitive* pBox=pNewTPEBody->AddBox(plankMass,g_BridgeDemoPlankDims.x,g_BridgeDemoPlankDims.y,g_BridgeDemoPlankDims.z);
        pNewTPEBody->SetPosition(midPoint.x,midPoint.y,midPoint.z);
        pBox->SetElasticity(g_BridgeDemoPlankElasticity);
        pBox->SetFriction(g_BridgeDemoPlankFriction);
        pNewTPEBody->RotateAxisAngle(0,0,1,midAngle+PI/2.0f);

        // make first joint
        if(i==0)
        {
			pTPEEngine->JointMngr.CreateHingeJoint(pNewTPEBody,pGhostBody,startLoc.x,startLoc.y,startLoc.z,0.0f,0.0f,1.0f);
        }
        // intermediate joints
        else
        {
			pTPEEngine->JointMngr.CreateHingeJoint(g_TPEPlankBodies.back(),pNewTPEBody,startLoc.x,startLoc.y,startLoc.z,0,0,1);
        }

        // and the final joint
        if(i==(g_BridgePoints.size()-2))
        {
			pTPEEngine->JointMngr.CreateHingeJoint(pNewTPEBody,pGhostBody,endLoc.x,endLoc.y,endLoc.z,0,0,1);
        }

        // add the body to the vector
        g_TPEPlankBodies.push_back(pNewTPEBody);
    }
}




void RenderBridgeForSelection()
{

    Vector3 cameraLoc;
    Vector3 mouseRay;
    Graphics()->ComputeMouseRay(Mouse()->m_Loc.x,Mouse()->m_Loc.y,cameraLoc.v,mouseRay.v);

    Segment mouseSeg;
    mouseSeg.v0=g_BridgeCamera.m_Loc;
    mouseSeg.v1=g_BridgeCamera.m_Loc+mouseRay*10.0f;

    if(Mouse()->m_bLButtonDown&&(g_BridgePlankIndex==-1))
    {
        for(unsigned int i=0;i<g_TPEPlankBodies.size();i++)
        {
            float dist=10000.0f;
            nV::Body* pCurrentPlank=g_TPEPlankBodies[i];
            //Vector3 position;
            Matrix4 currMatrix;
            ComputeTPEBodyTransform(pCurrentPlank,currMatrix);
            //pCurrentPlank->GetPositon(position.x,position.y,position.z);

            //if(RayHitSphere(cameraLoc,mouseRay,position,g_BridgeDemoPlankDims.x/2.0f))
            if(RayHitBox(cameraLoc,mouseRay,currMatrix,g_BridgeDemoPlankDims.x,g_BridgeDemoPlankDims.y,g_BridgeDemoPlankDims.z))
            {
                float newDist=(g_BridgeCamera.m_Loc-currMatrix.GetLoc()).Mag();
                if(newDist<dist)
                {
                    g_BridgePlankIndex=i;
                    dist=newDist;
                }
            }
        }
    }
    mouseSeg.v1=g_BridgeCamera.m_Loc+mouseRay*10.0f;
    Plane plane(0,0,1,0);
    g_BridgeClickPos=plane.Split(mouseSeg.v0,mouseSeg.v1);

    if(!Mouse()->m_bLButtonDown)
    {
        g_BridgePlankIndex=-1;
    }
}




// render the Bridge scene here
void RenderBridgeTPE(float dt,bool updateOnly)
{
    if(KEYUP(VK_ADD)&&KEYUP(VK_SUBTRACT)&&KEYUP(VK_INSERT)&&KEYUP(VK_DELETE))
        pTPEEngine->RunStep(dt);

    if(updateOnly)
        return;

    RenderBridgeForSelection();


    Matrix4 currMatrix;


    // render the planks themselves here
    g_pBridgeWoodtexture->Activate();

    g_BridgePlankMesh.Enable();
    for(unsigned int i=0;i<g_TPEPlankBodies.size();i++)
    {
        nV::Body* pCurrentPlank=g_TPEPlankBodies[i];
        ComputeTPEBodyTransform(pCurrentPlank,currMatrix);

        Graphics()->PushMatrix();
        Graphics()->MultMatrix(currMatrix.d);
      
        g_BridgePlankMesh.RenderIndexTrianglesRange(0,0,g_BridgePlankMesh.m_NumIndices);
        Graphics()->PopMatrix();
    }
    g_BridgePlankMesh.Disable();
    g_pBridgeWoodtexture->Deactivate();
    

    if(g_BridgePlankIndex>-1)
    {
        nV::Body* pDraggedBrick=g_TPEPlankBodies[g_BridgePlankIndex];
        Vector3 brickPosition;
        pDraggedBrick->GetPositon(brickPosition.x,brickPosition.y,brickPosition.z);


        sMesh mesh;
        mesh.m_VertexFormat=MESH_VERTEXFORMAT_POSITION;
        mesh.m_RenderMode=MESH_RENDERMODE_LINELIST;
        mesh.AddVertex(brickPosition.v);
        mesh.AddVertex(g_BridgeClickPos.v);

        mesh.Compile();
        mesh.Render();
    }
}


// destroy the Bridge here
void DestroyBridgeTPE()
{
    g_TPEPlankBodies.clear();
    g_BridgePoints.clear();
    SAFE_DELETE(pTPEEngine);
    SetEngine(NULL);
}





// push the Bridge here
void PushBridgeTPE()
{
    int i=g_TPEPlankBodies.size()/2;
    nV::Body* pBody=g_TPEPlankBodies[i];

    if(KEYDOWN(VK_ADD))
    {
        g_BridgeDemoBridgeCenter.y+=1.0f;
        g_BridgeDemoBridgeRadius+=1.0f;

        DestroyBridgeTPE();
        CreateBridgeTPE();
    }
    if(KEYDOWN(VK_SUBTRACT))
    {
        g_BridgeDemoBridgeCenter.y-=1.0f;
        g_BridgeDemoBridgeRadius-=1.0f;

        DestroyBridgeTPE();
        CreateBridgeTPE();
    }

    if(KEYDOWN(VK_INSERT))
    {
        g_BridgeDemoNumPlanks++;
        g_BridgeDemoBridgeCenter.y+=9.0f;
        g_BridgeDemoBridgeRadius+=9.0f;

        DestroyBridgeTPE();
        CreateBridgeTPE();
    }
    if(KEYDOWN(VK_DELETE))
    {
        g_BridgeDemoNumPlanks--;
        g_BridgeDemoBridgeCenter.y-=9.0f;
        g_BridgeDemoBridgeRadius-=9.0f;


        if(g_BridgeDemoNumPlanks<1)
            g_BridgeDemoNumPlanks=1;
        DestroyBridgeTPE();
        CreateBridgeTPE();
    }

    if(g_BridgePlankIndex>-1)
    {
        nV::Body* pDraggedPlank=g_TPEPlankBodies[g_BridgePlankIndex];
        Vector3 BallPosition;
        pDraggedPlank->GetPositon(BallPosition.x,BallPosition.y,BallPosition.z);
        Vector3 Force=g_BridgeClickPos-BallPosition;

        float Mag=Force.Mag();
        Force.Normalize();
        Force*=Mag*Mag*1.0f;
        pDraggedPlank->AddOneFrameForce(Force.x,Force.y,Force.z);
        
    }
}






void CreateBridge(int APIID)
{
    g_pBridgeWoodtexture=new sTexture("data/wood003.tga");
    g_pBridgeMusic=new cMusic("data/meditation.mid");
    g_pBridgeMusic->Play();
    g_BridgePlankMesh.Create(g_BridgeDemoPlankDims.x,g_BridgeDemoPlankDims.y,g_BridgeDemoPlankDims.z,g_BridgeDemoPlankDims.x/10.0f,g_BridgeDemoPlankDims.y/10.0f,g_BridgeDemoPlankDims.z/10.0f);
    CreateBridgeTPE();
}


void RenderBridge(float dt,int APIID,bool updateOnly)
{
     RenderBridgeTPE(dt,updateOnly);
}
void PushBridge(int APIID)
{
      PushBridgeTPE();
}
void DestroyBridge(int APIID)
{
    DestroyBridgeTPE();
    SAFE_DELETE(g_pBridgeWoodtexture);
    g_pBridgeMusic->Stop();
    SAFE_DELETE(g_pBridgeMusic);
}

void DisplayBridgeHelp()
{
    TextPrintf(10,50,"Press +/- to change the bridge length");
    TextPrintf(10,30,"Press insert/delete to change number of planks");
    TextPrintf(10,70,"Use mouse wheel to zoom in out");
    TextPrintf(10,90,"Hold right mouse button to change camera");
    TextPrintf(10,110,"Hold left mouse on a plank to pull it");
    TextPrintf(10,130,"%d Planks",g_BridgeDemoNumPlanks); 
}



