
#include "system/engine.h"
#include "3d/graphics/graphicssubsystem.h"
#include "3d/3dtypes/3dtypes.h"


#include "boxmesh.h"
#include "car.h"

#include "3d/text/text.h"
#include "util/util.h"
#include "audio/audio.h"
#include "raytracing.h"
#include "boxmesh.h"

#include "3d/3dtypes/skybox.h"
#include "3d/md3/md3.h"

// and this is for TPE
#include "nvPhysics.h"

float g_CameraRadius=10.0f;
// header implemented here.
#include "car.h"
#include "vehicule.h"



// the playground dimension (the ground)
float       g_CarDemoGravity=-190.8f;
float       g_CarDemoDensity=0.001f;
float       g_CarDemoFriction=5000.0f;
float       g_CarDemoElasticity=0.2f;


cVehicule* g_pVehicule=NULL;
nV::Body* g_pCarGround=NULL;

sBoxMesh* g_pCarGroundMesh=NULL;





cSkyBox* pTestSkyBox=NULL;

CModelMD3* pMd3=NULL;
sTexture* pDummyTexture=NULL;

sMD3Model *pCarBody=NULL;
sMD3Model *pWheel=NULL;

Vector3 g_CarDemoGroundDims(4000.0f,10.0f,4000.0f);
cCamera g_CarDemoCamera;

cTPECollidableBSPFile* g_pCarBSPFile=NULL;

void SetCarDemoCamera(cCamera& camera)
{
    g_CarDemoCamera=camera;
}



void CreateCarDemo(int APIID)
{
    // the physics engine itself
    pTPEEngine=new PhysicsWorld();
    pTPEEngine->SetGravity(0.0f,g_CarDemoGravity,0.0f);
    pTPEEngine->SetNoOfSolvingIterations(50);
    pTPEEngine->SetObjectsAutoDeactivation(false);


    //g_pCarGround=pTPEEngine->BodyMngr.Create();
//    g_pCarGround->AddBox(-1.0f,g_CarDemoGroundDims.x,g_CarDemoGroundDims.y,g_CarDemoGroundDims.z);
  //  g_pCarGround->SetPosition(0,-10,0);
    //g_pCarGround->RotateAxisAngle(1,0,0,0.0f);

    g_pCarGroundMesh=new sBoxMesh;
    g_pCarGroundMesh->Create(g_CarDemoGroundDims.x,g_CarDemoGroundDims.y,g_CarDemoGroundDims.z,40.0f,1.0f,40.0f);

    pTestSkyBox=new cSkyBox();
    pTestSkyBox->Create("data/city",4000,2000,4000);

    
    pMd3=new CModelMD3();
//    pMd3->LoadModel("sorlag","sorlag");
  //  pMd3->LoadWeapon("lara","railgun");
    //pMd3->SetTorsoAnimation("TORSO_STAND");
    //pMd3->SetLegsAnimation("LEGS_IDLE");
    pDummyTexture=new sTexture("lara/r_flag2.tga");


    pCarBody=new sMD3Model();
    pMd3->CreateModel(pCarBody,"cars/roadster/body.md3","cars/roadster/yellow.skin","cars/slingshot");

    pWheel=new sMD3Model();
    pMd3->CreateModel(pWheel,"cars/raptor/wheel.md3","cars/wheels/brey_inspiration.skin","cars/wheels");


    g_pVehicule=new cVehicule();

    pCarBody->GetChildPosition("tag_wheelfr",g_pVehicule->m_FrontRightPos);
    pCarBody->GetChildPosition("tag_wheelfl",g_pVehicule->m_FrontLeftPos);
    pCarBody->GetChildPosition("tag_wheelrr",g_pVehicule->m_RearRightPos);
    pCarBody->GetChildPosition("tag_wheelrl",g_pVehicule->m_RearLeftPos);

    g_pVehicule->m_BodyPos.Assign(0,10,0);

    g_pVehicule->m_BodyMass=300.0f;
    g_pVehicule->m_WheelMass=10.0f;

    


    pDummyTexture=new sTexture("data/bottic2.jpg");


    // load the bsp file
    g_pCarBSPFile=new cTPECollidableBSPFile();
    g_pCarBSPFile->Load("maps/level.bsp");

    // create bodies
    //  g_pBSPFile->BuildCollisionTree(pLevelShape);
    vector<nV::ConvexShape*> convexShapes;
    g_pCarBSPFile->BuildConvexes(pTPEEngine,convexShapes);

    for(int i=0;i<convexShapes.size();i++)
    {
        nV::Body* pNewLevelBody=pTPEEngine->BodyMngr.Create();
        nV::Primitive* pLevelPrimitive=pNewLevelBody->AddNewPrimitive(convexShapes[i],-1);
        // pLevelPrimitive->SetLocalPosition(g_pBSPCamera->m_Loc.x,g_pBSPCamera->m_Loc.y,g_pBSPCamera->m_Loc.z);
        pLevelPrimitive->SetElasticity(0.2f);
        pLevelPrimitive->SetFriction(12000.0f);
    }




    //g_pVehicule->m_BodyPos=g_pCarBSPFile->m_StartPositions[0];
    g_pVehicule->m_BodyPos.Assign(120,100,100);
    if(g_pCarBSPFile->m_StartPositions.size()>0)
        g_pVehicule->m_BodyPos=g_pCarBSPFile->m_StartPositions[0];
    g_pVehicule->Create(pTPEEngine);

}




void RenderCarDemo(float dt,int APIID,bool updateOnly)
{
    Matrix4 currMatrix;



    ComputeTPEBodyTransform(g_pVehicule->m_pChasisBody,currMatrix);

    

    //g_CarDemoCamera.m_Loc=Vector3(10,60,10);
    g_CarDemoCamera.m_Up=Vector3(0,1,0);
    g_CarDemoCamera.m_At=currMatrix.GetLoc();
    
    
    g_CarDemoCamera.m_Loc=currMatrix.GetLoc()-g_CameraRadius*(g_CarDemoCamera.m_Fwd);
    g_CarDemoCamera.Activate();

    Graphics()->DisableLighting();



    //pTPEEngine->RunUntil(GetTickCount(),0.01f,4);
    pTPEEngine->RunStep(0.02f);
    g_pVehicule->Handle();

    Graphics()->CalculateFrustum();

    g_pCarBSPFile->RenderFrom(g_CarDemoCamera.m_Loc);
    

    

    // render the body
    ComputeTPEBodyTransform(g_pVehicule->m_pChasisBody,currMatrix);
    Graphics()->PushMatrix();
    Graphics()->MultMatrix(currMatrix.d);
    if(KEYUP(VK_SPACE))
    pCarBody->Render();
    Graphics()->PopMatrix();

    // then render the 4 wheels
    for(int i=0;i<4;i++)
    {
        Graphics()->PushMatrix();
        ComputeTPEBodyTransform(g_pVehicule->m_Wheels[i],currMatrix);
        Graphics()->MultMatrix(currMatrix.d);

        // this will orient the wheels correctly
        if(i%2)
            Graphics()->Rotate(90.0f,0,1,0);
        else
            Graphics()->Rotate(-90.0f,0,1,0);
        pWheel->Render();
        Graphics()->PopMatrix();
    }


    if(KEYDOWN(VK_SHIFT))
    {
        g_pVehicule->m_pChasisBody->AddOneFrameForce(0,100000,0);

    }

    if(KEYDOWN('X'))
    {
        g_pVehicule->m_pChasisBody->AddOneFrameTorque(1000000,0,0);
    }
    

    
   // ComputeTPEBodyTransform(g_pCarGround,currMatrix);
    Graphics()->PushMatrix();
    //Graphics()->MultMatrix(currMatrix.d);
  //  pDummyTexture->Activate();
//    g_pCarGroundMesh->Render();
    //pDummyTexture->Deactivate();
    Graphics()->PopMatrix();
    Graphics()->EnableLighting();
}


void PushCarDemo(int APIID)
{
}


void DestroyCarDemo(int APIID)
{
    SAFE_DELETE(g_pVehicule);
    SAFE_DELETE(pTPEEngine);
    SAFE_DELETE(pTestSkyBox);
    SAFE_DELETE(pMd3);
    SAFE_DELETE(pDummyTexture);

    SAFE_DELETE(pCarBody);
    SAFE_DELETE(pWheel);
    SAFE_DELETE(g_pCarGroundMesh);

    SAFE_DELETE(g_pCarBSPFile);
  
    
}



void DisplayCarDemoHelp()
{
    TextPrintf(10,50,"Hold right mouse to pickup Chains");
    TextPrintf(10,30,"Press Space to throw the ball");
    TextPrintf(10,70,"Use mouse wheel to zoom in out");
    TextPrintf(10,90,"Hold right mouse button to change camera");
    TextPrintf(10,110,"Press insert/delete/+/- to change number of Chains");
}

