#include "system/engine.h"
#include "3d/3dtypes/3dtypes.h"
#include "3d/text/text.h"
#include "3d/graphics/graphicssubsystem.h"
#include "util/util.h"
#include "audio/audio.h"
#include "raytracing.h"
#include "boxmesh.h"

// and this is for TPE
#include "nvPhysics.h"

// header implemented here.
#include "Chain.h"
#include "tpe.h"


// the playground dimensaion (the ground)
float       g_ChainDemoGravity=-40.8f;
float       g_ChainDemoBallDensity=0.001f;
float       g_ChainDemoBallFriction=5000.0f;
float       g_ChainDemoBallElasticity=0.2f;
float       g_ChainDemoBallRadius=2.5f;
float       g_ChainDemoForce=10.0f;

Vector3     g_ChainDemoPushingForce(1000.0f,0.0f,0.0f);
Vector3     g_ChainDemoBallStartPos(0.0f,5.0f,0.0f);

Vector3     g_ChainDims(4.0f,2.0f,2.0f);
float       g_ChainDensity=0.01f;

float       g_ChainElasticity=0.1f;
float       g_ChainFriction=500.0f;

sBoxMesh    g_ChainBoxMesh;

sTexture*   g_pChainTexture=NULL;
cCamera     g_ChainDemoCamera;
cSound*     g_pChainDemoMusic=NULL;
int         g_RingDragID=-1;
Vector3     g_RingClickPosition;

nV::Body*     g_pChainDemoTPEBall=NULL;

vector<nV::Body*>         g_TPEChainBodies;
vector<nV::Primitive*>     g_RingPrimitives;
vector<nV::Body*>         g_RingBodies;




void SetChainDemoCamera(cCamera& camera)
{
        g_ChainDemoCamera=camera;
}

struct sChainData
{
    Vector3 Position;
    float Angle;
};


vector<sChainData> g_AllChainsData;


void CreateRing(float ringMass,Vector3& position,Vector3& Axis,float Angle,int numRingChains=10)
{
    Matrix4 worldMatrix;
    worldMatrix.MakeIdent();
    worldMatrix=Matrix4::AxisRotation(Axis,Angle);
    worldMatrix.GetLoc()=position;

    // the angle step is here
    float angleStep=2.0f*PI/numRingChains;
    nV::Body* pRingBody=pTPEEngine->BodyMngr.Create();
    pRingBody->SetPosition(worldMatrix._41,worldMatrix._42,worldMatrix._43);
    worldMatrix.GetLoc().Assign(0.0f,0.0f,0.0f);
    float radius=numRingChains*0.4f;
    float currAngle=0.0f;
    Vector3 Center;
    Vector3 rotAxis(0.0f,0.0f,1.0f);
    rotAxis=rotAxis*worldMatrix;

    // loop on Chains per level..
    for(int j=0;j<numRingChains;j++)
    {
        float currAngle=j*angleStep;
        Center.x=cosf(currAngle)*(radius+(g_ChainDims.z/2.0f));
        Center.y=sinf(currAngle)*(radius+(g_ChainDims.z/2.0f));
        Center.z=0.0f;
        Center=Center*worldMatrix;
        currAngle+=PI/2.0f;
        nV::Primitive* pNewPrim=pRingBody->AddBox(ringMass,g_ChainDims.x,g_ChainDims.y,g_ChainDims.z);
        pNewPrim->SetFriction(90000.0f);
        pNewPrim->SetLocalPosition(Center.x,Center.y,Center.z);
        pNewPrim->RotateAxisAngle(rotAxis.x,rotAxis.y,rotAxis.z,currAngle);
        pNewPrim->RotateAxisAngle(Axis.x,Axis.y,Axis.z,Angle);
        g_RingPrimitives.push_back(pNewPrim);
    }
    g_RingBodies.push_back(pRingBody);
}

void CreateChain(int numRings,int numChainsPerRing,Vector3& Position)
{
    float angle;
    Vector3 Axis(0,1,0);
    for(int i=0;i<numRings;i++)
    {
        if(i%2)
            angle=PI/2.0f;
        else
            angle=0.0f;

        if(i==(numRings-1))
            CreateRing(-1,Vector3(Position.x,i*0.7f*numChainsPerRing+Position.y,Position.z),Axis,angle,numChainsPerRing);
        else
            CreateRing(0.1f,Vector3(Position.x,i*0.7f*numChainsPerRing+Position.y,Position.z),Axis,angle,numChainsPerRing);
    }
}

void CreateMidRings(int numRings,int numChainsPerRing,Vector3& Position)
{
    float angle;
    Vector3 Axis(1,0,0);
    for(int i=0;i<numRings;i++)
    {
        angle=PI/2.0f;
        if(i%2==0)
            CreateRing(0.1f,Vector3(Position.x,i*0.7f*numChainsPerRing+Position.y,Position.z),Axis,angle,numChainsPerRing);
    }
}


void CreateGridChain(int numChains,int numRingsPerChain,int numBricksPerRing,Vector3& position)
{
    int i;

    //int numRingsPerChain=numBricksPerRing;

    Vector3 startPosition=position;
    startPosition.x-=((numChains-1)*1.4f*numBricksPerRing)*0.5f;
    startPosition.y-=((numRingsPerChain-1)*0.7f*numBricksPerRing)*0.5f;

    for(i=0;i<numChains;i++)
    {
        Vector3 currPosition=startPosition;
        currPosition.x+=1.4f*i*numBricksPerRing;
        CreateChain(numRingsPerChain,numBricksPerRing,currPosition);
        currPosition.x+=0.7f*numBricksPerRing;
        if(i<(numChains-1))
            CreateMidRings(numRingsPerChain,numBricksPerRing,currPosition);
    }
}


void SelectRing()
{
    Vector3 cameraLoc;
    Vector3 mouseRay;
    Graphics()->ComputeMouseRay(Mouse()->m_Loc.x,Mouse()->m_Loc.y,cameraLoc.v,mouseRay.v);

    Segment mouseSeg;
    mouseSeg.v0=g_ChainDemoCamera.m_Loc;
    mouseSeg.v1=g_ChainDemoCamera.m_Loc+mouseRay*10.0f;

    if(Mouse()->m_bLButtonDown&&(g_RingDragID==-1))
    {
        float dist=10000.0f;
        for(unsigned int i=0;i<g_RingPrimitives.size();i++)
        {
            nV::Primitive*  pCurrBox=g_RingPrimitives[i];
            Vector3 position;
            pCurrBox->GetWorldPosition(position.x,position.y,position.z);
            if(RayHitSphere(cameraLoc,mouseRay,position,2.0f))
            {
                float newDist=(g_ChainDemoCamera.m_Loc-position).Mag();
                if(newDist<dist)
                {
                    g_RingDragID=i;
                    dist=newDist;
                }
            }
        }
    }


    if(g_RingDragID!=-1)
    {
        nV::Body* draggedBody=g_RingPrimitives[g_RingDragID]->GetBody();
        Vector3 OnPlane;
        draggedBody->GetPositon(OnPlane.x,OnPlane.y,OnPlane.z);
        Plane plane(g_ChainDemoCamera.m_Fwd,OnPlane);
        g_RingClickPosition=plane.Split(mouseSeg.v0,mouseSeg.v1);
    }
}






void CreateChainDemo(int APIID)
{
//    if(inside==false)
    {
        g_pChainTexture       =new sTexture("data/marble002.jpg");
        g_pChainDemoMusic     =new cSound("data/never_on_sunday.mid");
        g_pChainDemoMusic->Play();
        g_ChainBoxMesh.Create(g_ChainDims.x,g_ChainDims.y,g_ChainDims.z,-0.5f,-0.5f,-0.5f);
    }



    // the physics engine itself
    pTPEEngine=new PhysicsWorld();
    SetEngine(pTPEEngine);
    pTPEEngine->SetGravity(0.0f,g_ChainDemoGravity,0.0f);
    pTPEEngine->SetNoOfSolvingIterations(50);
    pTPEEngine->SetObjectsAutoDeactivation(true);


    CreateGridChain(4,4,15,Vector3(0,0,0));

    // now loop on them one by one, create a geom and a body
    for(unsigned int i=0;i<g_AllChainsData.size();i++)
    {
        sChainData& currChain=g_AllChainsData[i];

        Vector3 pos=currChain.Position;
        float angle=currChain.Angle;

        float boxMass=g_ChainDensity*g_ChainDims.x*g_ChainDims.y*g_ChainDims.z;


        nV::Body* pNewBody=pTPEEngine->BodyMngr.Create();
        nV::Primitive* pBox=pNewBody->AddBox(boxMass,g_ChainDims.x,g_ChainDims.y,g_ChainDims.z);
        pNewBody->SetPosition(pos.x,pos.y,pos.z);
        pNewBody->RotateAxisAngle(0,1,0,angle);
        pBox->SetElasticity(g_ChainElasticity);
        pBox->SetFriction(g_ChainFriction);

        g_TPEChainBodies.push_back(pNewBody);
    }


    // create the ball in a far place
    float ballMass=4.0f/3.0f*PI*g_ChainDemoBallRadius*g_ChainDemoBallRadius*g_ChainDemoBallRadius;
    ballMass*=g_ChainDemoBallDensity;


    g_pChainDemoTPEBall=pTPEEngine->BodyMngr.Create();
    g_pChainDemoTPEBall->AddSphere(ballMass,g_ChainDemoBallRadius);
    g_pChainDemoTPEBall->SetPosition(0,0,-5000.0f);
}




void RenderChainDemo(float dt,int APIID,bool updateOnly)
{
    //pTPEEngine->RunUntil(GetTickCount(),0.01f,4);

	pTPEEngine->RunStep(0.01f);

    if(updateOnly)
        return;
    if(Mouse()->m_bLButtonDown)
    {
        SelectRing();
    }
    else
    {
        g_RingDragID=-1;
    }
    Matrix4 currMatrix;

    if(g_RingDragID>-1)
    {
        nV::Body* pDraggedRing=g_RingPrimitives[g_RingDragID]->GetBody();
        Vector3 ChainPosition;
        pDraggedRing->GetPositon(ChainPosition.x,ChainPosition.y,ChainPosition.z);


        sMesh mesh;
        mesh.m_VertexFormat=MESH_VERTEXFORMAT_POSITION;
        mesh.m_RenderMode=MESH_RENDERMODE_LINELIST;
        mesh.AddVertex(ChainPosition.v);
        mesh.AddVertex(g_RingClickPosition.v);

        mesh.Compile();
        mesh.Render();
    }


    unsigned int i=0;

    g_pChainTexture->Activate();

    g_ChainBoxMesh.Enable();


    for(i=0;i<g_RingPrimitives.size();i++)
    {
        nV::Primitive* pCurrPrim=g_RingPrimitives[i];
        ComputeTPEPrimitiveTransform(pCurrPrim,currMatrix);
        Graphics()->PushMatrix();
        Graphics()->MultMatrix(currMatrix.d);
        g_ChainBoxMesh.RenderIndexTrianglesRange(0,0,g_ChainBoxMesh.m_NumIndices);
        Graphics()->PopMatrix();
    }
    g_ChainBoxMesh.Disable();
    g_pChainTexture->Deactivate();






    // render the ball
    ComputeTPEBodyTransform(g_pChainDemoTPEBall,currMatrix);
    Graphics()->PushMatrix();
    Graphics()->MultMatrix(currMatrix.d);
    Graphics()->RenderSphere(g_ChainDemoBallRadius);
    Graphics()->PopMatrix();
}


void PushChainDemo(int APIID)
{
    static bool keyPressed=false;

    if(KEYDOWN(VK_SPACE)&&(keyPressed==false))
    {
        keyPressed=true;
        Vector3& pos=g_ChainDemoCamera.m_Loc;
        pos+=g_ChainDemoCamera.m_Fwd*10.0f;
        g_pChainDemoTPEBall->SetPosition(pos.x,pos.y,pos.z);
        g_pChainDemoTPEBall->SetLinearVelocity(0,0,0);
        g_pChainDemoTPEBall->SetAngularVelocity(0,0,0);
        Vector3 pushForce(g_ChainDemoCamera.m_Fwd);
        pushForce*=2000.0f;
        pushForce*=g_ChainDemoBallDensity*1000.0f;
        g_pChainDemoTPEBall->AddOneFrameForce(pushForce.x,pushForce.y,pushForce.z);
        // create the hitting ball
    }

    if(KEYUP(VK_SPACE)&&KEYUP('R')&&keyPressed&&KEYUP(VK_TAB))
    {
        keyPressed=false;
    }

    if(g_RingDragID>-1)
    {
        nV::Body* pDraggedRing=g_RingPrimitives[g_RingDragID]->GetBody();
        Vector3 Position;
        pDraggedRing->GetPositon(Position.x,Position.y,Position.z);
        Vector3 Force=g_RingClickPosition-Position;
        //   Force.Normalize();
        pDraggedRing->AddOneFrameForce(g_ChainDemoForce*Force.x,g_ChainDemoForce*Force.y,g_ChainDemoForce*Force.z);
    }

}
void DestroyChainDemo(int APIID)
{
    g_AllChainsData.clear();
    g_TPEChainBodies.clear();
    g_RingPrimitives.clear();
    g_RingBodies.clear();
    SAFE_DELETE(pTPEEngine);

//    if(inside==false)
    {
        g_pChainDemoMusic->Stop();
        SAFE_DELETE(g_pChainTexture);
        SAFE_DELETE(g_pChainDemoMusic);
    }
    SetEngine(NULL);
}


void DisplayChainHelp()
{
    TextPrintf(10,50,"Hold right mouse to pull rings");
    TextPrintf(10,30,"Press Space to throw a ball");
    TextPrintf(10,70,"Use mouse wheel to zoom in out");
    TextPrintf(10,90,"Hold right mouse button to change camera");
    TextPrintf(10,130,"%d Rings",g_RingBodies.size());
}

