

#include "system/engine.h"
#include "3d/graphics/graphicssubsystem.h"
#include "3d/3dtypes/3dtypes.h"
#include "3d/meshes/indexmesh.h"
#include "audio/audio.h"

#include "3d/text/text.h"
#include "util/util.h"
#include "audio/audio.h"
#include "raytracing.h"
#include "boxmesh.h"

// and this is for TPE
#include "nvPhysics.h"

#include "tpe.h"
#include "convex.h"


// the playground dimensaion (the ground)
float       g_ConvexDemoGravity=-9.8f;
float       g_ConvexDemoDensity=0.001f;
float       g_ConvexDemoFriction=5000.0f;
float       g_ConvexDemoElasticity=0.2f;


cCamera     g_ConvexDemoCamera;
Vector3 g_ConvexDemoClickPos;
int g_ConvexDemoDragID=-1;

nV::Body* g_pConvexGround=NULL;

vector<sMesh*> g_ConvexDemoMeshes;
vector<nV::Body*> g_ConvexDemoBodies;
sBoxMesh* g_ConvexDemoGroundMesh=NULL;



sTexture* g_pConvexDemoGroundTexture=0;
sTexture* g_pConvexDemoEnvTexture=0;

void SetConvexDemoCamera(cCamera& camera)
{
        g_ConvexDemoCamera=camera;
}




void CreateConvexDemo(int APIID)
{

    g_pConvexDemoGroundTexture=new sTexture("data/ggfol.jpg");
    g_pConvexDemoEnvTexture=new sTexture("data/envmap.jpg");
    // the physics engine itself
    pTPEEngine=new PhysicsWorld();
    SetEngine(pTPEEngine);
    pTPEEngine->SetGravity(0.0f,g_ConvexDemoGravity,0.0f);
    pTPEEngine->SetNoOfSolvingIterations(50);
    pTPEEngine->SetObjectsAutoDeactivation(false);


    g_pConvexGround=pTPEEngine->BodyMngr.Create();
    g_pConvexGround->AddBox(-1.0f,100,10,100);
    g_pConvexGround->SetPosition(0,-10,0);
    //g_pConvexGround->RotateAxisAngle(1,0,0,0.0f);


    g_ConvexDemoMeshes.clear();
    int meshType=0;
    // create some random objects
    for(int i=0;i<20;i++)
    {
        sMesh* pNewMesh=new sMesh();
        nV::Body* pNewBody=pTPEEngine->BodyMngr.Create();
        nV::ConvexShape* pNewConvex=pTPEEngine->ShapeMngr.CreateConvex();

        meshType=rand()%2;
        meshType=1;

        pNewConvex->Begin();
        if(meshType==1)
        {
            pNewMesh->CreateCylinder(3+rand()%10,1+rand()%5,1+rand()%5,3+rand()%15);
            pNewMesh->Compile();
            pNewConvex->AddTriangles(pNewMesh->m_pVertexData,6*sizeof(float),pNewMesh->m_NumVertices);
        }
        else
        {
            pNewMesh->CreateSphere(5,3+rand()%5,3+rand()%5,1,1,1);
            pNewMesh->Compile();
            pNewConvex->AddTriangleStrip(pNewMesh->m_pVertexData,6*sizeof(float),pNewMesh->m_NumVertices);
        }
        pNewConvex->End();

        pNewBody->AddNewPrimitive(pNewConvex,1.0);
        pNewBody->SetPosition(-20+(i/5)*10.0f,20,-20+i%5*10.0f);
        
        //pNewConvex->
        g_ConvexDemoMeshes.push_back(pNewMesh);
        g_ConvexDemoBodies.push_back(pNewBody);
    }

    g_ConvexDemoGroundMesh=new sBoxMesh();
    g_ConvexDemoGroundMesh->Create(100,10,100,3,3,3);
}




void RenderConvexDemoForSelection()
{
    Vector3 cameraLoc;
    Vector3 mouseRay;
    Graphics()->ComputeMouseRay(Mouse()->m_Loc.x,Mouse()->m_Loc.y,cameraLoc.v,mouseRay.v);

    Segment mouseSeg;
    mouseSeg.v0=g_ConvexDemoCamera.m_Loc;
    mouseSeg.v1=g_ConvexDemoCamera.m_Loc+mouseRay*10.0f;

    static Plane selectionPlane(0,1,0,0);

    if(Mouse()->m_bLButtonDown&&(g_ConvexDemoDragID==-1))
    {
        float dist=10000.0f;
        for(unsigned int i=0;i<g_ConvexDemoBodies.size();i++)
        {
            nV::Body*  pCurrDominoPiece=g_ConvexDemoBodies[i];
            Vector3 position;
            pCurrDominoPiece->GetPositon(position.x,position.y,position.z);
            if(RayHitSphere(cameraLoc,mouseRay,position,2.0f))
            {
                float newDist=(g_ConvexDemoCamera.m_Loc-position).Mag();
                if(newDist<dist)
                {
                    g_ConvexDemoDragID=i;
                    dist=newDist;
                }
            }
        }

        if(g_ConvexDemoDragID!=-1)
        {
            nV::Body* draggedBody=g_ConvexDemoBodies[g_ConvexDemoDragID];
            Vector3 OnPlane;
            draggedBody->GetPositon(OnPlane.x,OnPlane.y,OnPlane.z);
            selectionPlane=Plane(g_ConvexDemoCamera.m_Fwd,OnPlane);
        }
    }

    if(g_ConvexDemoDragID!=-1)
    {
        g_ConvexDemoClickPos=selectionPlane.Split(mouseSeg.v0,mouseSeg.v1);
    }
}


void RenderConvexDemo(float dt,int APIID,bool updateOnly)
{
    //pTPEEngine->RunUntil(GetTickCount(),0.01f,4);
	pTPEEngine->RunStep(0.01f);
    Matrix4 currMatrix;

    g_pConvexDemoGroundTexture->Activate();
    ComputeTPEBodyTransform(g_pConvexGround,currMatrix);
    Graphics()->PushMatrix();
    Graphics()->MultMatrix(currMatrix.d);
    g_ConvexDemoGroundMesh->Render();
    Graphics()->PopMatrix();
    g_pConvexDemoGroundTexture->Deactivate();

    unsigned int i;

    //g_pConvexDemoEnvTexture->Activate();
    Graphics()->EnableSphereMap();

    for(i=0;i<g_ConvexDemoMeshes.size();i++)
    {
        sMesh* pCurrMesh=g_ConvexDemoMeshes[i];
        nV::Body* pCurrBody=g_ConvexDemoBodies[i];

        ComputeTPEBodyTransform(pCurrBody,currMatrix);
        Graphics()->PushMatrix();
        Graphics()->MultMatrix(currMatrix.d);
        pCurrMesh->Render();
        Graphics()->PopMatrix();

    }
    Graphics()->DisableSphereMap();
//    g_pConvexDemoEnvTexture->Deactivate();

    if(Mouse()->m_bLButtonDown)
    {
        RenderConvexDemoForSelection();
    }
    else
    {
        g_ConvexDemoDragID=-1;

    }

    if(g_ConvexDemoDragID>-1)
    {
        nV::Body* pDraggedBrick=g_ConvexDemoBodies[g_ConvexDemoDragID];
        Vector3 brickPosition;
        pDraggedBrick->GetPositon(brickPosition.x,brickPosition.y,brickPosition.z);


        sMesh mesh;
        mesh.m_VertexFormat=MESH_VERTEXFORMAT_POSITION;
        mesh.m_RenderMode=MESH_RENDERMODE_LINELIST;
        mesh.AddVertex(brickPosition.v);
        mesh.AddVertex(g_ConvexDemoClickPos.v);

        mesh.Compile();
        mesh.Render();
    }


}


void PushConvexDemo(int APIID)
{
    if(g_ConvexDemoDragID>-1)
    {
        nV::Body* pDraggedBrick=g_ConvexDemoBodies[g_ConvexDemoDragID];
        Vector3 Position;
        pDraggedBrick->GetPositon(Position.x,Position.y,Position.z);
        Vector3 Force=g_ConvexDemoClickPos-Position;
        float Mag=Force.Mag();
        Force.Normalize();
        Force*=Mag*Mag*0.1f;
        pDraggedBrick->AddOneFrameForce(Force.x,Force.y,Force.z);
    }
}


void DestroyConvexDemo(int APIID)
{
    DestroyVector(g_ConvexDemoMeshes);
    g_ConvexDemoMeshes.clear();
    g_ConvexDemoBodies.clear();
    SAFE_DELETE(pTPEEngine);
    SAFE_DELETE(g_ConvexDemoGroundMesh);
    SAFE_DELETE(g_pConvexDemoGroundTexture);
    SAFE_DELETE(g_pConvexDemoEnvTexture);
    SetEngine(NULL);
}



void DisplayConvexDemoHelp()
{
    TextPrintf(10,70,"Hold left mouse to pickup Objects");
    TextPrintf(10,90,"Number of convex shapes %d",g_ConvexDemoMeshes.size());
}

