
#include "3d/graphics/graphicssubsystem.h"
#include "system/engine.h"
#include "3d/3dtypes/3dtypes.h"
#include "3d/shaders/vertexshader.h"
#include "3d/shaders/pixelshader.h"
#include "3d/text/text.h"
#include "util/util.h"
#include "audio/audio.h"
#include "raytracing.h"
#include "boxmesh.h"

// and this is for TPE
#include "nvphysics.h"

// header implemented here.
#include "destruction.h"
#include "tpe.h"

// the playground dimensaion (the ground)
float       g_DestructionDemoGravity=-30.0f;
float       g_DestructionDemoBallDensity=0.01f;
float       g_DestructionDemoBallFriction=5000.0f;
float       g_DestructionDemoBallElasticity=0.2f;
float       g_DestructionDemoBallRadius=2.5f;
float       g_DestructionDemoForce=100.0f;

Vector3     g_DestructionDemoPushingForce(1000.0f,0.0f,0.0f);
Vector3     g_DestructionDemoGroundPos(0.0f,-5.0f,0.0f);
Vector3     g_DestructionDemoGroundDims(1400.0f,10.0f,1400.0f);
Vector3     g_DestructionDemoBallStartPos(0.0f,5.0f,0.0f);

Vector3     g_BrickDims(4.0f,2.0f,2.0f);
float       g_BrickSpacingX=0.2f;
float       g_BrickSpacingY=0.0f;
float       g_BrickDensity=0.01f;


float       g_BrickElasticity=0.1f;
float       g_BrickFriction=1500.0f;
int         g_DestructionDemoSceneID=0;
int         g_DestructionDemoPrevSceneID=0;

Vector3     g_DestructionClickPos(0,0,0);
void        ChangeDestructionScene();
bool        g_DestructionDemoUseFPSCamera=false;
bool        g_DestructionDemoLoaded=false;
sBoxMesh    g_DestructionBoxMesh;
sBoxMesh    g_DestructionGroundMesh;
sTexture*   g_pDestructionTexture=NULL;
sTexture*   g_pDestructionGroundTexture=NULL;
sTexture*   g_pDestructionEnvTexture=NULL;
cCamera     g_DestructionDemoCamera;
cSound*     g_pDestructionDemoMusic=NULL;
int         g_DestructionDemoDragID=-1;

nV::Body*     g_pDestructionDemoTPEGround=NULL;
nV::Body*     g_pDestructionDemoTPEBall=NULL;
nV::Body*     g_pDestructionDemoTPEPlayer=NULL;

vector<nV::Body*>         g_TPEBrickBodies;

cVertexShader* g_pTextVertexShader=NULL;
cPixelShader* g_pTextPixelShader=NULL;


void SetDestructionDemoCamera(cCamera& camera)
{
    if(g_DestructionDemoUseFPSCamera==false)
        g_DestructionDemoCamera=camera;
}

void SetDestructionDemoScene(int id)
{
    g_DestructionDemoSceneID=id;
}


struct sBrickData
{
    Vector3 Position;
    float Angle;
};


vector<sBrickData> g_AllBricksData;


int g_NumBricksRequired=1;

void ArrangePyramid(Vector3& center,float angle,int numLayers)
{
    if(g_AllBricksData.size()==g_NumBricksRequired)
        return;

    int numBricksPerLayer=numLayers;
    float layerWidth=numBricksPerLayer*g_BrickDims.x+(numBricksPerLayer-1)*g_BrickSpacingX;
    float startY=center.y;

    sBrickData newBrick;
    newBrick.Angle=0.0f;
    newBrick.Position.z=center.z;

    Matrix4 rotMatrix;
    rotMatrix.MakeIdent();
    rotMatrix.Rotate(0.0f,angle,0.0f);

    for(int i=0;i<numLayers;i++)
    {
        for(int j=0;j<numBricksPerLayer;j++)
        {
            for(int k=0;k<numBricksPerLayer;k++)
            {
                newBrick.Position.x=-layerWidth/2.0f+j*(g_BrickDims.x+g_BrickSpacingX);
                newBrick.Position.y=i*(g_BrickDims.y+g_BrickSpacingY);
                newBrick.Position.z=-layerWidth/2.0f+k*(g_BrickDims.z+g_BrickSpacingX);
                newBrick.Position.x+=i*(g_BrickDims.x+g_BrickSpacingX)/2.0f;
                newBrick.Position.z+=i*(g_BrickDims.z+g_BrickSpacingX)/2.0f;

                // rotate the whole points
                newBrick.Position=rotMatrix*newBrick.Position;
                newBrick.Angle=angle;
                newBrick.Position.x+=center.x;
                newBrick.Position.y+=center.y;
                newBrick.Position.z+=center.z;
                g_AllBricksData.push_back(newBrick);

                if(g_AllBricksData.size()==g_NumBricksRequired)
                    return;
            }
        }
        numBricksPerLayer--;
    }
}



void ArrangeWall(Vector3& center,float angle,int numLayers,int numBricksPerLayer,bool startBig=true)
{
    if(g_AllBricksData.size()==g_NumBricksRequired)
        return;

    float layerWidth=numBricksPerLayer*g_BrickDims.x+(numBricksPerLayer-1)*g_BrickSpacingX;
    float startY=center.y;

    sBrickData newBrick;
    newBrick.Angle=0.0f;
    newBrick.Position.z=center.z;

    Matrix4 rotMatrix;
    rotMatrix.MakeIdent();
    rotMatrix.Rotate(0.0f,angle,0.0f);

    int checkInt=0;

    for(int i=0;i<numLayers;i++)
    {
        for(int j=0;j<numBricksPerLayer;j++)
        {
            checkInt=i+(startBig? 1:0);
            newBrick.Position.x=-layerWidth/2.0f+j*(g_BrickDims.x+g_BrickSpacingX)+(g_BrickDims.x+g_BrickSpacingX)/2.0f;
            newBrick.Position.y=i*(g_BrickDims.y+g_BrickSpacingY);
            newBrick.Position.z=0.0f;
            if(checkInt%2)
                newBrick.Position.x+=(g_BrickDims.x+g_BrickSpacingX)/2.0f;

            // rotate the whole points
            newBrick.Position=rotMatrix*newBrick.Position;
            newBrick.Angle=angle;
            newBrick.Position+=center;
            if(checkInt%2&&(j==numBricksPerLayer-1))
                continue;
            g_AllBricksData.push_back(newBrick);

            if(g_AllBricksData.size()==g_NumBricksRequired)
                return;

        }
    }
}


void ArrangePyramidWall(Vector3& center,float angle,int numLayers)
{
    if(g_AllBricksData.size()==g_NumBricksRequired)
        return;

    int numBricksPerLayer=numLayers;
    float layerWidth=numBricksPerLayer*g_BrickDims.x+(numBricksPerLayer-1)*g_BrickSpacingX;
    float startY=center.y;

    sBrickData newBrick;
    newBrick.Angle=0.0f;
    newBrick.Position.z=center.z;

    Matrix4 rotMatrix;
    rotMatrix.MakeIdent();
    rotMatrix.Rotate(0.0f,angle,0.0f);

    for(int i=0;i<numLayers;i++)
    {
        for(int j=0;j<numBricksPerLayer;j++)
        {
            newBrick.Position.x=-layerWidth/2.0f+j*(g_BrickDims.x+g_BrickSpacingX);
            newBrick.Position.y=i*(g_BrickDims.y+g_BrickSpacingY);
            newBrick.Position.z=0.0f;
            newBrick.Position.x+=i*(g_BrickDims.x+g_BrickSpacingX)/2.0f;

            // rotate the whole points
            newBrick.Position=rotMatrix*newBrick.Position;
            newBrick.Angle=angle;

            newBrick.Position.x+=center.x;
            newBrick.Position.y+=center.y;
            newBrick.Position.z+=center.z;
            g_AllBricksData.push_back(newBrick);

            if(g_AllBricksData.size()==g_NumBricksRequired)
                return;

        }
        numBricksPerLayer--;
    }
}


void ArrangeCylinderWall(Vector3& center,int numLevels,int NumBricksPerLevel)
{
    if(g_AllBricksData.size()==g_NumBricksRequired)
        return;

    // the angle step is here
    float angleStep=2.0f*PI/NumBricksPerLevel;

    float radius=((g_BrickDims.x+g_BrickSpacingX)/2.0f)/(sinf(angleStep/2.0f));

    sBrickData newBrick;

    // loop on levels
    for(int i=0;i<numLevels;i++)
    {
        newBrick.Angle=0.0f;
        newBrick.Position.Assign(0,0,0);

        // loop on bricks per level..
        for(int j=0;j<NumBricksPerLevel;j++)
        {

            float currAngle=j*angleStep;

            float useAngle=currAngle;

            if(i%2)
                useAngle+=angleStep/2.0f;

            newBrick.Position.x=cosf(useAngle)*(radius+(g_BrickDims.z/2.0f));
            newBrick.Position.y=i*(g_BrickDims.y+g_BrickSpacingY);
            newBrick.Position.z=sinf(useAngle)*(radius+(g_BrickDims.z/2.0f));
            newBrick.Angle=-useAngle+PI/2.0f;
            newBrick.Position+=center;
            g_AllBricksData.push_back(newBrick);

            if(g_AllBricksData.size()==g_NumBricksRequired)
                return;

        }
    }
}






// this will arrange the bricks
void ArrangeBricks()
{
    if(g_DestructionDemoSceneID!=g_DestructionDemoPrevSceneID)
        g_NumBricksRequired=1;

    


    int i=0;
    int j=0;
    switch(g_DestructionDemoSceneID)
    {
    case 1:
        for(i=0;i<5;i++)
            for(j=0;j<5;j++)
            {
                Vector3 Pos(i*40.0f,1,j*35.0f);
                Pos.x-=4*40.0f/2.0f;
                Pos.z-=4*35.0f/2.0f;
                ArrangeWall(Pos,0.0f,5,4);
            }
        break;
    case 2:
        ArrangeWall(Vector3(0,1,32.4f),0.0f,4,16,true);
        ArrangeWall(Vector3(32.4f,1,0),PI/2.0f,4,16,false);
        ArrangeWall(Vector3(-32.4f,1,0),PI/2.0f,4,16,false);
        ArrangeWall(Vector3(0,1,-32.4f),0.0f,4,16,true);
        break;
    case 3:

        for(i=0;i<5;i++)
            for(j=0;j<5;j++)
            {
                Vector3 Pos(i*30.0f,1,j*30.0f);
                Pos.x-=4*30.0f/2.0f;
                Pos.z-=4*30.0f/2.0f;
                ArrangeCylinderWall(Pos,6,6);
            }
        break;
    case 4:
        for(i=0;i<5;i++)
            for(j=0;j<5;j++)
            {
                Vector3 Pos(i*40.0f,1,j*30.0f);
                Pos.x-=4*40.0f/2.0f;
                Pos.z-=4*30.0f/2.0f;
                ArrangePyramid(Pos,0.0f,5);
            }
        break;
    default:
        ArrangeCylinderWall(Vector3(12.5f,1.0f,12.5f),5,4);
        ArrangeCylinderWall(Vector3(-12.5f,1.0f,12.5f),5,4);
        ArrangeCylinderWall(Vector3(12.5f,1.0f,-12.5f),5,4);
        ArrangeCylinderWall(Vector3(-12.5f,1.0f,-12.5f),5,4);
        ArrangeWall(Vector3(0,1,12.5f),0.0f,5,4);
        ArrangeWall(Vector3(0,1,-12.5f),0.0f,5,4);
        ArrangeWall(Vector3(12.5f,1,0.0f),PI/2.0f,5,4);
        ArrangeWall(Vector3(-12.5f,1,0.0f),PI/2.0f,5,4);
        break;
    }
    g_DestructionDemoPrevSceneID=g_DestructionDemoSceneID;
}

int g_DestructionDemoOldNumBodies=0;


void CreateNewBodies()
{
    g_DestructionDemoOldNumBodies=g_AllBricksData.size();
    g_AllBricksData.clear();

    // make the bricks
    ArrangeBricks();

    if(g_DestructionDemoOldNumBodies==(g_AllBricksData.size()))
    {
        g_NumBricksRequired=g_AllBricksData.size();
        g_DestructionDemoLoaded=true;
    }


    //how to destroy?
    if(g_DestructionDemoOldNumBodies>g_AllBricksData.size())
    {
        // should delete the bodies here somehow..
        int numDeletes=g_DestructionDemoOldNumBodies-g_AllBricksData.size();

        for(int i=0;i<numDeletes;i++)
        {
            nV::Body* pBodyToDelete=g_TPEBrickBodies.back();
            pTPEEngine->BodyMngr.Destroy(pBodyToDelete);
            g_TPEBrickBodies.pop_back();

        }
    }
    else
    {
        // THIS TO CREATE
        // now loop on them one by one, create a geom and a body
        for(unsigned int i=g_DestructionDemoOldNumBodies;i<g_AllBricksData.size();i++)
        {
            sBrickData& currBrick=g_AllBricksData[i];

            Vector3 pos=currBrick.Position;
            float angle=currBrick.Angle;

            float boxMass=g_BrickDensity*g_BrickDims.x*g_BrickDims.y*g_BrickDims.z;
            boxMass=30.0f;


            nV::Body* pNewBody=pTPEEngine->BodyMngr.Create();
            nV::Primitive* pBox=pNewBody->AddBox(boxMass,g_BrickDims.x,g_BrickDims.y,g_BrickDims.z);
            pNewBody->SetPosition(pos.x,pos.y,pos.z);
            //pNewBody->SetPosition(0,g_TPEBrickBodies.size()*2,0);
            pNewBody->RotateAxisAngle(0,1,0,angle);
            pBox->SetElasticity(g_BrickElasticity);
            pBox->SetFriction(g_BrickFriction);

            g_TPEBrickBodies.push_back(pNewBody);
        }
    }
}


// create the pivot scene here
void CreateDestructionDemoTPE(bool inside)
{
    g_DestructionDemoLoaded=false;
    if(inside==false)
    {
        g_pDestructionTexture       =new sTexture("data/stone04a.jpg");
        g_pDestructionGroundTexture =new sTexture("data/ggfol.jpg");
        g_pDestructionEnvTexture    =new sTexture("data/envmap.jpg");
        g_pDestructionDemoMusic     =new cSound("data/never_on_sunday.mid");
        g_pDestructionDemoMusic->Play();
        g_DestructionBoxMesh.Create(g_BrickDims.x,g_BrickDims.y,g_BrickDims.z,-0.5f,-0.5f,-0.5f);
        g_DestructionGroundMesh.Create(g_DestructionDemoGroundDims.x,g_DestructionDemoGroundDims.y,g_DestructionDemoGroundDims.z,60,60,60);
    }

    // the physics engine itself
    pTPEEngine=new PhysicsWorld();
    SetEngine(pTPEEngine);
    pTPEEngine->SetGravity(0.0f,g_DestructionDemoGravity,0.0f);
    pTPEEngine->SetNoOfSolvingIterations(10);
    pTPEEngine->SetObjectsAutoDeactivation(true);

    // create the ground...
    g_pDestructionDemoTPEGround=pTPEEngine->BodyMngr.Create();
    g_pDestructionDemoTPEGround->SetPosition(0,-g_DestructionDemoGroundDims.y/2.0f,0);

    nV::Primitive *pGroundPlane=g_pDestructionDemoTPEGround->AddBox(-1.0f,1000,10,1000);//Plane(0,0,0,normal.x,normal.y,normal.z);
    pGroundPlane->SetElasticity(g_BrickElasticity);
    pGroundPlane->SetFriction(g_BrickFriction);

    CreateNewBodies();

    // create the ball in a far place
    float ballMass=4.0f/3.0f*PI*g_DestructionDemoBallRadius*g_DestructionDemoBallRadius*g_DestructionDemoBallRadius;
    ballMass*=g_DestructionDemoBallDensity;


    g_pDestructionDemoTPEBall=pTPEEngine->BodyMngr.Create();
    g_pDestructionDemoTPEBall->AddSphere(ballMass,g_DestructionDemoBallRadius);
    g_pDestructionDemoTPEBall->SetPosition(0,0,-5000.0f);

    g_pDestructionDemoTPEPlayer=pTPEEngine->BodyMngr.Create();
    g_pDestructionDemoTPEPlayer->AddSphere(1.0f,10.0f);
    g_DestructionDemoCamera.m_Loc.Assign(100,100,100);
    g_pDestructionDemoTPEPlayer->SetPosition(g_DestructionDemoCamera.m_Loc.x,g_DestructionDemoCamera.m_Loc.y,g_DestructionDemoCamera.m_Loc.z);
}





void RenderDestructionForSelection()
{
    Vector3 cameraLoc;
    Vector3 mouseRay;
    Graphics()->ComputeMouseRay(Mouse()->m_Loc.x,Mouse()->m_Loc.y,cameraLoc.v,mouseRay.v);

    Segment mouseSeg;
    mouseSeg.v0=g_DestructionDemoCamera.m_Loc;
    mouseSeg.v1=g_DestructionDemoCamera.m_Loc+mouseRay*10.0f;

    static Plane selectionPlane(0,1,0,0);

    if(Mouse()->m_bLButtonDown&&(g_DestructionDemoDragID==-1))
    {
        float dist=10000.0f;
        for(unsigned int i=0;i<g_TPEBrickBodies.size();i++)
        {
            nV::Body*  pCurrDominoPiece=g_TPEBrickBodies[i];
            Vector3 position;
            pCurrDominoPiece->GetPositon(position.x,position.y,position.z);
            if(RayHitSphere(cameraLoc,mouseRay,position,2.0f))
            {
                float newDist=(g_DestructionDemoCamera.m_Loc-position).Mag();
                if(newDist<dist)
                {
                    g_DestructionDemoDragID=i;
                    dist=newDist;
                }
            }
        }

        if(g_DestructionDemoDragID!=-1)
        {
            nV::Body* draggedBody=g_TPEBrickBodies[g_DestructionDemoDragID];
            Vector3 OnPlane;
            draggedBody->GetPositon(OnPlane.x,OnPlane.y,OnPlane.z);
            selectionPlane=Plane(g_DestructionDemoCamera.m_Fwd,OnPlane);
        }

    }

    if(g_DestructionDemoDragID!=-1)
    {
        g_DestructionClickPos=selectionPlane.Split(mouseSeg.v0,mouseSeg.v1);
    }
     g_DestructionDemoDragID=-1;
}

// update the current shader
void UpdateShader()
{
    if(g_pTextVertexShader)
    {
        Matrix4 projMatrix;
        Matrix4 worldMatrix;
        Matrix4 viewMatrix;
        Matrix4 textureMatrix;
        textureMatrix.MakeIdent();

        Graphics()->GetWorldMatrix(worldMatrix.d);
        Graphics()->GetViewMatrix(viewMatrix.d);
        Graphics()->GetProjectionMatrix(projMatrix.d);
        Graphics()->GetTextureMatrix(textureMatrix.d);

        //GetProjectionMatrix()
        g_pTextVertexShader->SetMatrix("ProjMatrix",projMatrix.d);
        g_pTextVertexShader->SetMatrix("ViewMatrix",viewMatrix.d);
        g_pTextVertexShader->SetMatrix("WorldMatrix",worldMatrix.d);
        g_pTextVertexShader->SetMatrix("TextureMatrix",textureMatrix.d);
    }
}




// render the pivot scene here
void RenderDestructionDemoTPE(float dt,bool updateOnly)
{
    //pTPEEngine->RunUntil(GetTickCount(),0.01f,4);
    //pTPEEngine->RunStep(1.0f/30.0f);
	pTPEEngine->RunStep(0.01f);

    if(updateOnly)
        return;



    if(g_DestructionDemoUseFPSCamera)
    {
        Vector3 cameraLoc;
        g_pDestructionDemoTPEPlayer->GetPositon(cameraLoc.x,cameraLoc.y,cameraLoc.z);
        g_DestructionDemoCamera.m_Loc=cameraLoc;
        g_DestructionDemoCamera.RotateLocal(-Mouse()->m_DeltaX/100.0f,-Mouse()->m_DeltaY/100.0f,0);
        Vector3 fwdVector=g_DestructionDemoCamera.m_Fwd;
        fwdVector.Normalize();
        fwdVector*=100.0f;
        Vector3 sideVector=g_DestructionDemoCamera.m_Side;
        sideVector.Normalize();
        sideVector*=100.0f;
        if(KEYDOWN(VK_LEFT)||KEYDOWN('A'))
        {
            g_pDestructionDemoTPEPlayer->AddOneFrameForce(sideVector.x,sideVector.y,sideVector.z);
        }
        if(KEYDOWN(VK_RIGHT)||KEYDOWN('D'))
        {
            g_pDestructionDemoTPEPlayer->AddOneFrameForce(-sideVector.x,-sideVector.y,-sideVector.z);
        }
        if(KEYDOWN(VK_UP)||KEYDOWN('W'))
        {
            g_pDestructionDemoTPEPlayer->AddOneFrameForce(fwdVector.x,fwdVector.y,fwdVector.z);
        }
        else
            if(KEYDOWN(VK_DOWN)||KEYDOWN('S'))
            {
                g_pDestructionDemoTPEPlayer->AddOneFrameForce(-fwdVector.x,-fwdVector.y,-fwdVector.z);
            }
            else
            {
                Vector3 vel;
                g_pDestructionDemoTPEPlayer->GetLinearVelocity(vel.x,vel.y,vel.z);
                vel*=-5.0f;
                g_pDestructionDemoTPEPlayer->AddOneFrameForce(vel.x,vel.y,vel.z);
            }
            g_DestructionDemoCamera.ComputeInfo();
            g_DestructionDemoCamera.Activate();
    }
    if(Mouse()->m_bLButtonDown)
    {
        RenderDestructionForSelection();
    }
    else
    {
        g_DestructionDemoDragID=-1;
    }
    Matrix4 currMatrix;

    if(g_DestructionDemoDragID>-1)
    {
        nV::Body* pDraggedBrick=g_TPEBrickBodies[g_DestructionDemoDragID];
        Vector3 brickPosition;
        pDraggedBrick->GetPositon(brickPosition.x,brickPosition.y,brickPosition.z);


        sMesh mesh;
        mesh.m_VertexFormat=MESH_VERTEXFORMAT_POSITION;
        mesh.m_RenderMode=MESH_RENDERMODE_LINELIST;
        mesh.AddVertex(brickPosition.v);
        mesh.AddVertex(g_DestructionClickPos.v);

        mesh.Compile();
        mesh.Render();
    }

    unsigned int i=0;

    // render the box
    g_pDestructionGroundTexture->Activate();
    ComputeTPEBodyTransform(g_pDestructionDemoTPEGround,currMatrix);
    Graphics()->PushMatrix();
    Graphics()->MultMatrix(currMatrix.d);
    g_DestructionGroundMesh.Render();
    Graphics()->PopMatrix();


    g_pDestructionTexture->Activate();

    g_DestructionBoxMesh.Enable();


    g_pTextVertexShader->Activate();
    for(i=0;i<g_TPEBrickBodies.size();i++)
    {
        nV::Body* pBody=g_TPEBrickBodies[i];
        ComputeTPEBodyTransform(g_TPEBrickBodies[i],currMatrix);
        Graphics()->PushMatrix();
        Graphics()->MultMatrix(currMatrix.d);

        UpdateShader();

 //       if(pBody->IsSleeping()==false)
   //         Graphics()->DisableTexturing();
        g_DestructionBoxMesh.RenderIndexTrianglesRange(0,0,g_DestructionBoxMesh.m_NumIndices);
     //   if(pBody->IsSleeping()==false)
       //     Graphics()->EnableTexturing();

        Graphics()->PopMatrix();
    }
    g_pTextVertexShader->Deactivate();
    g_DestructionBoxMesh.Disable();
    g_pDestructionTexture->Deactivate();

    // render the ball
    ComputeTPEBodyTransform(g_pDestructionDemoTPEBall,currMatrix);
    Graphics()->PushMatrix();
    Graphics()->MultMatrix(currMatrix.d);
    Graphics()->RenderSphere(g_DestructionDemoBallRadius);
    Graphics()->PopMatrix();
}
// push the pivot here
void PushDestructionDemoTPE()
{
    static bool keyPressed=false;

    if(KEYDOWN(VK_SPACE)&&(keyPressed==false))
    {
        keyPressed=true;
        Vector3& pos=g_DestructionDemoCamera.m_Loc;
        pos+=g_DestructionDemoCamera.m_Fwd*10.0f;
        g_pDestructionDemoTPEBall->SetPosition(pos.x,pos.y,pos.z);
        g_pDestructionDemoTPEBall->SetLinearVelocity(0,0,0);
        g_pDestructionDemoTPEBall->SetAngularVelocity(0,0,0);
        Vector3 pushForce(g_DestructionDemoCamera.m_Fwd);
        pushForce*=20000.0f;
      //  pushForce*=g_DestructionDemoBallDensity*1000.0f;
        g_pDestructionDemoTPEBall->AddOneFrameForce(pushForce.x,pushForce.y,pushForce.z);
        // create the hitting ball
    }

    if(KEYUP(VK_SPACE)&&KEYUP('R')&&keyPressed&&KEYUP(VK_TAB))
    {
        keyPressed=false;
    }


    if(KEYDOWN(VK_TAB)&&!keyPressed)
    {
        g_DestructionDemoUseFPSCamera=!g_DestructionDemoUseFPSCamera;
        keyPressed=true;
    }


    if(g_DestructionDemoDragID>-1)
    {
        nV::Body* pDraggedBrick=g_TPEBrickBodies[g_DestructionDemoDragID];
        Vector3 Position;
        pDraggedBrick->GetPositon(Position.x,Position.y,Position.z);
        Vector3 Force=g_DestructionClickPos-Position;
        float Mag=Force.Mag();
        Force.Normalize();
        Force*=Mag*Mag*0.5f;
        pDraggedBrick->AddOneFrameForce(Force.x,Force.y,Force.z);
    }
    ChangeDestructionScene();
}


// destroy the pivot here
void DestroyDestructionDemoTPE(bool inside)
{
    g_AllBricksData.clear();
    g_TPEBrickBodies.clear();
    SAFE_DELETE(pTPEEngine);

    if(inside==false)
    {
        g_pDestructionDemoMusic->Stop();
        SAFE_DELETE(g_pDestructionGroundTexture);
        SAFE_DELETE(g_pDestructionTexture);
        SAFE_DELETE(g_pDestructionEnvTexture);
        SAFE_DELETE(g_pDestructionDemoMusic);
    }
    SetEngine(NULL);

}



void ChangeDestructionScene()
{
    static DWORD lastTime=timeGetTime();

    DWORD currentTime=timeGetTime();
    DWORD timeDiff=currentTime-lastTime;

    if(KEYDOWN(VK_INSERT)&& timeDiff>20)
    {
        lastTime=timeGetTime();
           g_NumBricksRequired++;
    }

    if(KEYDOWN(VK_DELETE)&&g_NumBricksRequired>1&& timeDiff>20)
    {
        lastTime=timeGetTime();
        g_NumBricksRequired--;
    }


    if(g_DestructionDemoLoaded==false)
    {
        g_NumBricksRequired++;
        CreateNewBodies();
    }

    if(KEYDOWN(VK_INSERT)||KEYDOWN(VK_DELETE)||KEYDOWN(VK_ADD)||KEYDOWN(VK_SUBTRACT))
    {
        CreateNewBodies();
    }
}


void CreateDestructionDemo(int APIID)
{
    g_pTextVertexShader= new cVertexShader("shaders/transform.vs");
    g_pTextPixelShader=new cPixelShader("shaders/pixel.ps");

  //  g_pTextPixelShader->Activate();
    

    CreateDestructionDemoTPE(false);
}


void RenderDestructionDemo(float dt,int APIID,bool updateOnly)
{
   RenderDestructionDemoTPE(dt,updateOnly);
}
void PushDestructionDemo(int APIID)
{   
        PushDestructionDemoTPE();
}
void DestroyDestructionDemo(int APIID)
{
        DestroyDestructionDemoTPE(false);
    g_pTextPixelShader->Deactivate();
    SAFE_DELETE(g_pTextVertexShader);
    SAFE_DELETE(g_pTextPixelShader);
    g_DestructionDemoOldNumBodies=0;
    g_AllBricksData.clear();
    g_NumBricksRequired=1;
}

void DisplayDestructionHelp()
{
    TextPrintf(10,50,"Hold right mouse to pickup bricks");
    TextPrintf(10,30,"Press Space to throw the ball");
    TextPrintf(10,70,"Use mouse wheel to zoom in out");
    TextPrintf(10,90,"Hold right mouse button to change camera");
    TextPrintf(10,110,"Press insert/delete/+/- to change number of bricks");
    TextPrintf(10,130,"%d Bricks",g_AllBricksData.size());
    TextPrintf(10,150,"Press TAB to toggle First Person");
}