/****************************************************
*	domino.cpp                                      *
*	source file for the domino struct               *
****************************************************/

#include "domino.h"	//	header implemented here
#include "3d/graphics/graphicssubsystem.h"

/****************************************************
*   struct sDomino implementation                   *
****************************************************/

struct sDominoVertex
{
    Vector3 position;
    Vector3 normal;
    Vector2 texCoord;
};



sDomino::sDomino()
{
 
}


sDomino::~sDomino()
{
    DestroyVector(m_Faces);
}

// create one face of the dive, (make its vertex buffer
void sDomino::Createface(Matrix4& matrix)
{
    sDominoVertex v[4];
 
    // 4 vertices
    v[0].position.Assign(1.0f,1.0f,0.0f);
    v[1].position.Assign(1.0f,-1.0f,0.0f);
    v[2].position.Assign(-1.0f,-1.0f,0.0f);
    v[3].position.Assign(-1.0f,1.0f,0.0f);
    

    v[0].normal.Assign(0,0,1);
    v[1].normal.Assign(0,0,1);
    v[2].normal.Assign(0,0,1);
    v[3].normal.Assign(0,0,1);

    float edge=0.05f;
    v[0].texCoord.Assign(edge,1-edge);
    v[1].texCoord.Assign(edge,edge);
    v[2].texCoord.Assign(1-edge,edge);
    v[3].texCoord.Assign(1-edge,1-edge);
    

    Matrix4 RotMatrix=matrix;
    RotMatrix._41=0.0f;
    RotMatrix._42=0.0f;
    RotMatrix._43=0.0f;

    // create a new indexed mesh
    sIndexedMesh* pNewIndexedMesh=new sIndexedMesh();

    pNewIndexedMesh->m_VertexFormat          =   MESH_VERTEXFORMAT_POSITION|MESH_VERTEXFORMAT_TEXCOORD0|MESH_VERTEXFORMAT_NORMAL;
    pNewIndexedMesh->m_RenderMode            =   MESH_RENDERMODE_TRIANGLESTRIP;


    // now transform them all
    for(int i=0;i<4;i++)
    {
        v[i].position=v[i].position*matrix;
        v[i].normal=v[i].normal*RotMatrix;
        v[i].normal.Normalize();
        pNewIndexedMesh->AddVertex(v[i].position.v);

    }

    pNewIndexedMesh->AddIndex(1);
    pNewIndexedMesh->AddIndex(2);
    pNewIndexedMesh->AddIndex(0);
    pNewIndexedMesh->AddIndex(3);


    pNewIndexedMesh->Compile();

    m_Faces.push_back(pNewIndexedMesh);
}

void sDomino::Create(float pieceWidth)
{

    m_Textures[0].Create("dice/zero.bmp");
    m_Textures[1].Create("dice/one.bmp");
    m_Textures[2].Create("dice/two.bmp");
    m_Textures[3].Create("dice/three.bmp");
    m_Textures[4].Create("dice/four.bmp");
    m_Textures[5].Create("dice/five.bmp");
    m_Textures[6].Create("dice/six.bmp");

    float pieceHeight=pieceWidth*2.0f;
    float pieceThickness=pieceWidth/4.0f;


    Matrix4 matrix;

    // create the back face first
    matrix.MakeIdent();
    matrix.GetLoc().z-=pieceThickness/2.0f;
    matrix.Scale(pieceWidth/2.0f,pieceHeight/2.0f,1.0f);
    Createface(matrix);

    matrix.MakeIdent();
    matrix.Rotate(0.0f,PI/2.0f,0.0f);
    matrix.GetLoc().x-=pieceWidth/2.0f;
    matrix.Scale(pieceThickness/2.0f,pieceHeight/2.0f,1.0f);
    Createface(matrix);

    matrix.MakeIdent();
    matrix.Rotate(0.0f,-PI/2.0f,0.0f);
    matrix.GetLoc().x+=pieceWidth/2.0f;
    matrix.Scale(pieceThickness/2.0f,pieceHeight/2.0f,1.0f);
    Createface(matrix);

    matrix.MakeIdent();
    matrix.Rotate(PI/2.0f,0.0f,0.0f);
    matrix.GetLoc().y+=pieceHeight/2.0f;
    matrix.Scale(pieceWidth/2.0f,pieceThickness/2.0f,1.0f);
    Createface(matrix);

    matrix.MakeIdent();
    matrix.Rotate(-PI/2.0f,0.0f,0.0f);
    matrix.GetLoc().y-=pieceHeight/2.0f;
    matrix.Scale(pieceWidth/2.0f,pieceThickness/2.0f,1.0f);
    Createface(matrix);

    matrix.MakeIdent();
    matrix.Rotate(0.0f,PI,0.0f);
    matrix.GetLoc().z+=pieceThickness/2.0f;
    matrix.GetLoc().y+=pieceWidth/2.0f;
    matrix.Scale(pieceWidth/2.0f,pieceWidth/2.0f,1.0f);
    Createface(matrix);


    matrix.MakeIdent();
    matrix.Rotate(0.0f,PI,0.0f);
    matrix.GetLoc().z+=pieceThickness/2.0f;
    matrix.GetLoc().y-=pieceWidth/2.0f;
    matrix.Scale(pieceWidth/2.0f,pieceWidth/2.0f,1.0f);
    Createface(matrix);

}

void sDomino::Render(unsigned int up,unsigned int down)
{
    m_Textures[0].Activate();
    for(int face=0;face<5;face++)
    {
        m_Faces[face]->Render();
    }
    m_Textures[up%7].Activate();
    m_Faces[5]->Render();
    m_Textures[down%7].Activate();
    m_Faces[6]->Render();
}
/*--------end of struct sDomino implementaiotn-------*/

