#include "system/engine.h"

#include "3d/Graphics/graphicssubsystem.h"
#include "3d/3dtypes/3dtypes.h"
#include "input/input.h"
#include "raytracing.h"

// this is for ODE
//#include "../physics/physics.h"
#include "audio/audio.h"
#include "3d/text/text.h"
// and this is for TPE
#include "nvphysics.h"

// header implemented here
#include "dominoes.h"
#include "tpe.h"
#include "domino.h"
#include "boxmesh.h"

#define DOMINO_PIECE_WIDTH 1.5f

sDomino* g_pDominoMesh=NULL;
int     g_DominoDemoSceneID=0;

// the playground dimension (the ground)
Vector3 g_DominoesGroundDims(150.0f,10.0f,150.0f);
float g_DominoesDemoGravity=-9.8f;
float g_DominoesDensity=0.1f;
Color4 g_DominoBallColor(1,1,1,1);

sTexture* g_pDominoGroundTexture=NULL;
sTexture* g_pDominoEnvTexture=NULL;
sMesh * g_DominoBallMesh=NULL;


Vector3 g_DominoPieceDims(DOMINO_PIECE_WIDTH,DOMINO_PIECE_WIDTH*2.0f,DOMINO_PIECE_WIDTH/4.0f);
float g_DominoPushingForce=70.0f;
float g_DominoPickingForce=3.0f;
Vector3 g_DominoDemoCenter(0.0f,0.0f,0.0f);

sBoxMesh g_pDominoGroundBoxMesh;


Vector3 g_DominoDemoBallPosition(0.0,15.0f,0.0);
float g_DominoDemoBallRadius=1.0f;
float g_DominoDemoBallDensity=0.01f;

int g_DominoPickID=-1;


cSound* pDominoSound=NULL;
cMusic* pDominoMusic=NULL;


Vector3 g_DominoClickPosition(0,0,0);


cCamera g_DominoesCamera;

void SetDominoCamera(cCamera& camera)
{
    g_DominoesCamera=camera;
}


void SetDominoScene(int sceneID)
{
    g_DominoDemoSceneID=sceneID;
}



struct sDominoPiece
{
    Vector3 Position;
    float Angle;
};


vector<sDominoPiece> g_DominoPiecesData;


int     g_DominoesDemoNumPieces=50;
float   g_DominoesDemoRadius=15.0f;
float   g_DominoesDemoStartAngle=0.0f;
float   g_DominoesDemoEndAngle=300.0f;


void FindPathNormal(vector<Vector2>& Path,unsigned int index,bool loopPath,Vector2& result)
{
    if(index<0||index>=Path.size())
        return;
    Vector3 currPoint=Path[index];
    Vector3 normal(0,0,0);
    if(index>0)
        normal+=Path[index-1]-currPoint;
    if(index<(Path.size()-1))
        normal+=currPoint-Path[index+1];
    result=normal;
    result.Normalize();


}


// compute the points between which we will create the bride planks
// the star
void SetDominoesPositions1()
{
    g_DominoDemoBallPosition.Assign(0.0f,20.0f,0.0f);

    int numPiecesPerRing=10;
    int numRings=10;
    float ringSpacing=1.2f;

    float angleStep=PI*2.0f/numPiecesPerRing;

    sDominoPiece newPiece;
    newPiece.Position.y=g_DominoesGroundDims.y/2.0f+g_DominoPieceDims.y/2.0f;

    for(int i=0;i<numRings;i++)
    {
        float currRadius=i*ringSpacing+2.5f;
        //for(float currAngle=0.0f;currAngle<PI*2.0f;currAngle+=angleStep)
        for(int j=0;j<numPiecesPerRing;j++)
        {
            float currAngle=j*angleStep;
            newPiece.Position.x=cosf(currAngle)*currRadius;
            newPiece.Position.z=sinf(currAngle)*currRadius;
            newPiece.Angle=currAngle+PI/2.0f;
            g_DominoPiecesData.push_back(newPiece);
        }
    }
}


// compute the points between which we will create the bride planks
void SetDominoesPositions2()
{

    sDominoPiece newPiece;
    int numLines=30;
    float lineSpacing=1.1f;
    float rowSpacing=1.6f;
    float offsetx=numLines*lineSpacing/2.0f;

    g_DominoDemoBallPosition.x=newPiece.Position.x=-offsetx-2.0f;
    g_DominoDemoBallPosition.y=10.0f;
    g_DominoDemoBallPosition.z=0.0f;

    for(int line=0;line<numLines;line++)
    {
        newPiece.Position.x=line*lineSpacing-offsetx;
        int useLine=line;

        if(useLine>10)
            useLine=10;

        float currLineOffset=((useLine-1)*rowSpacing)/2.0f;
        for(int i=0;i<useLine;i++)
        {
            newPiece.Position.z=(i*rowSpacing)-currLineOffset;
            newPiece.Angle=PI/2.0f;
            newPiece.Position.y=g_DominoesGroundDims.y/2.0f+g_DominoPieceDims.y/2.0f;
            g_DominoPiecesData.push_back(newPiece);
        }
    }
}

// compute the points between which we will create the bride planks
void SetDominoesPositions3()
{
    sDominoPiece newPiece;
    cPolygon2D poly;
    poly.AddCurve(Vector2(0,0),Vector2(20,0),Vector2(20,20),24);
    poly.AddCurve(Vector2(20,22),Vector2(20,40),Vector2(40,40),24);
    poly.AddCurve(Vector2(42,40),Vector2(60,40),Vector2(60,20),24);
    poly.AddCurve(Vector2(60,18),Vector2(60,0),Vector2(20,0),40);

    g_DominoDemoBallPosition.x=-5.0f;
    g_DominoDemoBallPosition.y=10.0f;
    g_DominoDemoBallPosition.z=0.0f;




    // loop on the curve points
    for(unsigned int i=0;i<poly.m_Points.size();i++)
    {
        Vector2 Norm(1,0);
        FindPathNormal(poly.m_Points,i,false,Norm);

        Vector2& currPoint=poly.m_Points[i];
        newPiece.Angle=atanf(Norm.y/Norm.x)+PI/2.0f;
        newPiece.Position=g_DominoDemoCenter;

        newPiece.Position.x+=currPoint.x;
        newPiece.Position.y+=g_DominoesGroundDims.y/2.0f+g_DominoPieceDims.y/2.0f;
        newPiece.Position.z+=currPoint.y;

        g_DominoPiecesData.push_back(newPiece);
    }
}



// compute the points between which we will create the bride planks
void SetDominoesPositions4()
{
    sDominoPiece newPiece;
    int numLines=30;
    float lineSpacing=1.1f;
    float rowSpacing=1.6f;
    float offsetx=numLines*lineSpacing/2.0f;

    g_DominoDemoBallPosition.x=newPiece.Position.x=-offsetx-2.0f;
    g_DominoDemoBallPosition.y=10.0f;
    g_DominoDemoBallPosition.z=0.0f;

    for(int line=0;line<numLines;line++)
    {
        newPiece.Position.x=line*lineSpacing-offsetx;
        int useLine=line;

        //        if(useLine>10)
        //          useLine=10;

        float currLineOffset=((useLine-1)*rowSpacing)/2.0f;
        for(int i=0;i<useLine;i++)
        {
            newPiece.Position.z=(i*rowSpacing)-currLineOffset;
            newPiece.Angle=PI/2.0f;
            newPiece.Position.y=g_DominoesGroundDims.y/2.0f+g_DominoPieceDims.y/2.0f;
            g_DominoPiecesData.push_back(newPiece);
        }
    }
}

// compute the points between which we will create the bride planks
void SetDominoesPositions5()
{
    sDominoPiece newPiece;
    int numLines=30;
    float lineSpacing=1.1f;
    float rowSpacing=1.1f;
    float offsetx=numLines*lineSpacing/2.0f;

    g_DominoDemoBallPosition.x=newPiece.Position.x=-offsetx-2.0f;
    g_DominoDemoBallPosition.y=10.0f;
    g_DominoDemoBallPosition.z=0.0f;

    for(int line=0;line<numLines;line++)
    {
        newPiece.Position.x=line*lineSpacing-offsetx;
        int useLine=line;

        if(useLine>10)
            useLine=10;

        float currLineOffset=((useLine-1)*rowSpacing)/2.0f;
        for(int i=0;i<useLine;i++)
        {
            newPiece.Position.z=(i*rowSpacing)-currLineOffset;
            newPiece.Angle=-PI/2.0f;
            newPiece.Position.y=g_DominoesGroundDims.y/2.0f+g_DominoPieceDims.y/2.0f;
            g_DominoPiecesData.push_back(newPiece);
        }
    }
}


// setting the scene
void SetDominoesPositions()
{
    switch(g_DominoDemoSceneID)
    {
    case 1:
        SetDominoesPositions1();
        break;
    case 2:
        SetDominoesPositions2();
        break;
    case 3:
        SetDominoesPositions3();
        break;
    case 4:
        SetDominoesPositions4();
        break;
    default:
        SetDominoesPositions1();
        break;
    }

    g_DominoBallColor.r=getRandFloat(0,1);
    g_DominoBallColor.g=getRandFloat(0,1);
    g_DominoBallColor.b=getRandFloat(0,1);
}

bool dominoCallBack(nV::Body* pBody1,nV::Body* pBody2)
{
    if(pDominoSound)
    {
        if(pBody2&&pBody1)
        {
            float x1,y1,z1;
            float x2,y2,z2;
            pBody1->GetLinearVelocity(x1,y1,z1);
            pBody2->GetLinearVelocity(x2,y2,z2);
            if(x1-x2>11.0f)
                pDominoSound->Play();
        }
    }
    return true;
}

// the TPE implementation
nV::Body* g_pDominoesTPEGround=NULL;
nV::Body* g_pDominoesTPEBall=NULL;
vector<nV::Body*> g_TPEDominoesBodies;

// create the pivot scene here
void CreateDominoesTPE()
{
    // the physics engine itself
    pTPEEngine=new PhysicsWorld();
    SetEngine(pTPEEngine);
    pTPEEngine->SetObjectsAutoDeactivation(false);
    pTPEEngine->SetGravity(0.0f,g_DominoesDemoGravity,0.0f);
    pTPEEngine->SetNoOfSolvingIterations(15);


    g_pDominoMesh=new sDomino();
    g_pDominoMesh->Create(DOMINO_PIECE_WIDTH);
    SetDominoesPositions();

    // create the ground
    g_pDominoesTPEGround=pTPEEngine->BodyMngr.Create();
    nV::Primitive* pBox=g_pDominoesTPEGround->AddBox(-1,g_DominoesGroundDims.x,g_DominoesGroundDims.y,g_DominoesGroundDims.z);
    g_pDominoesTPEGround->SetPosition(0,0,0);

    pBox->SetFriction(100000.0f);

    g_pDominoGroundBoxMesh.Create(g_DominoesGroundDims.x,g_DominoesGroundDims.y,g_DominoesGroundDims.z,g_DominoesGroundDims.x/20.0f,g_DominoesGroundDims.y/20.0f,g_DominoesGroundDims.z/20.0f);

    float dominoPieceMass=g_DominoesDensity*g_DominoPieceDims.x*g_DominoPieceDims.y*g_DominoPieceDims.z;


    // create the domino pieces
    for(unsigned int i=0;i<g_DominoPiecesData.size();i++)
    {
        Vector3& piecePosition=g_DominoPiecesData[i].Position;
        float Angle=g_DominoPiecesData[i].Angle;

        nV::Body* pNewDominoBody=pTPEEngine->BodyMngr.Create();
        nV::Primitive* pBox=pNewDominoBody->AddBox(dominoPieceMass,g_DominoPieceDims.x,g_DominoPieceDims.y,g_DominoPieceDims.z);
        pNewDominoBody->SetPosition(piecePosition.x,piecePosition.y,piecePosition.z);
        pNewDominoBody->RotateAxisAngle(0,1,0,-Angle);

        pBox->SetElasticity(0.8f);
        pBox->SetFriction(1.0f);
        g_TPEDominoesBodies.push_back(pNewDominoBody);
    }

    // create the ball
    float ballMass=4.0f/3.0f*PI*g_DominoDemoBallRadius*g_DominoDemoBallRadius*g_DominoDemoBallRadius;
    g_pDominoesTPEBall=pTPEEngine->BodyMngr.Create();
    nV::Primitive* pSphere=g_pDominoesTPEBall->AddSphere(dominoPieceMass,g_DominoDemoBallRadius);
    //nV::CPrimitive* pSphere=g_pDominoesTPEBall->AddCylinder(dominoPieceMass,g_DominoDemoBallRadius,g_DominoDemoBallRadius);
    pSphere->SetFriction(100000.0f);

    g_DominoBallMesh=new sMesh();
    g_DominoBallMesh->CreateSphere(g_DominoDemoBallRadius);
    //g_DominoBallMesh->CreateCylinder(g_DominoDemoBallRadius,g_DominoDemoBallRadius,g_DominoDemoBallRadius,30);

    g_DominoBallMesh->Compile();

    g_pDominoesTPEBall->SetPosition(g_DominoDemoBallPosition.x,g_DominoDemoBallPosition.y,g_DominoDemoBallPosition.z);

}

// render the pivot scene here
void RenderDominoesForSelection()
{
    Vector3 cameraLoc;
    Vector3 mouseRay;
    Graphics()->ComputeMouseRay(Mouse()->m_Loc.x,Mouse()->m_Loc.y,cameraLoc.v,mouseRay.v);

    Segment mouseSeg;
    mouseSeg.v0=g_DominoesCamera.m_Loc;
    mouseSeg.v1=g_DominoesCamera.m_Loc+mouseRay*10.0f;


    static Plane selectionPlane;
    if(Mouse()->m_bLButtonDown&&(g_DominoPickID==-1))
    {
        float dist=10000.0f;
        for(unsigned int i=0;i<g_DominoPiecesData.size();i++)
        {
            nV::Body*  pCurrDominoPiece=g_TPEDominoesBodies[i];
            Vector3 position;
            pCurrDominoPiece->GetPositon(position.x,position.y,position.z);
            Matrix4 currTransform;
            ComputeTPEBodyTransform(pCurrDominoPiece,currTransform);

            //if(RayHitSphere(cameraLoc,mouseRay,position,1.0f))
            if(RayHitBox(cameraLoc,mouseRay,currTransform,DOMINO_PIECE_WIDTH,DOMINO_PIECE_WIDTH*2.0f,DOMINO_PIECE_WIDTH/4.0f))
            {
                float newDist=(g_DominoesCamera.m_Loc-position).Mag();
                if(newDist<dist)
                {
                    g_DominoPickID=i;
                    dist=newDist;
                }
            }
        }

        if(g_DominoPickID!=-1)
        {
            nV::Body* draggedBody=g_TPEDominoesBodies[g_DominoPickID];
            Vector3 OnPlane;
            draggedBody->GetPositon(OnPlane.x,OnPlane.y,OnPlane.z);
            selectionPlane=Plane(g_DominoesCamera.m_Fwd,OnPlane);
        }

    }


    if(g_DominoPickID!=-1)
    {
        g_DominoClickPosition=selectionPlane.Split(mouseSeg.v0,mouseSeg.v1);
    }
}



// render the pivot scene here
void RenderDominoesTPE(float dt,bool updateOnly)
{
    pTPEEngine->RunStep(dt);

    if(updateOnly)
        return;
    if(Mouse()->m_bLButtonDown)
        RenderDominoesForSelection();
    else
        g_DominoPickID=-1;

    Matrix4 currMatrix;
    // pTPEEngine->RunUntil(GetTickCount(),0.02f,10);    
    


    Material dummyMaterial;//=Material::AQUA_GLAZE_METAL;
    dummyMaterial.SetID(MATERIAL_ID_AQUA_GLAZE_METAL);
    dummyMaterial.SetDiffuse(g_DominoBallColor);

    dummyMaterial.SetEmissive(Color4(0.2f,0.2f,0.2f,0.2f));


    ComputeTPEBodyTransform(g_pDominoesTPEGround,currMatrix);


    g_pDominoGroundTexture->Activate();

    Graphics()->PushMatrix();
    Graphics()->MultMatrix(currMatrix.d);
    g_pDominoGroundBoxMesh.Render();
    Graphics()->PopMatrix();
    g_pDominoGroundTexture->Deactivate();

    srand(100);

    for(unsigned int i=0;i<g_DominoPiecesData.size();i++)
    {
        nV::Body*  pCurrDominoPiece=g_TPEDominoesBodies[i];
        ComputeTPEBodyTransform(pCurrDominoPiece,currMatrix);
        Graphics()->PushMatrix();
        Graphics()->MultMatrix(currMatrix.d);
        g_pDominoMesh->Render(rand()%7,rand()%7);
        Graphics()->PopMatrix();
    }

    Graphics()->EnableSphereMap();
    g_pDominoEnvTexture->Activate();

    dummyMaterial.Activate();

    ComputeTPEBodyTransform(g_pDominoesTPEBall,currMatrix);
    Graphics()->PushMatrix();
    Graphics()->MultMatrix(currMatrix.d);
    //   Graphics()->RenderSphere(g_DominoDemoBallRadius);
    g_DominoBallMesh->Render();
    Graphics()->PopMatrix();
    Graphics()->DisableSphereMap();

    g_pDominoEnvTexture->Deactivate();

    g_DominoDemoBallPosition=currMatrix.GetLoc();

    if(g_DominoPickID>-1)
    {

        nV::Body* pDraggedBrick=g_TPEDominoesBodies[g_DominoPickID];
        Vector3 brickPosition;
        pDraggedBrick->GetPositon(brickPosition.x,brickPosition.y,brickPosition.z);


        sMesh mesh;
        mesh.m_VertexFormat=MESH_VERTEXFORMAT_POSITION;
        mesh.m_RenderMode=MESH_RENDERMODE_LINELIST;
        mesh.AddVertex(brickPosition.v);
        mesh.AddVertex(g_DominoClickPosition.v);

        mesh.Compile();
        mesh.Render();
    }

}


// push the pivot here
void PushDominoesTPE()
{
    if(g_TPEDominoesBodies.size())
    {
        nV::Body* pDominoPiece=g_TPEDominoesBodies.front();

        Vector3 fwdForce=g_DominoesCamera.m_Fwd;//
        fwdForce.y=0.0f;
        fwdForce.Normalize();
        fwdForce*=g_DominoPushingForce*0.0537f;

        // commented by sa6ry: 8.3.2006
        //if(KEYDOWN(VK_UP)||KEYDOWN('W'))
        if(KEYDOWN(VK_UP))
            g_pDominoesTPEBall->AddOneFrameForce(fwdForce.x,0,fwdForce.z);
        //if(KEYDOWN(VK_DOWN)||KEYDOWN('S'))
        if(KEYDOWN(VK_DOWN))
            g_pDominoesTPEBall->AddOneFrameForce(-fwdForce.x,0,-fwdForce.z);

        Vector3 sideForce=-fwdForce^Vector3::j;
        //if(KEYDOWN(VK_LEFT)||KEYDOWN('A'))
        if(KEYDOWN(VK_LEFT))
            g_pDominoesTPEBall->AddOneFrameForce(-sideForce.x,0,-sideForce.z);
        //if(KEYDOWN(VK_RIGHT)||KEYDOWN('D'))
        if(KEYDOWN(VK_RIGHT))
            g_pDominoesTPEBall->AddOneFrameForce(sideForce.x,0,sideForce.z);



        static bool keyPressed=false;
        if(KEYDOWN(VK_SPACE)&&keyPressed==false)
        {
            g_pDominoesTPEBall->AddOneFrameForce(0,100.0f,0);
            keyPressed=true;
        }


        if(KEYUP(VK_SPACE))
            keyPressed=false;


        if(KEYDOWN('F'))
        {
            g_pDominoesTPEBall->SetAngularVelocity(0,0,0);
            g_pDominoesTPEBall->SetLinearVelocity(0,0,0);

        }


        if(g_DominoPickID>-1)
        {
            nV::Body* pDraggedBrick=g_TPEDominoesBodies[g_DominoPickID];
            Vector3 Position;
            pDraggedBrick->GetPositon(Position.x,Position.y,Position.z);
            Vector3 Force=g_DominoClickPosition-Position;

            float Mag=Force.Mag();
            Force.Normalize();
            Force*=Mag*Mag*0.1f;
            pDraggedBrick->AddOneFrameForce(Force.x,Force.y,Force.z);

            //   Force.Normalize();
//            pDraggedBrick->AddOneFrameForce(g_DominoPickingForce*Force.x,g_DominoPickingForce*Force.y,g_DominoPickingForce*Force.z);
        }


    }
}


// destroy the pivot here
void DestroyDominoesTPE()
{
    SAFE_DELETE(pTPEEngine);
    SAFE_DELETE(g_pDominoMesh);
    g_TPEDominoesBodies.clear();
    g_DominoPiecesData.clear();
    SAFE_DELETE(pDominoSound);
    SetEngine(NULL);
}

void CreateDominoes(int APIID)
{
    pDominoMusic=new cMusic("data/mozart_50.mid");
    pDominoMusic->Play();

    g_pDominoGroundTexture=new sTexture("data/parquet.tga");
    g_pDominoEnvTexture       =new sTexture("data/envmap.jpg");


    if(APIID==0)
        CreateDominoesTPE();
    //    else
    //      CreateDominoesODE();
}


void RenderDominoes(float dt,int APIID,bool updateOnly)
{
    if(APIID==0)
        RenderDominoesTPE(dt,updateOnly);
    //    else
    //      RenderDominoesODE(dt);

}
void PushDominoes(int APIID)
{
    if(APIID==0)
        PushDominoesTPE();
    //    else
    //      PushDominoesODE();
}
void DestroyDominoes(int APIID)
{
    if(APIID==0)
        DestroyDominoesTPE();
    //    else
    //      DestroyDominoesODE();

    pDominoMusic->Stop();
    SAFE_DELETE(pDominoMusic);
    SAFE_DELETE(g_pDominoGroundTexture);
    SAFE_DELETE(g_pDominoEnvTexture);
    SAFE_DELETE(g_DominoBallMesh);


}


Vector3& GetDominoDemoBallPosition()
{
    return g_DominoDemoBallPosition;
}


void DisplayDominoesHelp()
{
    // s6ary: text has been changed 8.3.2006
    TextPrintf(10,30,"Press Space to make the ball jump");
    TextPrintf(10,50,"Use arrow keys to move the ball");
    TextPrintf(10,70,"Use mouse wheel to zoom in out");
    TextPrintf(10,90,"Hold right mouse button to change camera");
    TextPrintf(10,110,"Press W,A,S,D to move the camera");
    TextPrintf(10,150,"%d Domino pieces",g_TPEDominoesBodies.size());
    TextPrintf(10,130,"Press F to stop the ball");
}



