///************************************************************
//*   pendulum.cpp                                            *
//* source file for the pendulum scene                        *
//************************************************************/
//
//#include "../system/engine.h"
//#include "../3d/3dtypes/3dtypes.h"
//#include "../3d/text/text.h"
//#include "../input/input.h"
//#include "../3d/selection/selectionengine.h"
//#include "../audio/audio.h"
//#include "../physics/physics.h"
//#include "phengine.h"
//#include "tpe.h"
//#include "../audio/audio.h"
//#include "pendulum.h"
//#include "raytracing.h"
//
///************************************************************
//*   global data for the scene                               *
//************************************************************/
//
//cSound* g_pBallsCollisionSound      =   NULL    ;
//unsigned int g_NumPendulumBalls     =   2       ;
//float g_PendulumBallDensity         =   0.4f    ;
//float g_PendulumBallRadius          =   3.0f    ;
//float g_PendulumWirelength          =   30.0f   ;
//Vector3 RodDimensions(1.0f,1.0f,1.0f);
//Vector3 g_PendulumWorldClickPos;
//cCamera g_PendulumCamera;
//
//
//
//void SetPendulumCamera(cCamera& camera)
//{
//    g_PendulumCamera=camera;
//}
//
//float g_BallsElasticity         =   0.99f   ;
//float g_BallsFriction           =   0.0f    ;
//float g_PendulumGravity         =   -90.8f  ;
//float g_PendulumPushingForce    =   1000.0f ;
//int   g_PendulumBallIndex       =   -1      ;
//
//cMusic* g_PendulumMusic=NULL;
//
//vector<Vector3> g_PendulumWires;
//sTexture* g_PendulumEnvTexture=0;
//
//
//// ODE global data items..
//vector<cRigidBody*> PendulumBodies;
//vector<cGeom*>      PendulumGeoms;
//vector<cJoint*>     PendulumHinges;
//cStaticBody*        pRodBody  =   NULL;
//cGeomBox*           pRodGeom  =   NULL;
//
//
//
//// create the pendulum, using ODE here
//void CreatePendulumODE()
//{
//    
//    Physics()->SetWorldGravity(0,g_PendulumGravity,0);
//
//    PendulumBodies.clear();
//    PendulumGeoms.clear();
//    PendulumHinges.clear();
//
//    float ballDiameter=g_PendulumBallRadius*2.0f;
//    float totalWidth=ballDiameter*g_NumPendulumBalls;
//
//    // create the fixed rod
//    for(unsigned int i=0;i<g_NumPendulumBalls;i++)
//    {
//        // create a geom..
//        cGeomSphere* pNewSphere=new cGeomSphere(g_PendulumBallDensity,g_PendulumBallRadius);
//
//        // create a body
//        cRigidBody* pNewBall=new cRigidBody();
//
//        pNewBall->AddGeometry(*pNewSphere);
//        pNewBall->LockGeometries();
//        pNewBall->m_Elasticity=g_BallsElasticity;
//        pNewBall->m_Friction=g_BallsFriction;
//
//        PendulumGeoms.push_back(pNewSphere);
//        PendulumBodies.push_back(pNewBall);
//        pNewBall->SetPosition(i*ballDiameter-((totalWidth-ballDiameter)/2.0f),0,0);
//
//        cHinge* pNewHinge=new cHinge();
//        pNewHinge->AttachBodies(PendulumBodies[i],NULL);
//
//        pNewHinge->SetAnchor(i*ballDiameter-((totalWidth-ballDiameter)/2.0f),g_PendulumWirelength,0);
//        pNewHinge->SetAxis(0.0f,0.0f,1.0f);
//        PendulumHinges.push_back(pNewHinge);
//    }
//
//
//
//
//    RodDimensions.x=totalWidth;
//
//    // create the rod
//    pRodGeom=new cGeomBox(1,totalWidth,RodDimensions.y,RodDimensions.z);
//    pRodGeom->SetPosition(0,g_PendulumWirelength,0);
//    pRodBody=new cStaticBody();
//    pRodBody->AddGeometry(*pRodGeom);
//}
//
//
//void RenderPendulumODE(float DeltaTime)
//{
//    Physics()->Update(DeltaTime);
//    pRodBody->RenderGeometries();
//    for(unsigned int i=0;i<PendulumBodies.size();i++)
//    {
//        cRigidBody* pCurrentBall=PendulumBodies[i];
//        pCurrentBall->RenderGeometries();
//    }
//}
//
//void DestroyPendulumODE()
//{
//    DestroyVector(PendulumGeoms);
//    DestroyVector(PendulumBodies);
//    SAFE_DELETE(pRodGeom);
//    SAFE_DELETE(pRodBody);
//}
//
//
//
//
//void PushPendulumODE()
//{
//    
//    if(PendulumBodies.size()<1)
//        return;
//
//    cRigidBody* pCurrentBall=PendulumBodies.back();
//
//
//    if(KEYDOWN(VK_SPACE))
//        pCurrentBall->AddForce(g_PendulumPushingForce,0,0);
//
//
//    static KeyPressed=false;
//
//    if(KEYUP(VK_ADD)&&KEYUP(VK_SUBTRACT))
//        KeyPressed=false;
//
//    if(KEYDOWN(VK_ADD)&&KeyPressed==false)
//    {
//        g_NumPendulumBalls++;
//        KeyPressed=true;
//        DestroyPendulumODE();
//        CreatePendulumODE();
//
//    }
//    if(KEYDOWN(VK_SUBTRACT)&&KeyPressed==false)
//    {
//        g_NumPendulumBalls--;
//        if(g_NumPendulumBalls<1)
//            g_NumPendulumBalls=1;
//        KeyPressed=true;
//
//        DestroyPendulumODE();
//        CreatePendulumODE();
//    }
//
//
//
//
//
//}
//
//
//
//// TPE implementation
//
//vector <CVBody*> TPEBodies;
//CVBody*         p_TPERod;
//
//
//bool collisionCallBack(CVBody* pBody1,CVBody* pBody2)
//{
//    if(g_pBallsCollisionSound)
//    {
//        if(pBody2&&pBody1)
//        {
//            float x1,y1,z1;
//            float x2,y2,z2;
//            pBody1->GetLinearVelocity(x1,y1,z1);
//            pBody2->GetLinearVelocity(x2,y2,z2);
//            if(x1-x2>10.0f)
//            g_pBallsCollisionSound->Play();
//        }
//    }
//    return true;
//}
//
//// create the pendulum, using ODE here
//void CreatePendulumTPE()
//{
//
//    // the physics engine itself
//    pTPEEngine=new CPhEngine();
//    pTPEEngine->SetGravity(0.0f,g_PendulumGravity,0.0f);
////    pTPEEngine->SetCollisionNotify(collisionCallBack);
//
//    // clear the bodies initially
//    TPEBodies.clear();
//
//    float ballMass=4.0f/3.0f*PI*g_PendulumBallDensity*g_PendulumBallRadius*g_PendulumBallRadius*g_PendulumBallRadius;
//    float ballDiameter=g_PendulumBallRadius*2.0f;
//    float totalWidth=ballDiameter*g_NumPendulumBalls;
//
//    // create the rod
//    RodDimensions.x=totalWidth+2.0f*g_PendulumWirelength;
//    p_TPERod=pTPEEngine->VBodies.Create();
//    p_TPERod->CreateBox(-1.0f,RodDimensions.x,RodDimensions.y,RodDimensions.z);
//    p_TPERod->SetPosition(0,g_PendulumWirelength,0);
//
//    // create the balls and hinges
//    for(unsigned int i=0;i<g_NumPendulumBalls;i++)
//    {
//        Vector3 ballPos(i*ballDiameter-((totalWidth-ballDiameter)/2.0f),0,0);
//        Vector3 hingePos(i*ballDiameter-((totalWidth-ballDiameter)/2.0f),g_PendulumWirelength,0);
//        CVBody* pNewBody=pTPEEngine->VBodies.Create();
//        //CPSphere * pSphere=pNewBody->CreateSphere(ballMass,g_PendulumBallRadius);
//        pNewBody->SetPosition(ballPos.x,ballPos.y,ballPos.z);
//        // this is how gravity is made until now
//        pSphere->SetElasticity(g_BallsElasticity);
//        pSphere->SetFriction(g_BallsFriction);
//        TPEBodies.push_back(pNewBody);
//
//        // create the hinges
//		pTPEEngine->Joints.CreateHingeJoint(p_TPERod,pNewBody,hingePos.x,hingePos.y,hingePos.z,0,0,1);
//        g_PendulumWires.push_back(ballPos);
//        g_PendulumWires.push_back(hingePos);
//    }
//
//    // create the solo texture here
//    g_PendulumEnvTexture=new sTexture("data/envmap.jpg");
//}
//
//
//
//
//void RenderPendulumForSelection()
//{
//    glRenderMode(GL_SELECT);
//    glMatrixMode(GL_PROJECTION);
//    int viewPort[4];
//    glGetIntegerv(GL_VIEWPORT,viewPort);
//    glPushMatrix();
//    glLoadIdentity();
//    gluPickMatrix(Mouse()->m_Loc.x,Mouse()->m_Loc.y,2,2,viewPort);
//    Graphics()->SetPerspectiveProjection(60.0f,4.0f/3.0f,1.0f,1000.0f);
//    glMatrixMode(GL_MODELVIEW);
//
//    Matrix4 CurrMatrix;
//    for(unsigned int i=0;i<TPEBodies.size();i++)
//    {
//        CVBody* pCurrentBall=TPEBodies[i];
//        ComputeTPEBodyTransform(pCurrentBall,CurrMatrix);
//        Graphics()->PushMatrix();
//        Graphics()->MultMatrix(CurrMatrix.d);
//        glPushName(i);
//        Graphics()->RenderSphere(g_PendulumBallRadius);
//        glPopName();
//        Graphics()->PopMatrix();
//    }
//
//
//    glMatrixMode(GL_PROJECTION);
//    glPopMatrix();
//    SelectionEngine()->m_NumHits=glRenderMode(GL_RENDER);
//    SelectionEngine()->ReadSelections();
//    glMatrixMode(GL_MODELVIEW);
//
//    Plane plane(0,0,-1,0);
//    Segment mouseSeg;
//    mouseSeg.v0=g_PendulumCamera.m_Loc;
//    Vector3 cameraLoc;
//    Vector3 mouseRay(0,0,0);//=RayFromMousePos(Mouse()->m_Loc.x,Mouse()->m_Loc.y,cameraLoc);
//
//    mouseSeg.v1=g_PendulumCamera.m_Loc+mouseRay*10.0f;
//    g_PendulumWorldClickPos=plane.Split(mouseSeg.v0,mouseSeg.v1);
//
//    if(Mouse()->m_bLButtonDown)
//    {
//        if(SelectionEngine()->m_Selections.size())
//        {
//            CSelectionEntry* pEntry=SelectionEngine()->m_Selections.front();
//            g_PendulumBallIndex=pEntry->m_SelectedObjects[0];
//        }
//    }
//    else
//    {
//        g_PendulumBallIndex=-1;
//    }
//}
//
//
//void RenderPendulumTPE(float DeltaTime)
//{
//    RenderPendulumForSelection();
//
//    glEnable(GL_TEXTURE_2D);
//    glTexGenf(GL_S,GL_TEXTURE_GEN_MODE,GL_SPHERE_MAP);
//    glTexGenf(GL_T,GL_TEXTURE_GEN_MODE,GL_SPHERE_MAP);
//    glEnable(GL_TEXTURE_GEN_S);
//    glEnable(GL_TEXTURE_GEN_T);
//    glTexEnvi(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_MODULATE);
//
//    g_PendulumEnvTexture->Activate();
//
//    pTPEEngine->RunStep(DeltaTime);
//
//    Matrix4 CurrMatrix;
//    for(unsigned int i=0;i<TPEBodies.size();i++)
//    {
//         CVBody* pCurrentBall=TPEBodies[i];
//         ComputeTPEBodyTransform(pCurrentBall,CurrMatrix);
//         Graphics()->PushMatrix();
//         Graphics()->MultMatrix(CurrMatrix.d);
//         Graphics()->RenderSphere(g_PendulumBallRadius);
//         Graphics()->PopMatrix();
//
//         g_PendulumWires[i*2]=Vector3(0,0,0)*CurrMatrix;
//    }
//
//    // render the wires
//    ComputeTPEBodyTransform(p_TPERod,CurrMatrix);
//    Graphics()->PushMatrix();
//    Graphics()->MultMatrix(CurrMatrix.d);
//    Graphics()->RenderBox(RodDimensions.x,RodDimensions.y,RodDimensions.z);
//    Graphics()->PopMatrix();
//
//    glBegin(GL_LINES);
//
//    for(i=0;i<g_PendulumWires.size();i++)
//    {
//        glVertex3fv(g_PendulumWires[i].v);
//    }
//    glEnd();
//
//    if(g_PendulumBallIndex>-1)
//    {
//        CVBody* pDraggedBall=TPEBodies[g_PendulumBallIndex];
//        Vector3 BallPosition;
//        pDraggedBall->GetPositon(BallPosition.x,BallPosition.y,BallPosition.z);
//        glBegin(GL_LINES);
//        glVertex3fv(BallPosition.v);
//        glVertex3fv(g_PendulumWorldClickPos.v);
//        glEnd();
//    }
//
//
//    glDisable(GL_TEXTURE_GEN_S);
//    glDisable(GL_TEXTURE_GEN_T);
//
//    glBindTexture(GL_TEXTURE_2D,0);
//}
//
//void DestroyPendulumTPE()
//{
//    // DestroyVector(TPEBodies);
//    SAFE_DELETE(pTPEEngine);
//    SAFE_DELETE(g_pBallsCollisionSound);
//    g_PendulumWires.clear();
//    SAFE_DELETE(g_PendulumEnvTexture);
//}
//
//
//
//void PushPendulumTPE()
//{
//    static bool KeyPressed=false;
//
//    if(KEYUP(VK_ADD)&&KEYUP(VK_SUBTRACT)&&(KeyPressed==true))
//        KeyPressed=false;
//
//    if(KEYDOWN(VK_ADD)&&KeyPressed==false)
//    {
//        g_NumPendulumBalls++;
//        KeyPressed=true;
//        DestroyPendulumTPE();
//        CreatePendulumTPE();
//    }
//    if(KEYDOWN(VK_SUBTRACT)&&KeyPressed==false)
//    {
//        g_NumPendulumBalls--;
//        if(g_NumPendulumBalls<1)
//            g_NumPendulumBalls=1;
//        KeyPressed=true;
//        DestroyPendulumTPE();
//        CreatePendulumTPE();
//    }
//
//    if(g_PendulumBallIndex>-1)
//    {
//        CVBody* pDraggedBall=TPEBodies[g_PendulumBallIndex];
//        Vector3 BallPosition;
//        pDraggedBall->GetPositon(BallPosition.x,BallPosition.y,BallPosition.z);
//        Vector3 Force=g_PendulumWorldClickPos-BallPosition;
//        pDraggedBall->AddOneFrameForce(g_PendulumPushingForce*Force.x,g_PendulumPushingForce*Force.y,g_PendulumPushingForce*Force.z);
//    }
//}
//
//
//
//
//
//void CreatePendulum(int APIID)
//{
//    g_PendulumMusic=new cMusic("data/sheela.mid");
//    g_PendulumMusic->Play();
//    if(APIID==1)
//        CreatePendulumTPE();
////    else
//  //      CreatePendulumODE();
//}
//
//
//void RenderPendulum(float dt,int APIID,bool updateOnly)
//{
//    if(APIID==1)
//        RenderPendulumTPE(dt);
////    else
//  //      RenderPendulumODE(dt);
//
//}
//void PushPendulum(int APIID)
//{
//    if(APIID==1)
//        PushPendulumTPE();
////    else
//  //      PushPendulumODE();
//}
//void DestroyPendulum(int APIID)
//{
//    if(APIID==1)
//        DestroyPendulumTPE();
////    else
//  //      DestroyPendulumODE();
//
//    g_PendulumMusic->Stop();
//    SAFE_DELETE(g_PendulumMusic);
//
//    
//}
//void DisplayPendulumHelp()
//{
//
//}
