//#include "nxphysics.h"

#include "system/engine.h"
#include "3d/3dtypes/3dtypes.h"
#include "3d/Graphics/graphicssubsystem.h"
#include "audio/audio.h"
#include "input/input.h"
#include "raytracing.h"
#include "boxmesh.h"
#include "3d/text/text.h"

// this is for ODE
//#include "../physics/physics.h"

// and this is for TPE
#include "nvphysics.h"

// header implemented here
#include "tpe.h"
#include "bridge.h"

cCamera g_StressCamera;

cMusic* g_pStressMusic=NULL;
sTexture* g_pStressBoxTexture=NULL;
sTexture* g_pStressWoodTexture=NULL;
sTexture* g_pStressEnvTexture=NULL;

int g_StressPickID=-1;
vector<sMesh*> g_StressMeshes;

void SetStressCamera(cCamera& camera)
{
    g_StressCamera=camera;
}



float g_StressDemoGravity=-90.8f;
unsigned int g_StressNumMaxDroppedBodies=30;
DWORD g_StressGenTime=100;

Vector3 g_StressTestBoxDims(175.0f,10.0f,175.0f);
Vector3 g_StressTestMinBoxDims(8.5f,8.5f,8.5f);
Vector3 g_StressTestMaxBoxDims(10.0f,10.0f,10.0f);
Vector3 g_StressClickPosition(0,0,0);

float g_StreeDemoMinFriction=10000.0f;
float g_StreeDemoMaxFriction=10000.0f;
float g_StressTestBoxThickness=10.0f;



int g_StressTestCurrTime=0;




struct sGeomData
{
    int type;
    float radius;
    Vector3 dimensions;
    Vector3 position;
    Color4  color;
};



vector<sGeomData> g_TPEBoxesData;
vector<sGeomData> g_TPEDroppedBodiesGeomData;
vector<nV::Body*> g_TPEBoxBodies;
vector<nV::Body*> g_TPEDroppedBodies;


vector<sBoxMesh*> g_StressBoxMeshes;

void GenerateBoxes()
{
    sGeomData boxData;

    boxData.position.Assign(0.0f,-g_StressTestBoxThickness/2.0f,0.0f);
    boxData.dimensions.Assign(g_StressTestBoxDims.x+g_StressTestBoxThickness*2.0f,g_StressTestBoxThickness,g_StressTestBoxDims.z+g_StressTestBoxThickness*2.0f);
    g_TPEBoxesData.push_back(boxData);

    boxData.position.Assign(-g_StressTestBoxDims.x/2.0f-g_StressTestBoxThickness/2.0f,g_StressTestBoxDims.y/2.0f,0.0f);
    boxData.dimensions.Assign(g_StressTestBoxThickness,g_StressTestBoxDims.y,g_StressTestBoxDims.z+g_StressTestBoxThickness*2.0f);
    g_TPEBoxesData.push_back(boxData);

    boxData.position.Assign(g_StressTestBoxDims.x/2.0f+g_StressTestBoxThickness/2.0f,g_StressTestBoxDims.y/2.0f,0.0f);
    boxData.dimensions.Assign(g_StressTestBoxThickness,g_StressTestBoxDims.y,g_StressTestBoxDims.z+g_StressTestBoxThickness*2.0f);
    g_TPEBoxesData.push_back(boxData);

    boxData.position.Assign(0.0f,g_StressTestBoxDims.y/2.0f,g_StressTestBoxDims.z/2.0f+g_StressTestBoxThickness/2.0f);
    boxData.dimensions.Assign(g_StressTestBoxDims.x,g_StressTestBoxDims.y,g_StressTestBoxThickness);
    g_TPEBoxesData.push_back(boxData);

    boxData.position.Assign(0.0f,g_StressTestBoxDims.y/2.0f,-g_StressTestBoxDims.z/2.0f-g_StressTestBoxThickness/2.0f);
    boxData.dimensions.Assign(g_StressTestBoxDims.x,g_StressTestBoxDims.y,g_StressTestBoxThickness);
    g_TPEBoxesData.push_back(boxData);


    ////*
    boxData.position.Assign(-g_StressTestBoxDims.x-g_StressTestBoxThickness/2.0f,g_StressTestBoxDims.y/2.0f,0.0f);
    boxData.dimensions.Assign(g_StressTestBoxThickness,g_StressTestBoxDims.y,(g_StressTestBoxDims.z+g_StressTestBoxThickness*2.0f)*2.0f);
    g_TPEBoxesData.push_back(boxData);

    boxData.position.Assign(g_StressTestBoxDims.x+g_StressTestBoxThickness/2.0f,g_StressTestBoxDims.y/2.0f,0.0f);
    boxData.dimensions.Assign(g_StressTestBoxThickness,g_StressTestBoxDims.y,(g_StressTestBoxDims.z+g_StressTestBoxThickness*2.0f)*2.0f);
    g_TPEBoxesData.push_back(boxData);

    boxData.position.Assign(0.0f,g_StressTestBoxDims.y/2.0f,g_StressTestBoxDims.z+g_StressTestBoxThickness*3.0f/2.0f);
    boxData.dimensions.Assign(g_StressTestBoxDims.x*2.0f,g_StressTestBoxDims.y,g_StressTestBoxThickness);
    g_TPEBoxesData.push_back(boxData);

    boxData.position.Assign(0.0f,g_StressTestBoxDims.y/2.0f,-g_StressTestBoxDims.z-g_StressTestBoxThickness*3.0f/2.0f);
    boxData.dimensions.Assign(g_StressTestBoxDims.x*2.0f,g_StressTestBoxDims.y,g_StressTestBoxThickness);
    g_TPEBoxesData.push_back(boxData);
    //*/

    boxData.position.Assign(0.0f,-g_StressTestBoxThickness,0.0f);
    boxData.dimensions.Assign(g_StressTestBoxDims.x*40.0f,g_StressTestBoxThickness,g_StressTestBoxDims.z*40.0f);
    g_TPEBoxesData.push_back(boxData);

}




// create the Stress scene here
void CreateStressTPE()
{
    // the physics engine itself
    pTPEEngine=new PhysicsWorld();
    SetEngine(pTPEEngine);
    pTPEEngine->SetNoOfSolvingIterations(50);
    pTPEEngine->SetObjectsAutoDeactivation(true);
    pTPEEngine->SetGravity(0,g_StressDemoGravity,0);

    GenerateBoxes();
    unsigned int i;




    for(i=0;i<g_TPEBoxesData.size();i++)
    {
        sGeomData& currBoxData=g_TPEBoxesData[i];
        nV::Body* pNewBoxBody=pTPEEngine->BodyMngr.Create();
        nV::Primitive *pBox=pNewBoxBody->AddBox(-1.0f,currBoxData.dimensions.x,currBoxData.dimensions.y,currBoxData.dimensions.z);
        pBox->SetFriction(10000.0f);
        
        sBoxMesh* pNewBoxMesh=new sBoxMesh();
        pNewBoxMesh->Create(currBoxData.dimensions.x,currBoxData.dimensions.y,currBoxData.dimensions.z,currBoxData.dimensions.x/50.0f,currBoxData.dimensions.y/50.0f,currBoxData.dimensions.z/50.0f);
        g_StressBoxMeshes.push_back(pNewBoxMesh);
        pNewBoxBody->SetPosition(currBoxData.position.x,currBoxData.position.y,currBoxData.position.z);
        g_TPEBoxBodies.push_back(pNewBoxBody);
    }
    g_StressTestCurrTime=timeGetTime();
    srand(100);

}

void RenderStressForSelection()
{
    Vector3 cameraLoc;
    Vector3 mouseRay;
    Graphics()->ComputeMouseRay(Mouse()->m_Loc.x,Mouse()->m_Loc.y,cameraLoc.v,mouseRay.v);

    Segment mouseSeg;
    mouseSeg.v0=g_StressCamera.m_Loc;
    mouseSeg.v1=g_StressCamera.m_Loc+mouseRay*10.0f;

    static Plane selectionPlane(0,1,0,0);

    if(Mouse()->m_bLButtonDown&&(g_StressPickID==-1))
    {
        float dist=10000.0f;
        float radius;

        for(unsigned int i=0;i<g_TPEDroppedBodies.size();i++)
        {
            nV::Body* pCurrentBody=g_TPEDroppedBodies[i];
            Vector3 position;
            pCurrentBody->GetPositon(position.x,position.y,position.z);

            sGeomData& currGeomData=g_TPEDroppedBodiesGeomData[i];
            if(currGeomData.type==0)
            {
                radius=max(currGeomData.dimensions.x,currGeomData.dimensions.y);
                radius=max(radius,currGeomData.dimensions.z);
                radius=radius/2.0f;
            }
            else
            {
                radius=currGeomData.radius;
            }

            if(RayHitSphere(cameraLoc,mouseRay,position,radius))
            {
                float newDist=(g_StressCamera.m_Loc-position).Mag();
                if(newDist<dist)
                {
                    g_StressPickID=i;
                    dist=newDist;
                }
            }
        }

        if(g_StressPickID!=-1)
        {
            nV::Body* draggedBody=g_TPEDroppedBodies[g_StressPickID];
            Vector3 OnPlane;
            draggedBody->GetPositon(OnPlane.x,OnPlane.y,OnPlane.z);
            selectionPlane=Plane(g_StressCamera.m_Fwd,OnPlane);
        }

    }


    if(!Mouse()->m_bLButtonDown)
    {
        g_StressPickID=-1;
    }

    if(g_StressPickID!=-1)
    {
        g_StressClickPosition=selectionPlane.Split(mouseSeg.v0,mouseSeg.v1);
    }
}

// render the Stress scene here
void RenderStressTPE(float dt,bool updateOnly)
{
    //pTPEEngine->RunStep(1.0f/30.0f);
	pTPEEngine->RunStep(0.01f);
    if(updateOnly)
        return;

    if(Mouse()->m_bLButtonDown)
        RenderStressForSelection();
    else
        g_StressPickID=-1;

    Matrix4 currMatrix;
    


    g_pStressBoxTexture->Activate();

    // render the box
    for(unsigned int i=0;i<g_TPEBoxBodies.size()-1;i++)
    {
        nV::Body* pCurrentBox=g_TPEBoxBodies[i];
        sGeomData& currBoxData=g_TPEBoxesData[i];;
        ComputeTPEBodyTransform(pCurrentBox,currMatrix);

        Graphics()->PushMatrix();
        Graphics()->MultMatrix(currMatrix.d);
        g_StressBoxMeshes[i]->Render();
        Graphics()->PopMatrix();
    }

    g_pStressWoodTexture->Activate();

    ComputeTPEBodyTransform(g_TPEBoxBodies.back(),currMatrix);
    Graphics()->PushMatrix();
    Graphics()->MultMatrix(currMatrix.d);
    g_StressBoxMeshes.back()->Render();
    Graphics()->PopMatrix();
    g_pStressWoodTexture->Deactivate();


    if(g_StressPickID>-1)
    {
        nV::Body* pDraggedBrick=g_TPEDroppedBodies[g_StressPickID];
        Vector3 brickPosition;
        pDraggedBrick->GetPositon(brickPosition.x,brickPosition.y,brickPosition.z);
        sMesh mesh;
        mesh.m_VertexFormat=MESH_VERTEXFORMAT_POSITION;
        mesh.m_RenderMode=MESH_RENDERMODE_LINELIST;
        mesh.AddVertex(brickPosition.v);
        mesh.AddVertex(g_StressClickPosition.v);
        mesh.Compile();
        mesh.Render();
    }
    Graphics()->EnableSphereMap();
    g_pStressEnvTexture->Activate();


    Material dummyMaterial;//=Material::AQUA_GLAZE_METAL;
    dummyMaterial.SetID(MATERIAL_ID_AQUA_GLAZE_METAL);

    dummyMaterial.SetEmissive(Color4(0.2f,0.2f,0.2f,0.2f));


    // render the dropped bodies here
    for(unsigned int i=0;i<g_TPEDroppedBodies.size();i++)
    {
        nV::Body* pCurrentBody=g_TPEDroppedBodies[i];
        sGeomData& currGeomData=g_TPEDroppedBodiesGeomData[i];
        ComputeTPEBodyTransform(pCurrentBody,currMatrix);
        Graphics()->PushMatrix();
        Graphics()->MultMatrix(currMatrix.d);
        dummyMaterial.SetDiffuse(currGeomData.color);
        dummyMaterial.Activate();
        g_StressMeshes[i]->Render();
        Graphics()->PopMatrix();
    }

    Graphics()->DisableSphereMap();
    g_pStressEnvTexture->Deactivate();


    // if a second passed...
    //if(timeGetTime()-g_StressTestCurrTime>g_StressGenTime)
        if(KEYDOWN(VK_SPACE))
        {
            g_StressTestCurrTime=timeGetTime();

            // create a random geometry
            // 1 is box...2.. is ball
            int geomType=rand()%2;


            Vector3 Position;

            Position.x=getRandFloat(-g_StressTestBoxDims.x/2.0f+5.0f,g_StressTestBoxDims.x/2.0f-5.0f);
            Position.z=getRandFloat(-g_StressTestBoxDims.z/2.0f+5.0f,g_StressTestBoxDims.z/2.0f-5.0f);
            Position.y=getRandFloat(60.0f,170.0f);

            sGeomData newGeomData;
            newGeomData.type=geomType;

            float density=getRandFloat(1.1f,1.7f);
            float friction=getRandFloat(g_StreeDemoMinFriction,g_StreeDemoMaxFriction);
            float elasticity=getRandFloat(0.1f,0.2f);
            Color3 color;
            color.r=getRandFloat(0.1f,1.0f);
            color.g=getRandFloat(0.1f,1.0f);
            color.b=getRandFloat(0.1f,1.0f);

            newGeomData.color=color;

            // a new rigid body
            nV::Body* pNewBody=NULL;

            Vector3 rotAxis;
            rotAxis.x=getRandFloat(0.0f,1.0f);
            rotAxis.y=getRandFloat(0.0f,1.0f);
            rotAxis.z=getRandFloat(0.0f,1.0f);
            rotAxis.Normalize();

            float angle=getRandFloat(0.0f,PI*2.0f);

            // creation
            // box
            if(geomType==0)
            {
                Vector3 Dimensions;
                Dimensions.x=getRandFloat(g_StressTestMinBoxDims.x,g_StressTestMaxBoxDims.x);
                Dimensions.y=getRandFloat(g_StressTestMinBoxDims.y,g_StressTestMaxBoxDims.y);
                Dimensions.z=getRandFloat(g_StressTestMinBoxDims.z,g_StressTestMaxBoxDims.z);
                float mass=density*Dimensions.x*Dimensions.y*Dimensions.z;
                newGeomData.dimensions=Dimensions;
                pNewBody=pTPEEngine->BodyMngr.Create();
                pNewBody->AddBox(mass,Dimensions.x,Dimensions.y,Dimensions.z);
                pNewBody->SetPosition(Position.x,Position.y,Position.z);

                sBoxMesh* pNewBoxMesh=new sBoxMesh();
                pNewBoxMesh->Create(Dimensions.x,Dimensions.y,Dimensions.z);
                g_StressMeshes.push_back(pNewBoxMesh);
            }
            else

            //sphere
            if(geomType==1)
            {
                float radius=getRandFloat(5.0f,7.0f);
                float mass=PI*4.0f/3.0f*density*radius*radius*radius;
                newGeomData.radius=radius;
                pNewBody=pTPEEngine->BodyMngr.Create();
                pNewBody->AddSphere(mass,radius);
                pNewBody->SetPosition(Position.x,Position.y,Position.z);

                sMesh* pNewSphereMesh=new sMesh;
                pNewSphereMesh->CreateSphere(radius);
                pNewSphereMesh->Compile();
                g_StressMeshes.push_back(pNewSphereMesh);
            }
            // cylinder
            else
            {
                float radius=getRandFloat(5.0f,7.0f);
                float mass=PI*4.0f/3.0f*density*radius*radius*radius;
                newGeomData.radius=radius;
                pNewBody=pTPEEngine->BodyMngr.Create();
                pNewBody->AddCylinder(mass,radius,radius);
                pNewBody->SetPosition(Position.x,Position.y,Position.z);

                sMesh* pNewSphereMesh=new sMesh;
                pNewSphereMesh->CreateCylinder(radius,radius,radius);
                pNewSphereMesh->Compile();
                g_StressMeshes.push_back(pNewSphereMesh);

            }

            //pNewBody->S

            pNewBody->RotateAxisAngle(rotAxis.x,rotAxis.y,rotAxis.z,angle);

            g_TPEDroppedBodiesGeomData.push_back(newGeomData);
            g_TPEDroppedBodies.push_back(pNewBody);
        }
}


// push the Stress here
void PushStressTPE()
{

    if(g_StressPickID>-1)
    {
        //System()->println("Picked %d",g_StressPickID);
        nV::Body* pDraggedBrick=g_TPEDroppedBodies[g_StressPickID];
        Vector3 Position;
        pDraggedBrick->GetPositon(Position.x,Position.y,Position.z);
        Vector3 Force=g_StressClickPosition-Position;
        float Mag=Force.Mag();
        Force.Normalize();
        Force*=Mag*Mag*13.0f;
        pDraggedBrick->AddOneFrameForce(Force.x,Force.y,Force.z);
    }
}


// destroy the Stress here
void DestroyStressTPE()
{
    g_TPEBoxesData.clear();
    g_TPEDroppedBodiesGeomData.clear();
    g_TPEBoxBodies.clear();
    g_TPEDroppedBodies.clear();

    DestroyVector(g_StressBoxMeshes);
    DestroyVector(g_StressMeshes);
    SetEngine(NULL);

    SAFE_DELETE(pTPEEngine);
}






void CreateStressTest(int APIID)
{
    g_pStressMusic=new cMusic("data/mozart.mid");
    g_pStressMusic->Play();

    g_pStressBoxTexture       =new sTexture("data/stone04a.jpg");
    g_pStressWoodTexture       =new sTexture("data/ggfol.jpg");
    g_pStressEnvTexture       =new sTexture("data/envmap.jpg");
    //    g_pDestructionGroundTexture =new sTexture("data/metalfloor_wall_14.jpg");
    //  g_pDestructionEnvTexture    =

    CreateStressTPE();
    
}


void RenderStressTest(float dt,int APIID,bool updateOnly)
{
    RenderStressTPE(dt,updateOnly);

}
void PushStressTest(int APIID)
{
    PushStressTPE();
}
void DestroyStressTest(int APIID)
{
    DestroyStressTPE();
    
    SAFE_DELETE(g_pStressBoxTexture);
    SAFE_DELETE(g_pStressEnvTexture);
    SAFE_DELETE(g_pStressWoodTexture);

    g_pStressMusic->Stop();
    SAFE_DELETE(g_pStressMusic);
    
}
void DisplayStressHelp()
{
    TextPrintf(10,30,"Press Space to drop a random body");
    TextPrintf(10,50,"Use mouse wheel to zoom in out");
    TextPrintf(10,70,"Hold right mouse button to change camera");
    TextPrintf(10,90,"%d Objects",g_TPEDroppedBodiesGeomData.size());
}