#include "system/engine.h"
#include "3d/graphics/graphicssubsystem.h"
#include "3d/3dtypes/3dtypes.h"
#include "3d/meshes/indexmesh.h"

#include "3d/bsp/bspcamera.h"
#include "audio/audio.h"
#include "3d/text/text.h"


// and thise is for TPE
#include "nvphysics.h"

// header implemented here
#include "test.h"
#include "tpe.h"

cCamera g_TestCamera;
cBSPCamera* g_pBSPCamera=NULL;

float g_TestGravity=-300.8f;

void SetTestCamera(cCamera& camera)
{
    g_TestCamera=camera;
}



sTexture* g_pSquashTexture=NULL;
vector<nV::Body*> g_SquashBalls;
vector<sMesh*> g_SquashMeshes;




cTPECollidableBSPFile* g_pBSPFile=NULL;

// create both as sphere initially
void CreateTest(int APIID)
{

    g_pSquashTexture=new sTexture("data/logo.tga");
    // the physics engine itself
    pTPEEngine=new PhysicsWorld();
    SetEngine(pTPEEngine);
    pTPEEngine->SetGravity(0.0f,g_TestGravity,0.0f);
    pTPEEngine->SetObjectsAutoDeactivation(false);
    pTPEEngine->SetNoOfSolvingIterations(20);
    //   pTPEEngine->debug_Freeze=true;

    
    nV::ConvexShape* pConvex=pTPEEngine->ShapeMngr.CreateConvex();
    pConvex->Begin();
    //pConvex->AddVertex()
    pConvex->End();
    
   
    

    // create the 2 bodies
   // nV::Body* pLevelBody=pTPEEngine->BodyMngr.Create();
    
    


    nV::TriMeshShape* pLevelShape=pTPEEngine->ShapeMngr.CreateTriMesh();

    // load the bsp file
    g_pBSPFile=new cTPECollidableBSPFile();
    g_pBSPFile->Load("maps/level.bsp");
    g_pBSPCamera=new cBSPCamera();
    g_pBSPCamera->SetMap(*g_pBSPFile);
    g_pBSPCamera->m_Gravity=0.7f;
    g_pBSPCamera->m_Loc=g_pBSPFile->m_StartPositions[0];

    
  //  g_pBSPFile->BuildCollisionTree(pLevelShape);
    vector<nV::ConvexShape*> convexShapes;
    g_pBSPFile->BuildConvexes(pTPEEngine,convexShapes);

    for(int i=0;i<convexShapes.size();i++)
    {
        nV::Body* pNewLevelBody=pTPEEngine->BodyMngr.Create();
    nV::Primitive* pLevelPrimitive=pNewLevelBody->AddNewPrimitive(convexShapes[i],-1);
   // pLevelPrimitive->SetLocalPosition(g_pBSPCamera->m_Loc.x,g_pBSPCamera->m_Loc.y,g_pBSPCamera->m_Loc.z);
    pLevelPrimitive->SetElasticity(0.9f);
    pLevelPrimitive->SetFriction(12000.0f);
    }
}


int g_NumConvexConvexTests=0;


void AddConvexCall()
{
    g_NumConvexConvexTests++;

}
void RenderTest(float dt,int APIID,bool updateOnly)
{
    g_NumConvexConvexTests=0;

    pTPEEngine->RunStep(0.01f);

    if(updateOnly)
        return;

   // System()->println("%d tests",g_NumConvexConvexTests);


    Matrix4 currMatrix;

    g_pBSPCamera->Handle(2.1f);
    g_pBSPCamera->Activate();
    Graphics()->CalculateFrustum();
    Graphics()->DisableLighting();
    if(KEYUP(VK_SHIFT))
    g_pBSPFile->RenderFrom(g_pBSPCamera->m_Loc);
    Graphics()->EnableLighting();

//    g_pBSPFile->m_Tree.CreateAllConvexes();

    //    Graphics()->DisableLighting();
    Graphics()->RenderAxis(4.0f);

    Vector3 testPos=g_pBSPFile->m_StartPositions[0];

    Graphics()->PushMatrix();
    Graphics()->Translate(testPos.x,testPos.y,testPos.z);
  //  Graphics()->RenderBox(100,100,100);
    Graphics()->PopMatrix();

//    g_pSquashTexture->Activate();
    // render the first
    for(unsigned int i=0;i<g_SquashBalls.size();i++)
    {
        nV::Body* pBallBody=g_SquashBalls[i];
        ComputeTPEBodyTransform(pBallBody,currMatrix);
        Graphics()->PushMatrix();
        Graphics()->MultMatrix(currMatrix.d);
        g_SquashMeshes[i]->Render();
        Graphics()->PopMatrix();
    }
  //  g_pSquashTexture->Deactivate();

}
void PushTest(int APIID)
{
    static DWORD lastTime=timeGetTime();
    DWORD thisTime=timeGetTime();

    DWORD timeDiff=thisTime-lastTime;


    if(KEYDOWN(VK_SPACE)&& timeDiff>500)
    {
        if(g_SquashMeshes.size()>20)
        {
            sMesh* pOldMesh=g_SquashMeshes.front();
            nV::Body* pOldBody=g_SquashBalls.front();

            delete pOldMesh;
            pTPEEngine->BodyMngr.Destroy(pOldBody);

            g_SquashMeshes.erase(&g_SquashMeshes[0]);
            g_SquashBalls.erase(&g_SquashBalls[0]);
        }
        lastTime=timeGetTime();
        Vector3 position=g_pBSPCamera->m_Loc+g_pBSPCamera->m_Fwd*100.0f;

        nV::Body* pBallBody=pTPEEngine->BodyMngr.Create();
        //CPrimitive* pBallPrimitive=pBallBody->AddSphere(0.01f,5.0f);

        srand(timeGetTime());
        int type=rand()%3;

        nV::Primitive* pPrimitive=NULL;
        sMesh* pNewMesh=NULL;
        if(type==2)
        {
            pPrimitive=pBallBody->AddSphere(0.01f,15);
            pNewMesh=new sMesh();
            pNewMesh->m_VertexFormat=MESH_VERTEXFORMAT_TEXCOORD;
            pNewMesh->CreateSphere(15);
            pNewMesh->Compile();

        }
        else
        if(type==1)
        {
            Vector3 boxSize;
            boxSize.x=30+rand()%15;
            boxSize.y=30+rand()%15;
            boxSize.z=30+rand()%15;
            pPrimitive=pBallBody->AddBox(0.01f,boxSize.x,boxSize.y,boxSize.z);
            sIndexedMesh* pBoxMesh=new sIndexedMesh();
            pBoxMesh->CreateBox(boxSize.x,boxSize.y,boxSize.z);
            pBoxMesh->Compile();
            pNewMesh=pBoxMesh;
        }
        else
        {
            nV::ConvexShape* pNewShape=pTPEEngine->ShapeMngr.CreateConvex();

            pNewMesh=new sMesh();
            pNewMesh->m_VertexFormat=MESH_VERTEXFORMAT_TEXCOORD;
            pNewMesh->CreateCylinder(20+rand()%15,10+rand()%25,10+rand()%25,3+rand()%15);
            pNewMesh->Compile();

            pNewShape->Begin();
            pNewShape->AddTriangles(pNewMesh->m_pVertexData,6*sizeof(float),pNewMesh->m_NumVertices);
            pNewShape->End();
            pPrimitive=pBallBody->AddNewPrimitive(pNewShape,0.01f);
            
        }


        
        g_SquashMeshes.push_back(pNewMesh);
        
        pPrimitive->SetElasticity(0.3f);
        pPrimitive->SetFriction(12000.0f);
        g_SquashBalls.push_back(pBallBody);



        pBallBody=g_SquashBalls.back();
        pBallBody->SetLinearVelocity(0.0f,0.0f,0.0f);
        pBallBody->AddOneFrameForce(g_pBSPCamera->m_Fwd.x*700.0f,g_pBSPCamera->m_Fwd.y*700.0f,g_pBSPCamera->m_Fwd.z*700.0f);
        position+=g_pBSPCamera->m_Up*100.0f;
        pBallBody->SetPosition(position.x,position.y,position.z);
    }
}


void DestroyTest(int APIID)
{
    DestroyVector(g_SquashMeshes);
    SAFE_DELETE(g_pSquashTexture);
    g_SquashBalls.clear();
    SAFE_DELETE(pTPEEngine);
    SAFE_DELETE(g_pBSPCamera);
    SAFE_DELETE(g_pBSPFile);
    SetEngine(NULL);
}

void DisplayTestHelp()
{
    TextPrintf(10,90,"Press Space to throw a random shape");
    TextPrintf(10,110,"use the mouse and cursor keys to navigate in the room");

}