#include "../package/nVPhysics.h"
#include "tpe.h"
#include "system/engine.h"


//CPhEngine *pTPEEngine=NULL;


// start TPE here
void BeginTPE()
{
    SAFE_DELETE(pTPEEngine);
    pTPEEngine=new PhysicsWorld();
}

// endit with this
void EndTPE()
{
    SAFE_DELETE(pTPEEngine);
}
void SetEngine(PhysicsWorld* pEngine)
{
    pTPEEngine=pEngine;
}
float GetLastTimeStep()
{
  //  if(pTPEEngine)
//        return pTPEEngine->GetTimeStep();
    return 0.0f;
}


int GetNumIterations()
{
//    if(pTPEEngine)
  //      return pTPEEngine->GetNoOfSolvingIterations();
    return 0;
}



// get the tarnsformtion of a rigid body within TPE
void ComputeTPEBodyTransform(nV::Body* pTPEBody,Matrix4& matrix)
{
    matrix.MakeIdent();
    pTPEBody->GetPositon(matrix._41,matrix._42,matrix._43);
    float* pRotMatrix=pTPEBody->GetRotationMatrix();

    matrix._11=pRotMatrix[0];
    matrix._21=pRotMatrix[1];
    matrix._31=pRotMatrix[2];
    matrix._12=pRotMatrix[3];
    matrix._22=pRotMatrix[4];
    matrix._32=pRotMatrix[5];
    matrix._13=pRotMatrix[6];
    matrix._23=pRotMatrix[7];
    matrix._33=pRotMatrix[8];
}

void ComputeTPEPrimitiveTransform(nV::Primitive* pTPEPrimitive,Matrix4& matrix)
{
    matrix.MakeIdent();

    pTPEPrimitive->GetWorldPosition(matrix._41,matrix._42,matrix._43);

    //pTPEPrimitive->GetLocalPosition(
    float* pRotMatrix=pTPEPrimitive->GetWorldRotationMatrix();

    matrix._11=pRotMatrix[0];
    matrix._21=pRotMatrix[1];
    matrix._31=pRotMatrix[2];
    matrix._12=pRotMatrix[3];
    matrix._22=pRotMatrix[4];
    matrix._32=pRotMatrix[5];
    matrix._13=pRotMatrix[6];
    matrix._23=pRotMatrix[7];
    matrix._33=pRotMatrix[8];


}

void ComputeTPEGeomTransform(nV::Primitive* pTPEPrimitive,Matrix4& matrix)
{
    matrix.MakeIdent();
    pTPEPrimitive->GetLocalPosition(matrix._41,matrix._42,matrix._43);
    float* pRotMatrix=pTPEPrimitive->GetLocalRotationMatrix();

    matrix._11=pRotMatrix[0];
    matrix._21=pRotMatrix[1];
    matrix._31=pRotMatrix[2];
    matrix._12=pRotMatrix[3];
    matrix._22=pRotMatrix[4];
    matrix._32=pRotMatrix[5];
    matrix._13=pRotMatrix[6];
    matrix._23=pRotMatrix[7];
    matrix._33=pRotMatrix[8];
}

void cTPECollidableBSPFile::BuildConvexes(PhysicsWorld* pEngine,vector<nV::ConvexShape*>& shapes)
{

    // loop on ALL the brushes...

    vector<Vector3> dummy;

    for(int i=0;i<m_Tree.NumBrushes;i++)
    {
        tBSPBrush* brush=&m_Tree.Brushes[i];

        if(brush->numOfBrushSides>0&&(m_Tree.Textures[brush->textureID].contents&1))
        {
            m_Tree.CreateConvex(dummy,i);
        }

        // create a convex and add its vertices
        nV::ConvexShape* pNewConvex=pEngine->ShapeMngr.CreateConvex();

        pNewConvex->Begin();
        pNewConvex->AddVertices(&dummy[0].v,3*sizeof(float),dummy.size());
        pNewConvex->End();
        dummy.clear();
        shapes.push_back(pNewConvex);
    }

    System()->println("Number of convex shapes in the bsp file is %d",shapes.size());
}


void cTPECollidableBSPFile::BuildCollisionTree(nV::TriMeshShape* pTriMesh)
{
    m_Tree.CreateAllConvexes();
    pTriMesh->Begin();

    int j;

    Vector3 A;
    Vector3 B;
    Vector3 C;


    // loop on all faces
    for(int i=0;i<m_NumFaces;i++)
    {
        // take the current face
        tBSPFace *pFace = &m_BSPLoader.m_pFaces[i];



        if(pFace->type==FACE_POLYGON)
        {
            //A=
            A.x=m_BSPLoader.m_pVertices[pFace->startVertIndex].vPosition.x;
            A.y=m_BSPLoader.m_pVertices[pFace->startVertIndex].vPosition.y;
            A.z=m_BSPLoader.m_pVertices[pFace->startVertIndex].vPosition.z;


            for(j=1;j<(pFace->numOfVerts-1);j++)
            {
                B.x=m_BSPLoader.m_pVertices[pFace->startVertIndex+j].vPosition.x;
                B.y=m_BSPLoader.m_pVertices[pFace->startVertIndex+j].vPosition.y;
                B.z=m_BSPLoader.m_pVertices[pFace->startVertIndex+j].vPosition.z;

                C.x=m_BSPLoader.m_pVertices[pFace->startVertIndex+j+1].vPosition.x;
                C.y=m_BSPLoader.m_pVertices[pFace->startVertIndex+j+1].vPosition.y;
                C.z=m_BSPLoader.m_pVertices[pFace->startVertIndex+j+1].vPosition.z;

                pTriMesh->AddTriangle((nvReal*)&A,(nvReal*)&B,(nvReal*)&C);

            }
        }
    }
    pTriMesh->End();
}
