#include "vehicule.h"



cVehicule::cVehicule()
{
    m_pChasisMesh=NULL;
    m_pWheelMesh=NULL;
    m_WheelsRadius=8.0f;

    m_ChasisDims.Assign(14,7,20);

    m_FrontLeftPos.Assign(-m_ChasisDims.x/2.0f-m_WheelsRadius,10,-m_ChasisDims.z/2.0f-m_WheelsRadius*2.0f);
    m_FrontRightPos.Assign(m_ChasisDims.x/2.0f+m_WheelsRadius,10,-m_ChasisDims.z/2.0f-m_WheelsRadius*2.0f);
    m_RearLeftPos.Assign(-m_ChasisDims.x/2.0f-m_WheelsRadius,10,+m_ChasisDims.z/2.0f+m_WheelsRadius*2.0f);
    m_RearRightPos.Assign(m_ChasisDims.x/2.0f+m_WheelsRadius,10,+m_ChasisDims.z/2.0f+m_WheelsRadius*2.0f);


    m_WheelsFriction=10000.0f;

    m_BodyPos.Assign(0,10,0);

    m_BodyMass=600.0f;

    m_WheelMass=m_BodyMass/10.0f;

    m_MaxAngle=60.0f*PI/180.0f;

}

void cVehicule::Create(PhysicsWorld* pEngine)
{
    m_pChasisBody=pEngine->BodyMngr.Create();

    // compute the chasis dimentions from the wheels positions and radii
    m_ChasisDims.x=abs(m_FrontLeftPos.x-m_RearRightPos.x);
    m_ChasisDims.x-=m_WheelsRadius*3.0f;
    m_ChasisDims.z=abs(m_FrontLeftPos.z-m_RearRightPos.z);
    //m_ChasisDims.x=m_FrontLeftPos.x-m_RearRightPos.x;


    m_pChasisBody->AddBox(m_BodyMass,m_ChasisDims.x,m_ChasisDims.y,m_ChasisDims.z);
    m_pChasisBody->SetPosition(m_BodyPos.x,m_BodyPos.y,m_BodyPos.z);

    m_pChasisMesh=new sIndexedMesh();
    //m_pChasisMesh-
    m_pChasisMesh->CreateBox(m_ChasisDims.x,m_ChasisDims.y,m_ChasisDims.z);
    m_pChasisMesh->Compile();


    m_pWheelMesh=new sMesh();
    //m_pWheelMesh->CreateSphere(m_WheelsRadius,10,10);
    m_pWheelMesh->CreateCylinder(3,m_WheelsRadius,m_WheelsRadius);
    m_pWheelMesh->Compile();


    nV::Primitive* pPrim=NULL;

    // create the 4 wheels
    nV::Body* pFrontLeftWheel=pEngine->BodyMngr.Create();

    pPrim=pFrontLeftWheel->AddSphere(m_WheelMass,m_WheelsRadius);
    pFrontLeftWheel->SetPosition(m_FrontLeftPos.x+m_BodyPos.x,m_FrontLeftPos.y+m_BodyPos.y,m_FrontLeftPos.z+m_BodyPos.z);
    pPrim->SetFriction(m_WheelsFriction);

    nV::Body* pFrontRightWheel=pEngine->BodyMngr.Create();
    pPrim=pFrontRightWheel->AddSphere(m_WheelMass,m_WheelsRadius);
    pPrim->SetFriction(m_WheelsFriction);
    pFrontRightWheel->SetPosition(m_FrontRightPos.x+m_BodyPos.x,m_FrontRightPos.y+m_BodyPos.y,m_FrontRightPos.z+m_BodyPos.z);

    nV::Body* pRearLeftWheel=pEngine->BodyMngr.Create();
    pPrim=pRearLeftWheel->AddSphere(m_WheelMass,m_WheelsRadius);
    pPrim->SetFriction(m_WheelsFriction);
    pRearLeftWheel->SetPosition(m_RearLeftPos.x+m_BodyPos.x,m_RearLeftPos.y+m_BodyPos.y,m_RearLeftPos.z+m_BodyPos.z);

    nV::Body* pRearRightWheel=pEngine->BodyMngr.Create();
    pPrim = pRearRightWheel->AddSphere(m_WheelMass,m_WheelsRadius);
    pPrim->SetFriction(m_WheelsFriction);
    pRearRightWheel->SetPosition(m_RearRightPos.x+m_BodyPos.x,m_RearRightPos.y+m_BodyPos.y,m_RearRightPos.z+m_BodyPos.z);

    m_Wheels.push_back(pFrontLeftWheel);
    m_Wheels.push_back(pFrontRightWheel);
    m_Wheels.push_back(pRearLeftWheel);
    m_Wheels.push_back(pRearRightWheel);

    // create the car wheel joints
    m_pFrontLeftJoint=(nV::CarWheelJoint*)pEngine->JointMngr.CreateCarWheelJoint(m_pChasisBody, pFrontLeftWheel,m_FrontLeftPos.x+m_BodyPos.x,m_FrontLeftPos.y+m_BodyPos.y,m_FrontLeftPos.z+m_BodyPos.z,0,0,1,0,1,0);
    m_pFrontRightJoint=(nV::CarWheelJoint*)pEngine->JointMngr.CreateCarWheelJoint(m_pChasisBody, pFrontRightWheel,m_FrontRightPos.x+m_BodyPos.x,m_FrontRightPos.y+m_BodyPos.y,m_FrontRightPos.z+m_BodyPos.z,0,0,1,0,1,0);
    m_pRearLeftJoint=(nV::CarWheelJoint*)pEngine->JointMngr.CreateCarWheelJoint(m_pChasisBody, pRearLeftWheel, m_RearLeftPos.x+m_BodyPos.x,m_RearLeftPos.y+m_BodyPos.y,m_RearLeftPos.z+m_BodyPos.z,0,0,1,0,1,0);
    m_pRearRightJoint=(nV::CarWheelJoint*)pEngine->JointMngr.CreateCarWheelJoint(m_pChasisBody, pRearRightWheel, m_RearRightPos.x+m_BodyPos.x,m_RearRightPos.y+m_BodyPos.y,m_RearRightPos.z+m_BodyPos.z,0,0,1,0,1,0);

    m_pRearLeftJoint->SetMaxTorque(10000);
    m_pRearRightJoint->SetMaxTorque(10000);
}
void cVehicule::Handle()
{
    Matrix4 currMatrix;

    if(KEYDOWN(VK_SPACE))
    {
        ComputeTPEBodyTransform(m_pChasisBody,currMatrix);
        Graphics()->PushMatrix();
        Graphics()->MultMatrix(currMatrix.d);
        m_pChasisMesh->Render();
        Graphics()->PopMatrix();
    }

    unsigned int i;


    if(KEYDOWN(VK_UP))
    {
        m_pRearLeftJoint->SetMaxTorque(100000);
        m_pRearRightJoint->SetMaxTorque(100000);

		m_pFrontLeftJoint->SetMaxTorque(0);
        m_pFrontRightJoint->SetMaxTorque(0);

        m_pRearLeftJoint->SetRequiredVelocity(300.0f);
        m_pRearRightJoint->SetRequiredVelocity(300.0f);
    }
    else

    if(KEYDOWN(VK_DOWN))
    {
        m_pRearLeftJoint->SetMaxTorque(100000);
        m_pRearRightJoint->SetMaxTorque(100000);

		m_pFrontLeftJoint->SetMaxTorque(0);
        m_pFrontRightJoint->SetMaxTorque(0);

        m_pRearLeftJoint->SetRequiredVelocity(-300.0f);
        m_pRearRightJoint->SetRequiredVelocity(-300.0f);
    }
    else if(KEYDOWN('B'))
    {
        m_pRearLeftJoint->SetMaxTorque(1000000);
        m_pRearRightJoint->SetMaxTorque(1000000);

        m_pRearLeftJoint->SetRequiredVelocity(0);
        m_pRearRightJoint->SetRequiredVelocity(0);

		m_pFrontLeftJoint->SetMaxTorque(1000000);
        m_pFrontRightJoint->SetMaxTorque(1000000);

		m_pFrontLeftJoint->SetRequiredVelocity(0);
		m_pFrontRightJoint->SetRequiredVelocity(0);
    }
    else
    {
        m_pRearLeftJoint->SetMaxTorque(0);
        m_pRearRightJoint->SetMaxTorque(0);

		m_pFrontLeftJoint->SetMaxTorque(0);
        m_pFrontRightJoint->SetMaxTorque(0);

    }

    static float steerAngle=0.0f;


    if(KEYDOWN(VK_RIGHT))
    {
        steerAngle-=0.1f;
    }
    else

    if(KEYDOWN(VK_LEFT))
    {
        steerAngle+=0.1f;
    }
    else
    {
        if(steerAngle>0.0f)
            steerAngle-=0.1f;
        if(steerAngle<0.0f)
            steerAngle+=0.1f;

		if(abs(steerAngle)< 0.1f) steerAngle =0;
    }

    if(steerAngle>m_MaxAngle)
        steerAngle=m_MaxAngle;
    if(steerAngle<-m_MaxAngle)
        steerAngle=-m_MaxAngle;

    m_pFrontLeftJoint->SetSteeringAngle(steerAngle);
    m_pFrontRightJoint->SetSteeringAngle(steerAngle);



    Vector3 FwdDirection=currMatrix.GetLocalZ();
    FwdDirection.Normalize();
    FwdDirection*=200.0f;


    for(i=0;i<m_Wheels.size();i++)
    {
        nV::Body* pCurrBody=m_Wheels[i];
        ComputeTPEBodyTransform(pCurrBody,currMatrix);
        Graphics()->PushMatrix();
        Graphics()->MultMatrix(currMatrix.d);
        //            Graphics()->RenderSphere(m_WheelsRadius);
        //          Graphics()->Rotate(90.0f,0,0,1);
        //        m_pWheelMesh->Render();
        Graphics()->PopMatrix();
    }
}
