#include "system/engine.h"
#include "3d/graphics/graphicssubsystem.h"
#include "3d/3dtypes/3dtypes.h"
#include "3d/text/text.h"

// this is for ODE
//#include "physics/physics.h"
#include "audio/audio.h"
#include "boxmesh.h"

// and this is for TPE
#include "nvphysics.h"

// header implemented here
#include "tpe.h"
#include "waterwheel.h"
#include "stress.h"

float g_WaterWheelDemoGravity=-20.8f;

int   g_WaterWheelDemoNumBalls=15;

float   g_WaterWheelDemoBallsRadius=2.0f;
float   g_WaterWheelDemoBallsDensity=40.0f;
float   g_WaterWheelDemoBallsEalsticy=0.9f;
float   g_WaterWheelDemoBallsFriction=0.1f;

int   g_WaterWheelDemoNumTeeth          =   10;
float g_WaterWheelDemoToothWidth        =   6.0f;
float g_WaterWheelDemoToothLength       =   20.0f;
float g_WaterWheelDemoToothThickness    =   2.5f;
float g_WaterWheelDemoToothDensity      =   0.01f;
float g_WaterWheelDemoInnerRadius       =   8.0f;
float g_WaterWheelDemoToothElasticity   =   0.2f;
float g_WaterWheelDemoToothFriction     =   0.0f;

cSound* g_pWaterWheelMusic=NULL;
sTexture *g_pWaterWheelTexture=NULL;


cCamera g_WaterWheelCamera;
sBoxMesh g_WaterWheelBoxMesh;
Vector3 g_WaterWheelSpawnLoc(0.0f,0.0f,0.0f);

void SetWaterWheelCamera(cCamera& camera)
{
    g_WaterWheelCamera=camera;
}




struct sTooth
{
    Vector3 Position;
    Vector3 Dimensions;
    float Angle;
};


struct sGear
{
    Vector3 m_Center;
    vector<sTooth> m_Teeth;


    sGear(Vector3& center,int numTeeth)
    {
        m_Teeth.clear();
        m_Center=center;

        float innerRadius=g_WaterWheelDemoInnerRadius;
        float toothWidth=g_WaterWheelDemoToothWidth;
        float toothThickness=g_WaterWheelDemoToothThickness;
        float toothDepth=g_WaterWheelDemoToothLength;


        for(int i=0;i<numTeeth;i++)
        {
            sTooth newTooth;

            // the tooth
            newTooth.Angle=((float)i)/numTeeth*2.0f*PI;

            // assign its dimensions
            newTooth.Dimensions.Assign(toothWidth,toothThickness,toothDepth);

            // location of the fan
            Vector3 Center;
            newTooth.Position.x=toothWidth/2.0f+innerRadius;
            newTooth.Position.y=0.0f;
            newTooth.Position.z=0.0f;

            Matrix4 rotMatrix;
            rotMatrix.MakeIdent();
            rotMatrix.Rotate(0,0,newTooth.Angle);
            newTooth.Position=newTooth.Position*rotMatrix;

            // get around the center
            //newTooth.Position+=Center;

            // add the tooth
            m_Teeth.push_back(newTooth);
        }
    }


};



vector<sGear*> g_WheelDemoGears;


void CreateAllGears()
{

    float offsetX=g_WaterWheelDemoToothWidth+g_WaterWheelDemoInnerRadius;
    float offsetY = 1.2f*offsetX;
    offsetX*=1.5f;

    //float offsetY=g_WaterWheelDemoToothThickness*7.0f;
    int numGears=10;

    float totalWidth=offsetX*numGears;

    g_WaterWheelSpawnLoc.x=totalWidth/2.0f;

    //for(int i=0;i<numGears;i++)
    //{
    //    sGear* pGear=new sGear(Vector3(1.5*offsetX*(i%2)+(totalWidth/2.0f),200-offsetY*i,0),g_WaterWheelDemoNumTeeth);
    //    g_WheelDemoGears.push_back(pGear);
    //}

    for(int i=0;i<numGears;i++)
    {
        sGear* pGear=new sGear(Vector3(-150+0.8f*offsetX*i+(totalWidth/2.0f),-offsetY*(i%2),0),g_WaterWheelDemoNumTeeth);
        g_WheelDemoGears.push_back(pGear);
    }

    g_WaterWheelBoxMesh.Create(g_WaterWheelDemoToothWidth,g_WaterWheelDemoToothThickness,g_WaterWheelDemoToothLength);
}


void DestroyAllGears()
{
    DestroyVector(g_WheelDemoGears);

}


vector<nV::Body*> g_TPEFanBodies;
vector<nV::Body*> g_TPEFBallsBodies;


void CreateGearTPE(sGear& gear)
{
    Vector3& center=gear.m_Center;

    // first create a dummy sphere at its center
    nV::Body* pDummyBody=pTPEEngine->BodyMngr.Create();
    pDummyBody->AddSphere(-1,0.001f);
    pDummyBody->SetPosition(center.x,center.y,center.z);


    nV::Body* pFirstBody=NULL;
    nV::Body* pNewTPEBody=pTPEEngine->BodyMngr.Create();

    // now loop on all the teeth..
    for(unsigned int i=0;i<gear.m_Teeth.size();i++)
    {
        // get the current tooth
        sTooth& currTooth=gear.m_Teeth[i];

        // get all the data items needed
        Vector3& toothCenter=currTooth.Position;
        float& toothAngle=currTooth.Angle;
        Vector3& toothDims=currTooth.Dimensions;
        float toothMass=toothDims.x*toothDims.y*toothDims.z*g_WaterWheelDemoToothDensity;


        nV::Primitive* pNewBox=pNewTPEBody->AddBox(toothMass,toothDims.x,toothDims.y,toothDims.z);
        pNewBox->SetLocalPosition(toothCenter.x,toothCenter.y,toothCenter.z);
        pNewBox->RotateAxisAngle(0.0f,0.0f,1.0f,toothAngle);
        pNewTPEBody->SetData(&currTooth);

        pNewBox->SetElasticity(g_WaterWheelDemoToothElasticity);
        pNewBox->SetFriction(g_WaterWheelDemoToothFriction);
    }
    pNewTPEBody->SetPosition(center.x,center.y,center.z);
	pTPEEngine->JointMngr.CreateHingeJoint(pDummyBody,pNewTPEBody,center.x,center.y,center.z,0,0,1);
    pFirstBody=pNewTPEBody;

    g_TPEFanBodies.push_back(pNewTPEBody);
    //pTPEEngine->Joints.Fixed.Create(g_TPEFanBodies.back(),pFirstBody);
}


// create the WaterWheel scene here
void CreateWaterWheelTPE()
{
    // the physics engine itself
    pTPEEngine=new PhysicsWorld();
    SetEngine(pTPEEngine);
    pTPEEngine->SetGravity(0,g_WaterWheelDemoGravity,0);
    pTPEEngine->SetNoOfSolvingIterations(100);
    pTPEEngine->SetObjectsAutoDeactivation(false);


    CreateAllGears();
    for(unsigned int i=0;i<g_WheelDemoGears.size();i++)
    {
        CreateGearTPE(*g_WheelDemoGears[i]);
    }

    float ballMass=4.0f/3.0f*PI*g_WaterWheelDemoBallsRadius*g_WaterWheelDemoBallsRadius*g_WaterWheelDemoBallsRadius;

    // create the balls
    for( int i=0;i<g_WaterWheelDemoNumBalls;i++)
    {
        nV::Body* pNewBallBody=pTPEEngine->BodyMngr.Create();
        pNewBallBody->AddSphere(ballMass,g_WaterWheelDemoBallsRadius);
        //pNewBallBody->SetPosition(g_WaterWheelSpawnLoc.x+g_WaterWheelDemoInnerRadius+g_WaterWheelDemoToothWidth/2.0f,200.0f+i*6.0f,0.0f);
        pNewBallBody->SetPosition(g_WaterWheelSpawnLoc.x-(g_WaterWheelDemoInnerRadius+g_WaterWheelDemoToothWidth/2.0f)-150,40.0f+i*40.0f,0.0f);
        g_TPEFBallsBodies.push_back(pNewBallBody);
    }
}

// render the WaterWheel scene here
void RenderWaterWheelTPE(float dt,bool updateOnly)
{
    pTPEEngine->RunStep(dt);

    if(updateOnly)
        return;

    Matrix4 currMatrix;
    Matrix4 geomMatrix;

    g_pWaterWheelTexture->Activate();

    


    for(unsigned int i=0;i<g_TPEFanBodies.size();i++)
    {
        nV::Body* pCurrentFan=g_TPEFanBodies[i];
        ComputeTPEBodyTransform(pCurrentFan,currMatrix);
        sTooth* pTooth=(sTooth*)pCurrentFan->GetData();
        Graphics()->PushMatrix();
        Graphics()->MultMatrix(currMatrix.d);
        //pCurrentFan->
        for(unsigned int j=0;j<pCurrentFan->Primitives.size();j++)
        {
            nV::Primitive* pCurrBox=(nV::Primitive*)pCurrentFan->Primitives[j];
            ComputeTPEGeomTransform(pCurrBox,geomMatrix);
            Graphics()->PushMatrix();
            Graphics()->MultMatrix(geomMatrix.d);
            //            Graphics()->RenderBox(pTooth->Dimensions.x,pTooth->Dimensions.y,pTooth->Dimensions.z);
            g_WaterWheelBoxMesh.Render();
            Graphics()->PopMatrix();
        }
        Graphics()->PopMatrix();
    }

    g_pWaterWheelTexture->Deactivate();


    for(unsigned int i=0;i<g_TPEFBallsBodies.size();i++)
    {
        nV::Body* pCurrentBall=g_TPEFBallsBodies[i];
        ComputeTPEBodyTransform(pCurrentBall,currMatrix);

        Graphics()->PushMatrix();
        Graphics()->MultMatrix(currMatrix.d);

        //Graphics()->RenderBox(g_WaterWheelDemoFanWidth,g_WaterWheelDemoFanThickness,g_WaterWheelDemoFanLength);
        Graphics()->RenderSphere(g_WaterWheelDemoBallsRadius);
        Graphics()->PopMatrix();

        // reset the balls
        if(currMatrix._42<-10.0f&&KEYDOWN(VK_SPACE))
        {
            //pCurrentBall->SetPosition(g_WaterWheelSpawnLoc.x+g_WaterWheelDemoInnerRadius+g_WaterWheelDemoToothWidth/2.0f,200.0f+i*6.0f,0.0f);
            pCurrentBall->SetPosition(g_WaterWheelSpawnLoc.x-(g_WaterWheelDemoInnerRadius+g_WaterWheelDemoToothWidth/2.0f)-150,40.0f+i*40.0f,0.0f);
            pCurrentBall->SetLinearVelocity(0,0,0);
        }
    }
}

// destroy the WaterWheel here
void DestroyWaterWheelTPE()
{
    g_TPEFanBodies.clear();
    g_TPEFBallsBodies.clear();
    DestroyAllGears();
    SAFE_DELETE(pTPEEngine);
    SetEngine(NULL);
    //pTPEEngine->Joints.StiffSpring.Create()
}

// push the WaterWheel here
void PushWaterWheelTPE()
{
    static bool keyPressed=false;

    if(KEYDOWN(VK_INSERT)&&(keyPressed==false))
    {
        keyPressed=true;
        g_WaterWheelDemoNumTeeth++;
    }

    if(KEYDOWN(VK_DELETE)&&(keyPressed==false))
    {
        keyPressed=true;
        g_WaterWheelDemoNumTeeth--;
    }


    if(g_WaterWheelDemoNumTeeth<2)
        g_WaterWheelDemoNumTeeth=2;


    if(g_WaterWheelDemoNumTeeth>8)
        g_WaterWheelDemoNumTeeth=8;

    if(KEYUP(VK_INSERT)&&KEYUP(VK_DELETE)&&(keyPressed==true))
    {
        keyPressed=false;
    }

    if(KEYDOWN(VK_INSERT)||KEYDOWN(VK_DELETE))
    {
        DestroyWaterWheelTPE();
        CreateWaterWheelTPE();
    }
}

void CreateWaterWheel(int APIID)
{
    g_pWaterWheelMusic      =new cSound("data/moonl.mid");
    g_pWaterWheelTexture    =new sTexture("data/plastic.jpg");
    g_pWaterWheelMusic->Play();
    if(APIID==0)
    {
        CreateWaterWheelTPE();
        //CreateStressTest(APIID);
    }
    //    else
    //      CreateWaterWheelODE();
}


void RenderWaterWheel(float dt,int APIID,bool updateOnly)
{
    //RenderStressTest(dt, APIID);
    if(APIID==0)
        RenderWaterWheelTPE(dt,updateOnly);

    //    else
    //      RenderWaterWheelODE(dt);

}
void PushWaterWheel(int APIID)
{
    if(APIID==0)
        PushWaterWheelTPE();
    //PushStressTest(APIID);
    //    else
    //      PushWaterWheelODE();
}
void DestroyWaterWheel(int APIID)
{
    if(APIID==0)
        DestroyWaterWheelTPE();
    //    else
    //      DestroyWaterWheelODE();


    g_pWaterWheelMusic->Stop();
    SAFE_DELETE(g_pWaterWheelMusic);
    SAFE_DELETE(g_pWaterWheelTexture);
}




void DisplayWaterWheelHelp()
{
    TextPrintf(10,30,"press Insert/Delete to change the number of gears' teeth");
    TextPrintf(10,50,"Press Space to drop more balls on the gears");
    TextPrintf(10,70,"Use mouse wheel to zoom in out");
    TextPrintf(10,90,"Hold right mouse button to change camera");
    TextPrintf(10,110,"%d gears used",g_TPEFanBodies.size());
}

