#include "system/engine.h"
#include "system/settings.h"
#include "3d/math/math.h"
#include "3d/meshes/meshes.h"
#include "3d/graphics/graphicssubsystem.h"
#include "3d/3dtypes/3dtypes.h"
#include "3d/bsp/bsp.h"
#include "3d/text/text.h"

//#include "physics/physics.h"
#include "util/framecontrol.h"
#include "input/input.h"
#include "audio/audio.h"
#include "resource.h"
#include <direct.h>

#include "nvphysics.h"
// the demos headers
#include "demos\pendulum.h"
#include "demos\pivot.h"
#include "demos\dominoes.h"
#include "demos\waterwheel.h"
#include "demos\bridge.h"
#include "demos\stress.h"
#include "demos\destruction.h"
#include "demos\test.h"
#include "demos\chain.h"
//#include "demos\car.h"
#include "demos\convex.h"
#include "demos\tpe.h"



#include "3d\meshes\indexmesh.h"


//	the call back function for the about dialog
BOOL CALLBACK AboutCallBack(HWND hwnd,UINT msg,WPARAM wParam,LPARAM lParam)
{
    int ClickId;
    //	switch on the sent message
    switch (msg)
    {
        //	at initialization... set the
    case WM_INITDIALOG:
        break;

    case WM_COMMAND:
        ClickId=LOWORD(wParam);
        if(ClickId==IDOK|| ClickId==IDCANCEL)			EndDialog(hwnd,ClickId);
        break;

    default:
        break;
    }
    return FALSE;
}



// the APIS used in here
#define PHYSICS_API_ID_NV 0x00000000
#define PHYSICS_API_ID_NOVODEX 0x00000001

// number of scenes
#define PHYSICS_SCENE_COUNT             20

// ID for each one of the scene
#define PHYSICS_SCENE_ID_PENDULUM       0x00000000
#define PHYSICS_SCENE_ID_PIVOT          0x00000001
#define PHYSICS_SCENE_ID_DOMINIOES      0x00000002
#define PHYSICS_SCENE_ID_WATERWHELL     0x00000003
#define PHYSICS_SCENE_ID_BRIDGE         0x00000004
#define PHYSICS_SCENE_ID_STRESS         0x00000005
#define PHYSICS_SCENE_ID_DESTRUCTION    0x00000006
#define PHYSICS_SCENE_ID_TEST           0x00000007
#define PHYSICS_SCENE_ID_CHAIN          0x00000008
#define PHYSICS_SCENE_ID_CAR            0x00000009
#define PHYSICS_SCENE_ID_CONVEX         0x0000000A



// global physics API and scene ID
unsigned int g_PhysicsAPIID=PHYSICS_API_ID_NV;
unsigned int g_CurrentSceneID=PHYSICS_SCENE_ID_DOMINIOES;
Vector3 g_CameraTarget(0,0,0);

cCamera g_Camera;
 float g_CameraRadius;

bool g_bLighting=true;
bool g_bTexturing=true;
bool g_bWireFrame=false;
bool g_bNoRendering=false;
bool g_bLockFrameRate=true;



// 4 type definitions
typedef  void (*BeginFunc)(int) ;
typedef  void (*EndFunc)(int) ;
typedef  void (*RenderFunc)(float,int,bool) ;
typedef  void (*InterActFunc)(int) ;
typedef  void (*CameraFunc)(cCamera&) ;
typedef  void (*HelpFunc)(void) ;

// the 4 main functions called everywhere
BeginFunc       CreateScene=NULL;
EndFunc         DestroyScene=NULL;
RenderFunc      RenderScene=NULL;
InterActFunc    InteractScene=NULL;
CameraFunc      CameraScene=NULL;
HelpFunc        SceneHelp=NULL;


// this small structure holds function pointer to a scene
struct sPhysicsScene
{
    // the 4 function for every scene
    BeginFunc       m_CreationFunction;
    EndFunc         m_DestructionFunction;
    RenderFunc      m_RenderingFunction;
    InterActFunc    m_InteractionFunction;
    CameraFunc      m_CameraFunction;
    HelpFunc        m_HelpFunction;

    // make they are all nulls initially
    sPhysicsScene()
    {
        m_CreationFunction      =   NULL;
        m_DestructionFunction   =   NULL;
        m_RenderingFunction     =   NULL;
        m_InteractionFunction   =   NULL;
        m_CameraFunction        =   NULL;
        m_HelpFunction          =   NULL;
    }
};

// this array should hold all the functions initially
sPhysicsScene AllScenes[PHYSICS_SCENE_COUNT];

// init all the scene with this function
void PrepareAllScenes()
{
    // the pendulum functions..
    AllScenes[PHYSICS_SCENE_ID_PENDULUM].m_CreationFunction=CreatePendulum;
    AllScenes[PHYSICS_SCENE_ID_PENDULUM].m_DestructionFunction=DestroyPendulum;
    AllScenes[PHYSICS_SCENE_ID_PENDULUM].m_RenderingFunction=RenderPendulum;
    AllScenes[PHYSICS_SCENE_ID_PENDULUM].m_InteractionFunction=PushPendulum;
    AllScenes[PHYSICS_SCENE_ID_PENDULUM].m_CameraFunction=SetPendulumCamera;
    AllScenes[PHYSICS_SCENE_ID_PENDULUM].m_HelpFunction=DisplayPendulumHelp;

    // the pivot functions...
    AllScenes[PHYSICS_SCENE_ID_PIVOT].m_CreationFunction=CreatePivot;
    AllScenes[PHYSICS_SCENE_ID_PIVOT].m_DestructionFunction=DestroyPivot;
    AllScenes[PHYSICS_SCENE_ID_PIVOT].m_RenderingFunction=RenderPivot;
    AllScenes[PHYSICS_SCENE_ID_PIVOT].m_InteractionFunction=PushPivot;
    AllScenes[PHYSICS_SCENE_ID_PIVOT].m_CameraFunction=SetPivotCamera;
    AllScenes[PHYSICS_SCENE_ID_PIVOT].m_HelpFunction=DisplayPivotHelp;

    // the dominoes functions...
    AllScenes[PHYSICS_SCENE_ID_DOMINIOES].m_CreationFunction=CreateDominoes;
    AllScenes[PHYSICS_SCENE_ID_DOMINIOES].m_DestructionFunction=DestroyDominoes;
    AllScenes[PHYSICS_SCENE_ID_DOMINIOES].m_RenderingFunction=RenderDominoes;
    AllScenes[PHYSICS_SCENE_ID_DOMINIOES].m_InteractionFunction=PushDominoes;
    AllScenes[PHYSICS_SCENE_ID_DOMINIOES].m_CameraFunction=SetDominoCamera;
    AllScenes[PHYSICS_SCENE_ID_DOMINIOES].m_HelpFunction=DisplayDominoesHelp;

    // the water wheel functions...
    AllScenes[PHYSICS_SCENE_ID_WATERWHELL].m_CreationFunction=CreateWaterWheel;
    AllScenes[PHYSICS_SCENE_ID_WATERWHELL].m_DestructionFunction=DestroyWaterWheel;
    AllScenes[PHYSICS_SCENE_ID_WATERWHELL].m_RenderingFunction=RenderWaterWheel;
    AllScenes[PHYSICS_SCENE_ID_WATERWHELL].m_InteractionFunction=PushWaterWheel;
    AllScenes[PHYSICS_SCENE_ID_WATERWHELL].m_CameraFunction=SetWaterWheelCamera;
    AllScenes[PHYSICS_SCENE_ID_WATERWHELL].m_HelpFunction=DisplayWaterWheelHelp;

    // next goes all the scenes...
    AllScenes[PHYSICS_SCENE_ID_BRIDGE].m_CreationFunction=CreateBridge;
    AllScenes[PHYSICS_SCENE_ID_BRIDGE].m_DestructionFunction=DestroyBridge;
    AllScenes[PHYSICS_SCENE_ID_BRIDGE].m_RenderingFunction=RenderBridge;
    AllScenes[PHYSICS_SCENE_ID_BRIDGE].m_InteractionFunction=PushBridge;
    AllScenes[PHYSICS_SCENE_ID_BRIDGE].m_CameraFunction=SetBridgeCamera;
    AllScenes[PHYSICS_SCENE_ID_BRIDGE].m_HelpFunction=DisplayBridgeHelp;

    // next goes all the scenes...
    AllScenes[PHYSICS_SCENE_ID_STRESS].m_CreationFunction=CreateStressTest;
    AllScenes[PHYSICS_SCENE_ID_STRESS].m_DestructionFunction=DestroyStressTest;
    AllScenes[PHYSICS_SCENE_ID_STRESS].m_RenderingFunction=RenderStressTest;
    AllScenes[PHYSICS_SCENE_ID_STRESS].m_InteractionFunction=PushStressTest;
    AllScenes[PHYSICS_SCENE_ID_STRESS].m_CameraFunction=SetStressCamera;
    AllScenes[PHYSICS_SCENE_ID_STRESS].m_HelpFunction=DisplayStressHelp;


    AllScenes[PHYSICS_SCENE_ID_DESTRUCTION].m_CreationFunction=CreateDestructionDemo;
    AllScenes[PHYSICS_SCENE_ID_DESTRUCTION].m_DestructionFunction=DestroyDestructionDemo;
    AllScenes[PHYSICS_SCENE_ID_DESTRUCTION].m_RenderingFunction=RenderDestructionDemo;
    AllScenes[PHYSICS_SCENE_ID_DESTRUCTION].m_InteractionFunction=PushDestructionDemo;
    AllScenes[PHYSICS_SCENE_ID_DESTRUCTION].m_CameraFunction=SetDestructionDemoCamera;
    AllScenes[PHYSICS_SCENE_ID_DESTRUCTION].m_HelpFunction=DisplayDestructionHelp;

    AllScenes[PHYSICS_SCENE_ID_TEST].m_CreationFunction=CreateTest;
    AllScenes[PHYSICS_SCENE_ID_TEST].m_DestructionFunction=DestroyTest;
    AllScenes[PHYSICS_SCENE_ID_TEST].m_RenderingFunction=RenderTest;
    AllScenes[PHYSICS_SCENE_ID_TEST].m_InteractionFunction=PushTest;
    AllScenes[PHYSICS_SCENE_ID_TEST].m_CameraFunction=SetTestCamera;
    AllScenes[PHYSICS_SCENE_ID_TEST].m_HelpFunction=DisplayTestHelp;


    AllScenes[PHYSICS_SCENE_ID_CHAIN].m_CreationFunction=CreateChainDemo;
    AllScenes[PHYSICS_SCENE_ID_CHAIN].m_DestructionFunction=DestroyChainDemo;
    AllScenes[PHYSICS_SCENE_ID_CHAIN].m_RenderingFunction=RenderChainDemo;
    AllScenes[PHYSICS_SCENE_ID_CHAIN].m_InteractionFunction=PushChainDemo;
    AllScenes[PHYSICS_SCENE_ID_CHAIN].m_CameraFunction=SetChainDemoCamera;
    AllScenes[PHYSICS_SCENE_ID_CHAIN].m_HelpFunction=DisplayChainHelp;


    //AllScenes[PHYSICS_SCENE_ID_CAR].m_CreationFunction=CreateCarDemo;
    //AllScenes[PHYSICS_SCENE_ID_CAR].m_DestructionFunction=DestroyCarDemo;
    //AllScenes[PHYSICS_SCENE_ID_CAR].m_RenderingFunction=RenderCarDemo;
    //AllScenes[PHYSICS_SCENE_ID_CAR].m_InteractionFunction=PushCarDemo;
    //AllScenes[PHYSICS_SCENE_ID_CAR].m_CameraFunction=SetCarDemoCamera;
    //AllScenes[PHYSICS_SCENE_ID_CAR].m_HelpFunction=DisplayCarDemoHelp;


    AllScenes[PHYSICS_SCENE_ID_CONVEX].m_CreationFunction=CreateConvexDemo;
    AllScenes[PHYSICS_SCENE_ID_CONVEX].m_DestructionFunction=DestroyConvexDemo;
    AllScenes[PHYSICS_SCENE_ID_CONVEX].m_RenderingFunction=RenderConvexDemo;
    AllScenes[PHYSICS_SCENE_ID_CONVEX].m_InteractionFunction=PushConvexDemo;
    AllScenes[PHYSICS_SCENE_ID_CONVEX].m_CameraFunction=SetConvexDemoCamera;
    AllScenes[PHYSICS_SCENE_ID_CONVEX].m_HelpFunction=DisplayConvexDemoHelp;
}

// called when needed to change the current scene
void SetScene(unsigned int newSceneID)
{
    // now destroy the old one
    if(DestroyScene)
        DestroyScene(g_PhysicsAPIID);

    SAFE_DELETE(Audio());
    Audio()         =   new cAudioSubSystem();


    // reassign all the functions again..
    CreateScene     =   AllScenes[newSceneID].m_CreationFunction;
    DestroyScene    =   AllScenes[newSceneID].m_DestructionFunction;
    RenderScene     =   AllScenes[newSceneID].m_RenderingFunction;
    InteractScene   =   AllScenes[newSceneID].m_InteractionFunction;
    CameraScene     =   AllScenes[newSceneID].m_CameraFunction;
    SceneHelp       =   AllScenes[newSceneID].m_HelpFunction;

    // save the scene ID
    g_CurrentSceneID    =   newSceneID;

    g_CameraTarget.Assign(0,0,0);

    g_Camera.m_Loc=Vector3(0,0,100);
    g_Camera.m_At=Vector3(0,0,0);

    g_Camera.m_Pitch=-PI/4.0f;
    g_Camera.m_Yaw=-PI/4.0f;


    if(newSceneID==PHYSICS_SCENE_ID_DOMINIOES)
        g_Camera.m_Yaw=PI/2.0f;

    g_CameraRadius=90.0f;
    if(CreateScene)
        CreateScene(g_PhysicsAPIID);

}


// switch the physics API with this
void SwitchAPI(unsigned int physicsAPIID)
{
    if(physicsAPIID==g_PhysicsAPIID)
        return;

    if(DestroyScene&&CreateScene)
    {
        if(DestroyScene)
            DestroyScene(g_PhysicsAPIID);
        g_PhysicsAPIID=physicsAPIID;
        if(CreateScene)
            CreateScene(g_PhysicsAPIID);
    }
}


/****************************************************
* main.cpp											*
*main application specific code should rely in here	*
****************************************************/

// handle the commands in the menu from here
// all demos can be called from here
void HandleMenuCommand(WPARAM param)
{
    int hiWord=HIWORD(param);
    int lowWord=LOWORD(param);

    static int lastCheckedID=ID_DOMINOES_SCENE1;

    CheckMenuItem(GetMenu(Graphics()->GetMainHWND()),lastCheckedID,MF_UNCHECKED);
    CheckMenuItem(GetMenu(Graphics()->GetMainHWND()),lowWord,MF_CHECKED);
    lastCheckedID=lowWord;

    switch(lowWord)
    {
    case ID_DEMO_PENDULUM:
        SetScene(PHYSICS_SCENE_ID_PENDULUM);

        break;
    case ID_DEMO_PIVOT:
        SetScene(PHYSICS_SCENE_ID_PIVOT);
        break;
    case ID_DOMINOES_SCENE1:
        SetDominoScene(1);
        SetScene(PHYSICS_SCENE_ID_DOMINIOES);
        break;
    case ID_DOMINOES_SCENE2:
        SetDominoScene(2);
        SetScene(PHYSICS_SCENE_ID_DOMINIOES);
        break;
    case ID_DOMINOES_SCENE3:
        SetDominoScene(3);
        SetScene(PHYSICS_SCENE_ID_DOMINIOES);
        break;
    case ID_DOMINOES_SCENE4:
        SetDominoScene(4);
        SetScene(PHYSICS_SCENE_ID_DOMINIOES);
        break;

    case ID_DESTRUCTION_SCENE1:
        SetDestructionDemoScene(1);
        SetScene(PHYSICS_SCENE_ID_DESTRUCTION);
        break;
    case ID_DESTRUCTION_SCENE2:
        SetDestructionDemoScene(2);
        SetScene(PHYSICS_SCENE_ID_DESTRUCTION);
        break;
    case ID_DESTRUCTION_SCENE3:
        SetDestructionDemoScene(3);
        SetScene(PHYSICS_SCENE_ID_DESTRUCTION);
        break;
    case ID_DESTRUCTION_SCENE4:
        SetDestructionDemoScene(4);
        SetScene(PHYSICS_SCENE_ID_DESTRUCTION);
        break;
    case ID_DESTRUCTION_SCENE5:
        SetDestructionDemoScene(5);
        SetScene(PHYSICS_SCENE_ID_DESTRUCTION);
        break;

    case ID_DEMO_WATERWHEEL:
        SetScene(PHYSICS_SCENE_ID_WATERWHELL);
        break;
    case ID_DEMO_UNROLLINGTHEBRIDGE:
        SetScene(PHYSICS_SCENE_ID_BRIDGE);
        break;
    case ID_DEMO_STRESSTEST:
        SetScene(PHYSICS_SCENE_ID_STRESS);
        break;
    case ID_DEMO_CAR:
        SetScene(PHYSICS_SCENE_ID_CAR);
        break;
    case ID_DEMO_CHAIN:
        SetScene(PHYSICS_SCENE_ID_CHAIN);
        break;
    case ID_DEMO_CONVEX:
        SetScene(PHYSICS_SCENE_ID_CONVEX);
        break;
    case ID_DEMO_SQUASH:
        SetScene(PHYSICS_SCENE_ID_TEST);
        break;

    case ID_OPTIONS_LIGHTING:
        g_bLighting=!g_bLighting;
        if(g_bLighting)
            Graphics()->EnableLighting();
        else
            Graphics()->DisableLighting();
        break;
    case ID_OPTIONS_TEXTURING:
        g_bTexturing=!g_bTexturing;
        if(g_bTexturing)
            Graphics()->EnableTexturing();
        else
            Graphics()->DisableTexturing();
        break;

    case ID_OPTIONS_LOCKFRAMERATE:
        g_bLockFrameRate=!g_bLockFrameRate;
        break;


    case ID_OPTIONS_WIREFRAME:
        g_bWireFrame=!g_bWireFrame;
        if(g_bWireFrame)
            Graphics()->EnableWireFrame();
        else
            Graphics()->DisableWireFrame();
        break;

    case ID_OPTIONS_NORENDERING:
        g_bNoRendering=!g_bNoRendering;
        break;


    case ID_DEMO_EXIT:
        System()->IsAlive=false;
        break;
    case ID_API_NV:
        CheckMenuItem(GetMenu(Graphics()->GetMainHWND()),ID_API_NV,MF_CHECKED);
        CheckMenuItem(GetMenu(Graphics()->GetMainHWND()),ID_API_NOVODEX,MF_UNCHECKED);
        SwitchAPI(PHYSICS_API_ID_NV);

        break;

    case ID_API_NOVODEX:
        CheckMenuItem(GetMenu(Graphics()->GetMainHWND()),ID_API_NV,MF_UNCHECKED);
        CheckMenuItem(GetMenu(Graphics()->GetMainHWND()),ID_API_NOVODEX,MF_CHECKED);
        SwitchAPI(PHYSICS_API_ID_NOVODEX);
        break;
    case ID_HELP_ABOUT:
        DialogBox(GetModuleHandle(NULL),MAKEINTRESOURCE(IDD_ABOUT),NULL,AboutCallBack);
        //MessageBox(NULL,"Should say something now","Yes!",MB_OK);
        break;
    }
}

// the message handler for the main window to be created and used for rendering
LRESULT CALLBACK cTwisterEngine::MessageHandler(
    HWND	hWnd,	// Handle For This Window
    UINT	uMsg,					// Message For This Window
    WPARAM	wParam,					// Additional Message Information
    LPARAM	lParam
    )
    // Additional Message Information
{
    // switch on message
    switch(uMsg)
    {
        // handle the menu commands in here
    case WM_COMMAND:
        
        HandleMenuCommand(wParam);

        case WM_ACTIVATE:

        if(g_bLighting)
        {
            CheckMenuItem(GetMenu(Graphics()->GetMainHWND()),ID_OPTIONS_LIGHTING,MF_CHECKED);
        }
        else
        {
            CheckMenuItem(GetMenu(Graphics()->GetMainHWND()),ID_OPTIONS_LIGHTING,MF_UNCHECKED);
        }
        if(g_bTexturing)
        {
            CheckMenuItem(GetMenu(Graphics()->GetMainHWND()),ID_OPTIONS_TEXTURING,MF_CHECKED);
        }
        else
        {
            CheckMenuItem(GetMenu(Graphics()->GetMainHWND()),ID_OPTIONS_TEXTURING,MF_UNCHECKED);
        }

        if(g_bWireFrame)
        {
            CheckMenuItem(GetMenu(Graphics()->GetMainHWND()),ID_OPTIONS_WIREFRAME,MF_CHECKED);
        }
        else
        {
            CheckMenuItem(GetMenu(Graphics()->GetMainHWND()),ID_OPTIONS_WIREFRAME,MF_UNCHECKED);
        }
        if(g_bNoRendering)
        {
            CheckMenuItem(GetMenu(Graphics()->GetMainHWND()),ID_OPTIONS_NORENDERING,MF_CHECKED);
        }
        else
        {
            CheckMenuItem(GetMenu(Graphics()->GetMainHWND()),ID_OPTIONS_NORENDERING,MF_UNCHECKED);
        }
        if(g_bLockFrameRate)
        {
            CheckMenuItem(GetMenu(Graphics()->GetMainHWND()),ID_OPTIONS_LOCKFRAMERATE,MF_CHECKED);
        }
        else
        {
            CheckMenuItem(GetMenu(Graphics()->GetMainHWND()),ID_OPTIONS_LOCKFRAMERATE,MF_UNCHECKED);
        }

        break;

    default:
        break;
    }
    return DefWindowProc(hWnd,uMsg,wParam,lParam);
}


//	this will Initialize the engine, i can use a certain handle to device context or not
void cTwisterEngine::Initialize(void * pInitData)
{
    //	Create the mouse pointer object
    Mouse()			=	new sMouse();
    //	Create the graphics sub system
    if(m_GraphicsDriver==TWISTER_GRAHICSDRIVER_DIRECTX)
        Graphics()		=	 NewGraphicsDX();
    else
        Graphics()		=	 NewGraphicsGL();

    //	make the window, if no dc was sent
    if(!pInitData)
        Graphics()->MakeWindow(DisplayMode);
    Graphics()->Initialize(DisplayMode);

    // init the audio subsystem
    Audio()         =   new cAudioSubSystem();
    Input()			=	new cInputSubSystem();
    TextInitialize();
}

//	this will shutdown the engine
void cTwisterEngine::ShutDown()
{
    TextShutDown();
    SAFE_DELETE(Input());
    SAFE_DELETE(Audio());
    SAFE_DELETE(Graphics());
    SAFE_DELETE(Mouse());
}

void RenderLogo()
{
    float totalWidth=(float)Graphics()->m_Width;
    float totalHeight=(float)Graphics()->m_Height;

    float width=totalWidth/4.0f;
    float height=totalHeight/6.0f;

    Graphics()->EnableBlending();
    Graphics()->PushMatrix();
    Graphics()->Translate(totalWidth-width,totalHeight-height,0);

    sMesh mesh;
    mesh.m_VertexFormat=MESH_VERTEXFORMAT_POSITION|MESH_VERTEXFORMAT_TEXCOORD0;
    mesh.m_RenderMode=MESH_RENDERMODE_TRIANGLESTRIP;

    float Vertex[5];
    Vertex[2]=0;    // z is always zero


    Vertex[0]=0;
    Vertex[1]=height;
    Vertex[3]=0;
    Vertex[4]=1;
    mesh.AddVertex(Vertex);

    Vertex[0]=width;
    Vertex[1]=height;
    Vertex[3]=1;
    Vertex[4]=1;
    mesh.AddVertex(Vertex);

    Vertex[0]=0;
    Vertex[1]=0;
    Vertex[3]=0;
    Vertex[4]=0;
    mesh.AddVertex(Vertex);

    Vertex[0]=width;
    Vertex[1]=0;
    Vertex[3]=1;
    Vertex[4]=0;
    mesh.AddVertex(Vertex);

    Vertex[0]=width;
    Vertex[1]=0;
    Vertex[3]=1;
    Vertex[4]=0;
    mesh.AddVertex(Vertex);

    Vertex[0]=0;
    Vertex[1]=0;
    Vertex[3]=0;
    Vertex[4]=0;
    mesh.AddVertex(Vertex);

    Vertex[0]=width;
    Vertex[1]=height;
    Vertex[3]=1;
    Vertex[4]=1;
    mesh.AddVertex(Vertex);

    Vertex[0]=0;
    Vertex[1]=height;
    Vertex[3]=0;
    Vertex[4]=1;
    mesh.AddVertex(Vertex);

    mesh.Compile();
    mesh.Render();

    Graphics()->PopMatrix();
  
    Graphics()->DisableBlending();
}



// an example for bsp
void cTwisterEngine::Main()
{
    g_CameraRadius=10.0f;
    CheckMenuItem(GetMenu(Graphics()->GetMainHWND()),ID_API_NV,MF_CHECKED);
    CheckMenuItem(GetMenu(Graphics()->GetMainHWND()),ID_DOMINOES_SCENE1,MF_CHECKED);
    CheckMenuItem(GetMenu(Graphics()->GetMainHWND()),ID_OPTIONS_TEXTURING,MF_CHECKED);
    CheckMenuItem(GetMenu(Graphics()->GetMainHWND()),ID_OPTIONS_LIGHTING,MF_CHECKED);
    CheckMenuItem(GetMenu(Graphics()->GetMainHWND()),ID_OPTIONS_LOCKFRAMERATE,MF_CHECKED);

    Graphics()->m_pWindow->SetLabel("nV Physics Demos");
    sTexture* pLogoTexture=new sTexture("data/logo.tga");

    cFrameController frameController;
    int FrameCount=0;
    frameController.setFps(80);

    Material material;
    material.SetID(MATERIAL_ID_BRUSHED_METAL);

    DirectionalLight light1(Vector3(1,1,1));
    DirectionalLight light2(Vector3(1,-1,-1));
    material.SetAmbient(Color4(0.3f,0.3f,0.3f,0.3f));
    light2.Activate();
//    light2.Ambient=ColorValue(Color4(0.6f,0.3f,0.5f,0.3f));

    Matrix4 bodyMatrix;
    bodyMatrix.MakeIdent();

    // this will setup all function pointers
    PrepareAllScenes();

    // set the first scene
    SetScene(PHYSICS_SCENE_ID_DOMINIOES);
    float deltaTime=0.01f;
    bool DisplayHelp=true;
    frameController.setFps(60);
    Graphics()->EnableLighting();


    while(!ProcessMessages())
    {
        if(frameController.isReady()||(g_bLockFrameRate==false))
        {
            // catch DirectInput here...
            if(Active)
                Input()->catchAll();
            char c=TextGetKey();

            if(c=='h')
                DisplayHelp=!DisplayHelp;

            // interact with the scene
            if(InteractScene)
                InteractScene(g_PhysicsAPIID);

            if(Mouse()->m_bRButtonDown)
            {
                g_Camera.RotateLocal(-Mouse()->m_DeltaX/100.0f,-Mouse()->m_DeltaY/100.0f,0);
                
                // restrict the camera angles
                if(g_Camera.m_Pitch>-0.1f)
                g_Camera.m_Pitch=-0.1f;
                if(g_Camera.m_Pitch<-PI/2.0f)
                g_Camera.m_Pitch=-PI/2.0f;
                
                g_Camera.ComputeInfo();
            }
            g_Camera.ComputeInfo();

            if(Active)
                g_CameraRadius+=Mouse()->m_DeltaZ/25.0f;

            if(g_CameraRadius<1.0f)
                g_CameraRadius=1.0f;

            g_Camera.m_At=g_CameraTarget;
            if(g_CurrentSceneID==PHYSICS_SCENE_ID_DOMINIOES)
            {
                g_Camera.m_At=GetDominoDemoBallPosition();
            }
            g_Camera.m_Loc=g_Camera.m_At-g_Camera.m_Fwd*g_CameraRadius;


            Graphics()->Clear();
            Graphics()->BeginScene();


            g_Camera.Activate();
            CameraScene(g_Camera);

            Graphics()->CalculateFrustum();
            Graphics()->BeginScene();

            light1.SetDirection(g_Camera.m_Fwd);
            light1.Activate();
            light2.SetDirection(-g_Camera.m_Fwd);
            light2.Activate();

            material.Activate();

            if(RenderScene)
                RenderScene(deltaTime,g_PhysicsAPIID,g_bNoRendering);


            Graphics()->Begin2D();

            Graphics()->DisableLighting();
            Graphics()->EnableBlending();
            Graphics()->EnableTexturing();
            Graphics()->DisableWireFrame();
            Graphics()->DisableDepthTest();
            TextPrintf(10,10,"%d",System()->CalculateFPS());


            
//            TextPrintf(10,200,"%d iterations",GetNumIterations());
  //          TextPrintf(10,220,"%f",GetLastTimeStep());
            
            SceneHelp();


            pLogoTexture->Activate();
            RenderLogo();
            pLogoTexture->Deactivate();
            Graphics()->DisableBlending();
            Graphics()->EnableDepthTest();
            Graphics()->End2D();
            if(g_bLighting)            Graphics()->EnableLighting();
            if(!g_bTexturing)          Graphics()->DisableTexturing();
            if(g_bWireFrame)           Graphics()->EnableWireFrame();
            Graphics()->EndScene();
            Graphics()->SwapBuffers();
            frameController.update();


        }
        //	escape ends the loop
        if(KEYDOWN(VK_ESCAPE))			break;
    }
    if(DestroyScene)
        DestroyScene(g_PhysicsAPIID);

    SAFE_DELETE(pLogoTexture);
}
