#ifndef _BEZIER_H_
#define _BEZIER_H_
/********************************************************
*	bezier.h											*
*	Header file for the bezier class					*
*	used to create curves and surfaces					*
********************************************************/

#include "../math/math.h"

/********************************************************
*	class cQuadraticCurve								*
*	used to create quadric bezier curves				*
*	3 control points									*
*	each point is an array of floats which can be		*
*	of any size to allow this class to be compatible	*
*	withaany type of vertices							*
********************************************************/
class cQuadraticCurve
{
private:

	int		m_NumSteps	;
	int		m_CurStep	;
	int		m_Size;

	//	the three control arrays
	float*	m_pControlVector1;
	float*	m_pControlVector2;
	float*	m_pControlVector3;


	float*	m_pQ		;			//	the current step in iteration
	float*	m_pdQ		;			//	pointer to delta Q
	float*	m_pddQ		;			//	pointer to delta of delta Q


public:


	//	constructor..
	cQuadraticCurve();

	//	destructor gets rid of the data
	~cQuadraticCurve();


	//	setting up the curve
	void Setup(int numSteps,int VectorSize,float* p1,float*	p2,float*p3);

	//	this will get to the next point on the curve
	void Next();
	//	call this to start the curve
	void Start();

	//	this will return true when the curve is finished
	bool Done()
	{	return !(m_CurStep<m_NumSteps);	}

	//	get current location on the curve
	float* GetCurr()
	{	return m_pQ;	}
};
/*-------end of class cQuadraticCurve  declaration------*/


/********************************************************
*	class cQuadraticPatch								*
*	used to create patches controlled by 9 control point*
********************************************************/
class cQuadraticPatch
{
private	:

	//	9 control vectors...(the dimensions can be anything
	float	*m_pControlPoints[9];
	int		m_Size		;		//	size of the arrays
	int		m_NumSteps	;
	int		m_CurStep	;

	//	3 control curves
	cQuadraticCurve	m_Curve1;
	cQuadraticCurve	m_Curve2;
	cQuadraticCurve	m_Curve3;

public	:

	//	the vertexes of the patch
	vector<float*> VertexData;

	//	constructor..
	cQuadraticPatch();

	//	destructor
	~cQuadraticPatch();

	//	using the 9 points..the surface is initialized
	void Initialize(int m_NumSteps,int ArraySize,float* p0,float* p1,float* p2,float* p3,float* p4,float* p5,float* p6,float* p7,float* p8);

	//	this will tesselate the surface
	void Tesselate();

	//	this static function used to count the number of indices for a certain quadratic patch
	static int CountIndices(int Smoothness);

	// this creates indexes for a triangle list on the internal points
	void CreateTriangleListIndexes(vector<unsigned short>& indexes);

	// this creates indexes for a triangle list on the internal points
	void CreateTriangleStripIndexes(vector<unsigned short>& indexes);
};
/*--------end of class cQuadraticPatch declaration-----*/
#endif // _BEZIER_H_
