#ifndef _CAMERA_H_
#define _CAMERA_H_
/********************************************************
*	camera.h											*
*	header file for the camera class					*
********************************************************/
#include "../math/vector3.h"

/********************************************************
*	class cCamera declaration							*
********************************************************/
class cCamera
{
public:


	//	the camera vectors
	Vector3 m_Loc;				//	location of the viewer
	Vector3 m_At;				//	what the viewer looks at..
	Vector3 m_Up;				//	world up vector

	Vector3 m_Fwd;				// Forward (Vector) 
	Vector3 m_Side;				// Side (to the right)


	// Orientation data
	// Yaw, Pitch, and Roll
	float	m_Yaw		;		//	orientation in the direction of nodding your head (no)
	float	m_Pitch		;		//	orientation in the direction of nodding your head (yes)
	float	m_Roll		;		//	the last orientation ..


	//	make a camera with a location and euler angles
	cCamera(Vector3& loc,float y,float p,float r)
	{	Init(loc,y,p,r);	}

	//	create an abstarct camera
	cCamera()
	{	Init(Vector3(0,0,0),0,0,0);	}

	//	initialize the data of the camera..the location and the angles
	void Init(Vector3 Loc,float Yaw,float Pitch,float Roll);

	//	compute the camera info accoring to the angles and the location
	void ComputeInfo();

	//	calling this will make the activate the camera
	void Activate()	;
	
	//	camera control functions..

	
	//	move the camera..relative to the world coordinates
	void Translate(float x,float y,float z);

	//	move relative to viewer point...
	void TranslateLocal(float x,float y,float z);

	//	this will rotate locally..
	void RotateLocal(float yaw,float pitch,float roll);
};
/*------------end of class cCamera declaration---------*/


#endif // _CAMERA_H_