#ifndef _CROSS_SECTION_H_
#define _CROSS_SECTION_H_

/********************************************************
*	crosssection.h										*
*	header declaration file for the cross section class	*
********************************************************/

#include "../math/math.h"	// i need the vector 2 here
#include <vector>
using std::vector;

// shapes possible for cross sections
#define CROSS_SECTION_SHAPE_UNDEFINED	0x00000000
#define CROSS_SECTION_SHAPE_I			0x00000001
#define CROSS_SECTION_SHAPE_H			0x00000002
#define CROSS_SECTION_SHAPE_RECT		0x00000004
#define CROSS_SECTION_SHAPE_ROUNDRECT	0x00000008
#define CROSS_SECTION_SHAPE_Z			0x00000010
#define CROSS_SECTION_SHAPE_C			0x00000020

/********************************************************
*	class CCrossSection delcration						*
*	this class repsents a cross section of a 3d			*
*	objects, defined in the x-y plane					*
*	using the mesh processors, cross sections could be	*
*	extruded into 3d meshes								*
********************************************************/
class CCrossSection
{
public:


	// the vertices of the cross section,
	// all vertices are assumed to be continuous, the last vertex is NOT connected to the first one
	// (this is not a polygon)
	vector<Vector2> m_Points;

	// the general shape of a cross section
	// could be I,H,RECT,Z,C..anything we need
	unsigned int m_Shape;

	// empty constrcutor
	CCrossSection();

	// empty destructor
	virtual ~CCrossSection();


	// use this to mirror it around the Y axis
	void MirrorVertically();
	// use this to mirror it around the X axis
	void MirrorHorizontally();

};
/*---------end of class CCrossSection declaration------*/
#endif // _CROSS_SECTION_H_