#ifndef _DIRECTIONAL_LIGHT_H_
#define _DIRECTIONAL_LIGHT_H_
/********************************************************
*	directlight.h										*
*	Header file for the directional light				*
********************************************************/
#include "../math/math.h"
#include "../hybrid/meshcompiler.h"
/********************************************************
*	struct sDirectionalLight declaration				*
********************************************************/
struct DirectionalLight
{
private:

    sInternalLightDirectional* m_pInternalLight;

public:

	//	constructor of the light, sets all the components
	DirectionalLight(
						 Vector3& dir,
						 Color4&	diff	=	Color4(1,1,1,1),
						 Color4&	spec	=	Color4(0,0,0,0),
						 Color4&	amb		=	Color4(0,0,0,0)
						);


    ~DirectionalLight()
    {
        Deactivate();
        SAFE_DELETE(m_pInternalLight);
    }

    void Activate()
    {        m_pInternalLight->Activate();    }
    void Deactivate()
    {        m_pInternalLight->Deactivate();    }
    void SetDiffuse(Color4& color)
    {        m_pInternalLight->SetDiffuse(color.c);    }
    void SetAmbient(Color4& color)
    {        m_pInternalLight->SetAmbient(color.c);    }
    void SetSpecular(Color4& color)
    {        m_pInternalLight->SetSpecular(color.c);    }
    void SetDirection(Vector3& Direction)
    {        m_pInternalLight->SetDirection(Direction.v);}
    void SetDirection(float* pDirection)
    {        m_pInternalLight->SetDirection(pDirection);}
};
/*-----------end of sDirectionalLight declaration------*/
#endif // _DIRECTIONAL_LIGHT_H_