#ifndef _TWISTER_MATERIAL_H_
#define _TWISTER_MATERIAL_H_
#include "../math/math.h"

/********************************************************
*	material.h											*
*	By Shady El Mously									*
*	part of the twister 3d engine						*
*	declaration file for the material class				*
********************************************************/

// predefined standard materials to be used
#define MATERIAL_ID_DEFAULT             0x00000000
#define MATERIAL_ID_AQUA_GLAZE          0x00000001
#define MATERIAL_ID_AQUA_GLAZE_METAL    0x00000002
#define MATERIAL_ID_BARREL_BLUE         0x00000003
#define MATERIAL_ID_BEIGE_MATTE         0x00000004
#define MATERIAL_ID_BEIGE_PLASTIC       0x00000005
#define MATERIAL_ID_BLACK_MATTE         0x00000006
#define MATERIAL_ID_BLACK_PLASTIC       0x00000007
#define MATERIAL_ID_BLUE_BUMPY_CLOTH    0x00000008
#define MATERIAL_ID_BLUE_METAL_SKY      0x00000009
#define MATERIAL_ID_BLUE_PLASTIC        0x0000000A
#define MATERIAL_ID_BRASS               0x0000000B
#define MATERIAL_ID_BRUSHED_METAL       0x0000000C
#define MATERIAL_ID_CEILING_TILE        0x0000000D
#define MATERIAL_ID_CHERRY_RED_METAL    0x0000000E
#define MATERIAL_ID_CHROME_BLUE_SKY     0x0000000F
#define MATERIAL_ID_CHROME_MOROCO       0x00000010
#define MATERIAL_ID_CHROME_PEARL        0x00000011
#define MATERIAL_ID_SKY_BLUE_METAL      0x00000012



#include "../hybrid/meshcompiler.h"

/********************************************************
*	class material										*
*	By	Shady El Mously									*
*	it is used to represent a material of an object		*
*	when activated all rendering will use it			*
********************************************************/
struct Material 
{
private:

    sInternalMaterial* m_pInternalMaterial;
	bool	m_bIsActive;		//	true when the material is active

public:
	//	default constructor
	//	a white material
	Material(
		Color4& amb		=	Color4(1.0f,1.0f,1.0f,1.0f),
		Color4& diff	=	Color4(1.0f,1.0f,1.0f,1.0f),
		Color4& spec	=	Color4(1.0f,1.0f,1.0f,1.0f),		
		Color4& emit	=	Color4(0.0f,0.0f,0.0f,0.0f),
		float pow=1)
	{
        m_pInternalMaterial=MeshCompiler()->CreateInternalMaterial();

		Set(amb,diff,spec,emit,pow);
		m_bIsActive	=	false;
	}

	//	destructor	deactivated the material
	~Material()
	{
        delete m_pInternalMaterial;
	}

	// changing the material
	void Set(
		Color4& amb		=	Color4(1.0f,1.0f,1.0f,1.0f),
		Color4& diff	=	Color4(1.0f,1.0f,1.0f,1.0f),
		Color4& spec	=	Color4(1.0f,1.0f,1.0f,1.0f),
		Color4& emit	=	Color4(0.0f,0.0f,0.0f,0.0f),
		float pow=1);

	// make the material the active one
	void Activate()
    {        m_pInternalMaterial->Activate();    }

    //	disable the material
    void Deactivate()
    {        m_pInternalMaterial->Deactivate();    }

    void SetDiffuse(Color4& color)
    {        m_pInternalMaterial->SetDiffuse(color.c);    }
    void SetAmbient(Color4& color)
    {        m_pInternalMaterial->SetAmbient(color.c);    }
    void SetSpecular(Color4& color)
    {        m_pInternalMaterial->SetSpecular(color.c);    }
    void SetEmissive(Color4& color)
    {        m_pInternalMaterial->SetEmissive(color.c);    }


    // set the material using a predefined ID
    void SetID(int materialID);

};
/*--------end of class material declaration--------------------*/
#endif // _TWISTER_MATERIAL_H_