#ifndef _POLYGON_2D_H_
#define _POLYGON_2D_H_

/************************************************************
*	polygon2d.h												*
*	By Shady El Mously										*
*	declaration file for the polygon2d class				*
*	which represents the 2d projection of a 3d object		*
************************************************************/

#include "crosssection.h"		//	 the base class  

/************************************************************
*	class CPolygon2D declaration							*
*	the polygon is a 2D containter containg					*
*	a coplanar polygon describing a certain object			*
*	from a projections, it might be in the x-y plane in		*
*	some cases or in the x-z plane in other cases			*
************************************************************/
class cPolygon2D : public CCrossSection
{
public:

	// default constructor()
	cPolygon2D();

	// destructor..
	virtual ~cPolygon2D();

	// get a segment using its index
	void GetSegment(Segment& seg,int index);

	// get the length of a side
	float GetSideLength(int index);

	// get the angle of a croner
	float GetCornerAngle(int index);

	// this cuts a steel polygon... using a plane, only leaves back what is in its front
	void Cut(Plane& cuttingPlane);

	// a simpler cut function..
	void Cut(float x1,float y1,float x2,float y2,int side);

	// this makes it a rectangular plate
	void CreateRectangle(float sizeX,float sizeY);

	// given a side index, we update the points to be vertices there
	void MakeSideVertical(int sideIndex);

	// apply a trnasformation to the polygon, get it in 2D again...
	void Transform(const Matrix4& matrix);

	// use this to generate a regular polygon, with number of sides, and their lengths, 
	// minimum is 2
	void CreateRegularPolygon(float Radius,unsigned int numSides,bool removeLast=false);
	
	// use this to generate a regular polygon, with number of sides, and their lengths, 
	// minimum is 2
	void AddRegularPolygon(float Radius,unsigned int numSides,Vector2& Center=Vector2(0,0),bool removeLast=false);

	// use this to generate a regular polygon, cutted from edges internally creating a star
	//	with number of sides, and their lengths, and the cutting length
	// minimum is 2
	void AddRegularCuttedPolygon(float Radius,unsigned int numSides,float cutLength,bool removeLast=false);

	// create a bezier curve, controlled by 3 points, given the 3 points and the smoothness required
	void AddCurve(Vector3* controlPoints,unsigned int smoothness);

	// add a curve
	void AddCurve(Vector2& p1,Vector2& p2,Vector2& p3,unsigned int smoothness);

	// add a curve using a vector of points..
	void AddCurve(vector<Vector2>& controlPoints,unsigned int smoothness);

	// add a single point
	void AddPoint(float x,float y);

	// create thick projection from a polygon and a thickness
	void CreateThickOutline(float thickness,vector<Vector2>& controlPoints,int align=0,bool edgeThickness=true);

	// does the same with the local data...
	void CreateThickOutline(float thickness);


};
/*----------end of class CSteelPolygon declaration---------*/
#endif	// _POLYGON_2D_H_