#ifndef _POLYGON_TESSELATOR_H_
#define _POLYGON_TESSELATOR_H_


/********************************************************
*	tesselator.h										*
*	declaration file for the tesselator class			*
*	this class is useful in making complex operations	*
*	of polygons											*
********************************************************/

#include "../../util/singleton.h"	//	for singleton
#include "../math/math.h"
#include "../meshes/mesh.h"

// some define for winding
#define TESSELATOR_WINDING_CW	0x00000001
#define TESSELATOR_WINDING_CCW	0x00000002


/********************************************************
*	class cTesselator declaration						*
*	this class mainly encapsulated a glu tessellation	*
*	and uses it for many polygon related useful			*
*	functionalities										*
********************************************************/
class cTesselator : public singleton<cTesselator>
{
private:

	// call this to render a contour
	void RenderContour(float* obj_data, int num_vertices);

	// call this when beginning a polygon
	void BeginPolygon(void);

	// call this when finishing a polygon
	void EndPolygon(void);

	// call this to beging a contour inside a polygon
	void BeginContour(void);

	// call this to end a contour inside a polygon
	void EndContour(void);




public:



	// constructor and destructor
	cTesselator();

	// tesselator destructor
	virtual ~cTesselator();

	// by sending  vector of vertices, which is a set of triangles, this function adjusts the winding in a direction 
	// normal to the vector sent, clock wise or counter clock wise
	void AdjustTrianglesWinding(vector<Vector3>& Vertices,Vector3& normal,int direction);

	
	// during polygon tesselation sequences, this vector is filled by pointers to indexed or simple meshes
	vector<sMesh*>*	m_pMeshSet;

	// tesselation methods..

	// set the rule used for winding, the rule GLU_TESS_WINDING_ODD or GLU_TESS_WINDING_POSITIVE are common
	void SetWindingRule(unsigned int winding_rule);

	// use this to know if need outlines or something else
	void SetOutlineMode(unsigned int winding_rule);

	// set the tesselation normal
	void SetNormal(Vector3& normal);

	// this function receives a mesh of vertices, get all the triangles insde it
	// and insert the vertices of those triangles one by one in MeshToFill
	// the vertx size determines how many components are there in each vertex (3,5,6,8)
	// the void pointer (Mesh to fill) is a pointer to a vectors of vertices
	// according to the vertex size, this function will work like this
	void TriangulizePolygon(Vector3* pVertexData,int numverts,vector<Vector3>& MeshToFill);

	// triangulizes a mesh...the render mode plays here
	void TriangulizeMesh(sMesh* pMesh,vector<Vector3>& MeshToFill);

	// this will compute the difference between two sets of triangles, 
	// the first set is the posistive triangles, the second is the negaive set which will be subtrcated
	// all triangles should be coplanar, and the normal to their plane is sent here
	// before calling this function, i should have set the pointer tothe mesh set
	// to get he result from there
	void ComputeDifference(vector<Vector3>& PTriangles,vector<Vector3>& NTriangles,Vector3& norm);

	// this will compute the intersection meshes between 2 sets of triangles
	void ComputeIntersection(vector<Vector3>& PTriangles,vector<Vector3>& NTriangles,Vector3& norm);

	// this will compute the union meshes between 2 sets of triangles
	void ComputeUnion(vector<Vector3>& PTriangles,vector<Vector3>& NTriangles);

	// this will compute the union of one set of triangles
	void ComputeUnion(vector<Vector3>& Triangles);
};
/*-----------end of class CTesselator delcaration------*/

// global access to the main tesselator object
inline cTesselator	*&	Tesselator(void){return cTesselator::GetSingletonPtr();}

#endif // _POLYGON_TESSELATOR_H_