#ifndef _BITSET_H_
#define _BITSET_H_
/********************************************************
*	bitset.h											*
*	header file for a bitset struct						*
*	which is sed to make sure no set bnit i used twice	*
********************************************************/

/********************************************************
*	class cBitSet										*
*	This is our bitset class for storing which face		*
*	has already been drawn								*
********************************************************/
class cBitset 
{
private:

	// Our private bit data that holds the bits and size
	unsigned int *m_pBits;
	int m_Size;

public:

	// Initialize all the data members
    cBitset() : m_pBits(0), m_Size(0) {}

	// This is our destructor
	~cBitset() 
	{
		// If we have valid memory, get rid of it
		if(m_pBits) 
		{
			delete [] m_pBits;
			m_pBits = NULL;
		}
	}

	// This resizes our bit set to a size so each face has a bit associated with it
	void Resize(int count) 
	{ 
		// Get the size of integers we need
		m_Size = count/32 + 1;

		// Make sure we haven't already allocated memory for the bits
        if(m_pBits) 
		{
			delete [] m_pBits;
			m_pBits = 0;
		}

		// Allocate the bits and Initialize them
		m_pBits = new unsigned int[m_Size];
		ClearAll();
	}

	// This does the binary math to set the desired bit
	void Set(int i) 
	{
		m_pBits[i >> 5] |= (1 << (i & 31));
	}

	// This returns if the desired bit slot is a 1 or a 0
	int On(int i) 
	{
		return m_pBits[i >> 5] & (1 << (i & 31 ));
	}

	// This clears a bit to 0
	void Clear(int i) 
	{
		m_pBits[i >> 5] &= ~(1 << (i & 31));
	}

	// This Initializes the bits to 0
	void ClearAll() 
	{
		memset(m_pBits, 0, sizeof(unsigned int) *m_Size);
	}
};
/*---------------end of cBitSet declaration------------*/
#endif // _BITSET_H_