#ifndef _QUAKE3BSP_H
#define _QUAKE3BSP_H
/********************************************************
*	bspfile.h											*
*	main quake bsp level class							*
*	used to load and Render an entire bsp file			*
********************************************************/
#include "../../system/engine.h"
#include "bsptypes.h"
#include "bspEntity.h"
#include "bitset.h"

#include "faces/faces.h"
#include "models/models.h"

#include "bsploader.h"
#include "bsptree.h"

// a single index mesh will hold all the vertex data
#include "../../3d/meshes/indexmesh.h"
#include "../../3d/3dtypes/texture.h"


/********************************************************
*	class cBSPFile										*
*	encapsulates an entire BSP level					*
********************************************************/
class cBSPFile
{
protected :

    // this mesh holds all vertices and all indices
    sIndexedMesh m_Mesh;

    // The texture and light map array for the level
    sTexture    m_EmptyTexture;
    sTexture	m_Textures	[MAX_TEXTURES]	;	//	the array of m_Textures (may be huge)
    sTexture	m_LightMaps	[MAX_TEXTURES]	;	// The light map texture array
    UINT	    m_Blenders	[MAX_TEXTURES]	;	//	blender id (0=none,1=normal,2=blended,3=alpha)

    //	the bsp loader object
    cBSPLoader	m_BSPLoader;

    //	doors in the map
    vector<cMapDoor*>	m_Doors;


    void			CreateVertices(int PatchSmoothness)		;	//	Create the map vertices (allocate) and Assign
    void			CreateTextures()						;	//	Create the textures used for Rendering
    void			CreateLightMaps()						;	//	Create the light maps used for Rendering
    void			CreateFaces(int PatchSmoothness)		;	//	Create all the faces, including the patches
    void			CreateStartLocations()					;	//	Create the start locations
    void			CreateDoors()							;	//	Create the doors
    void			CreateTree()							;	//	fill the bsp tree
    void			CreateCustomTextures()					;	//	Create all the custom m_Textures required
    int				FindTextureIndex(char* filename)		;	//	using the name, find the texture index

    unsigned int	m_NumVertices							;	//	number of Rendering vertices



    //	this will count the Rendered faces
    int			m_NumFacesDrawn		;
    int			m_NumTextures		;	// The number of texture maps
    int			m_NumLightMaps		;	// The number of light maps
    int			m_CustomTextureID	;	//	index for the currently selected custom texure
    int			m_NumFaces			;	//	The number of faces in the model
    sBSPFace**	m_ppFaces			;	//	a vector keeping pointers to all the faces

public:

    //	the bsp tree object used to set up the frame to be Rendered
    cBSPTree	m_Tree;


    //	enable/disable Rendering of doors
    bool	m_bIsRenderDoors	;
    bool	m_bIsLightMapped	;
    bool	m_bIsTextured		;

    //	start locations of the player
    vector<Vector3>	m_StartPositions;

    //	this static vector stores the camera location
    static	Vector3	m_CameraPosition;

    // Our constructor
    cBSPFile();

    // Our destructor..Destroy the data..
    virtual ~cBSPFile()
    {
        Destroy();		
        System()->println("BSP Map Destroyed");
    }

    // This render the level to the screen, currently the camera pos isn't being used
    void RenderFrom(const Vector3 &vPos);

    // This Destroys the level data
    void Destroy();

    //	Level Loading Functions
    bool	Load(char* filename)	;	//	load an entire bsp file

    //	function to return the nearest door
    cMapDoor*	GetNearestDoor(const Vector3& Loc,const Vector3& Fwd);

    //	checks if the point is in a solid space or not..
    bool	IsSolid(Vector3& pos)
    {	return (m_Tree.Leafs[m_Tree.FindLeaf(pos)].area==-1);	}

    // update the map state , until now inly animates the rotating and sliding doors
    void Update(float dt);

    friend class cBSPCamera;
};
/*-----------end of cBSPFile declaration---------------*/
#endif

