#ifndef _BSP_LOADER_H
#define _BSP_LOADER_H
/********************************************************
*	bsploader.h											*
*	By Shady El Mously									*
*	this header is the declaration of the bsp loader	*
********************************************************/
#include "bsptypes.h" 
#include "bspentity.h"


/********************************************************
*	class cBSPLoader									*
*	used to load a BSP file data structures				*
*	in order to be used by the BSP Renderer				*
********************************************************/
class cBSPLoader 
{
public :

    friend class cQuake3BSP;

    //	the file pointer inside the BSP file..
    FILE	*	m_pFile				;	//	pointer to BSP file

    //	all the lumps info is here
    tBSPLump		m_Lumps[kMaxLumps]	;
    void			LoadLumps();

    //	there are 16 lumps from 1 to 16
    //	each one has an offset and a size
    //	each one has a set ...of size/size of the data struct
    //	so i save here pointers to all these data structures...in their same order in the file


    //	all entities should be here for further processing
    vector<sEntity*>	m_Entities		;
    void				LoadEntities()	;

    //	 the m_Textures
    tBSPTexture *	m_pTextures			;
    int				m_NumTextures		;
    void			LoadTextures()		;

    //	the 3d planes
    tBSPPlane	*	m_pPlanes			;
    int				m_NumPlanes			;
    void			LoadPlanes()		;

    //	the BSP Nodes
    tBSPNode	*	m_pNodes			;
    int				m_NumNodes			;
    void			LoadNodes()			;

    //	the BSP Leaves
    tBSPLeaf	*	m_pLeafs			;
    int				m_NumLeafs			;
    void			LoadLeafs()			;

    // the leaves faces
    tBSPLeafFace*	m_pLeafFaces		;
    int				m_NumLeafFaces		;
    void			LoadLeafFaces()		;

    //	the leaves brushes
    tBSPLeafBrush*	m_pLeafBrushes		;
    int				m_NumLeafBrushes	;
    void			LoadLeafBrushes()	;

    // the models in the map
    tBSPModel	*	m_pModels			;
    int				m_NumModels			;
    void			LoadModels()		;

    //	the brushes
    tBSPBrush	*	m_pBrushes			;
    int				m_NumBrushes		;
    void			LoadBrushes()		;

    //	the brush sides
    tBSPBrushSide*	m_pBrushSides		;
    int				m_NumBrushSides		;
    void			LoadBrushSides()	;

    //	the vertexes
    tBSPVertex	*	m_pVertices			;
    int				m_NumVertices		;
    void			LoadVertices()		;

    //	 the mesh vertexes (indexes)
    tBSPMeshVertex*	m_pMeshVertices		;
    int				m_NumMeshVertices	;
    void			LoadMeshVertices()	;

    //	the shaders ???
    tBSPShader*		m_pShaders			;
    int				m_NumShaders		;
    void			LoadShaders()		;

    //	 the faces
    tBSPFace*		m_pFaces			;
    int				m_NumFaces			;
    void			LoadFaces()			;

    // the light maps
    tBSPLightmap*	m_pLightMaps		;
    int				m_NumLightMaps		;
    void			LoadLightMaps()		;

    // the lights
    tBSPLight*		m_pLights			;
    int				m_NumLights			;
    void			LoadLights()		;

    //	the visibility Data
    tBSPVisData		m_VisibilityData			;
    void			LoadVisibilityData()		;

public :

    //	constructor
    cBSPLoader()
    {	Initialize();	}

    //	destructor
    virtual ~cBSPLoader()
    {	Destroy();		}

    //	this will load the data..retuns true on success
    bool Load(const char * filename);

    //	this Initializes everything
    void Initialize();

    //	Destroy (Deallocate) the data after finished loading the bsp map
    void Destroy();
};

/*----------end of class cBSPLoader declaration--------*/

#endif // _BSP_LOADER_H