#ifndef _TWISTER_BSPFACE_H_
#define _TWISTER_BSPFACE_H_

/********************************************************
*	bspface.h                                           *
* By Shady El Mously									*
* Part of the Twister 3d engine							*
* declaration file for the sBSPFace class		    	*
********************************************************/

#include "../../graphics/graphicssubsystem.h"
#include "../../meshes/indexmesh.h"
#include "../../3dtypes/texture.h"
#include "../../math/math.h"
#include "../bsptypes.h"

// properties of faces..
#define FACE_NONE		0
#define FACE_NORMAL		1
#define FACE_BLENDED	2
#define FACE_ALPHA		3

//	some id for face 
#define FACE_STATIC		0
#define FACE_ROTDOOR	1
#define FACE_TRANSDOOR	2


/********************************************************
*	struct sBSPFace										*
*	base class for all face types						*
********************************************************/
struct sBSPFace 
{
    int				m_NumVertices		;
    int				m_FirstVertexIndex	;
    sTexture*	m_pTexture		;
    sTexture*	m_pLightMap		;
    int				m_RenderMode		;	//	how it will be Rendered ?
    int				m_Owner				;	//	owner of the face..?
    sIndexedMesh* pMesh;


    // constructor, everything is initially Zero
    sBSPFace()
    {
        pMesh=NULL;
        m_RenderMode		=	FACE_NORMAL		;	
        m_Owner				=	FACE_STATIC		;
        m_pLightMap			=	NULL			;
        m_pTexture			=	NULL			;
    }

    // destructor.. does nothing
    virtual ~sBSPFace()
    {}

    // each of the child classes for the bsp face should implement their rendering function
    inline virtual void Render()=0;


};
/*-------------end of sBSPFace declaration-------------*/


sBSPFace* CreatePolygonFace(sTexture* pTexture,
                            sTexture* pLightmap,
                            int renderMode,
                            int firstVertexIndex,
                            int vertexCount,
                            sIndexedMesh* pMainMesh
                            );
sBSPFace* CreateMeshFace(sTexture* pTexture,
                         sTexture* pLightmap,
                         int renderMode,
                         int startVertIndex,
                         int meshVertexIndex,
                         int numIndexes,
                         sIndexedMesh* pMainMesh
                         );

sBSPFace*	CreatePatchFace(sTexture* pTexture,
                            sTexture* pLightMap,
                            int renderMode,
                            tBSPFace *pFace,tBSPVertex* verts,sIndexedMesh* pMainMesh);


sBSPFace* CreateBillBoardFace();


#endif // _TWISTER_BSPFACE_H_
