#ifndef _COLOR_3_H_
#define _COLOR_3_H_
/********************************************************
*	color3.h											*
*	By Shady EL Mously									*
*	header file for the basic color data type			*
********************************************************/
#include "vector3.h"

/********************************************************
*	Color3 Declaration									*
********************************************************/
struct Color3
{
	// the data is 3 floats.. red, green and blue..
	// value of on1 is full and zero is empty
	// using unions enable the accessing as arrays also..
	union {
		struct 
		{
			float r, g, b; // Red, Green, and Blue color data
        };
		float c[3];
	};

	// empty constructor
	Color3(){}

	// construst usin 3 floats
	Color3( float inR, float inG, float inB ) :
		r( inR ), g( inG ), b( inB )
	{
	}

	// give some calues if needed
	void Assign( float inR, float inG, float inB )
	{
		r = inR;
		g = inG;
		b = inB;
	}
	// give some calues if needed
	void AssignRGB( int inR, int inG, int inB )
	{
		r = inR/255.0f;
		g = inG/255.0f;
		b = inB/255.0f;
	}

	// this will return the value of the coor as a dword..
	/// (like 0xff00ff00 per example.; usefeul if i wanna use the coor in other standard
	/// gdi or any other calls
	unsigned long MakeWord()
	{
		unsigned long iR = (int)(r * 255.f ) << 16;
		unsigned long iG = (int)(g * 255.f ) << 8;
		unsigned long iB = (int)(b * 255.f );
		return 0xff000000 | iR | iG | iB;
	}

	// same as make word
	unsigned long MakeWordSafe()
	{
		Color3 temp = *this;
		temp.Sat();
		return temp.MakeWord();
	}


	// this will bound the color channels value between 0 and 1 as is expected..
	// if any of the values are >1, cap them.
	void Sat()
	{
		if( r > 1 ) 
			r = 1.f;
		if( g > 1 ) 
			g = 1.f;
		if( b > 1 ) 
			b = 1.f;
		if( r < 0 ) 
			r = 0.f;
		if( g < 0 ) 
			g = 0.f;
		if( b < 0 ) 
			b = 0.f;
	}

	// operators for adding and subtracting and dividing and multiplying colors
	Color3& operator += ( const Color3& in );
	Color3& operator -= ( const Color3& in );
	Color3& operator *= ( const float& in );
	Color3& operator /= ( const float& in );


	// some basic colors.(static)
	static const Color3 Black;
	static const Color3 Gray;
	static const Color3 White;
	static const Color3 Red;
	static const Color3 Green;
	static const Color3 Blue;
	static const Color3 Magenta;
	static const Color3 Cyan;
	static const Color3 Yellow;
};
/*---------------end of Color3 Declaration-------------*/

/********************************************************
*	operator overloads for struct color3				*
********************************************************/

/**
 * Accumulative addition of two colors
 */
inline Color3& Color3::operator += ( const Color3& in )
{
	r += in.r;   
	g += in.g;   
	b += in.b;
	return *this;
}


/**
 * Accumulative subtraction of two colors
 */
inline Color3& Color3::operator -= (const Color3& in)
{
	r -= in.r;   
	g -= in.g;   
	b -= in.b;
	return *this;
}


/**
 * Accumulative multiplication of a color by a scalar
 */
inline Color3& Color3::operator *= (const float& in )
{
	r *= in;   
	g *= in;   
	b *= in;
	return *this;
}


/**
 * Accumulative division of a color by a scalar
 */
inline Color3& Color3::operator /= ( const float& in )
{
	float inv = 1.f / in;
	r *= inv;   
	g *= inv;   
	b *= inv;
	return *this;
}


/**
 * Adds two colors together: ret = a + b
 */
inline Color3 operator+(Color3 const &a,Color3 const &b)
{
	return Color3
	(
		a.r+b.r,
		a.g+b.g,
		a.b+b.b
	);
}; 
 

/**
 * Subtracts two colors : ret = a - b
 */
inline Color3 operator-(Color3 const &a,Color3 const &b)
{
	return Color3
	(
		a.r-b.r,
		a.g-b.g,
		a.b-b.b
	);
}; 


/**
 * Scales a color by a float : ret = a * b
 */
inline Color3 operator*(Color3 const &a, float const &b)
{
	return Color3
	(
		a.r*b,
		a.g*b,
		a.b*b
	);
}; 


/**
 * Scales a color by a float : ret = a * b
 */
inline Color3 operator*(float  const &a,Color3 const &b)
{
	return Color3
	(
		a*b.r,
		a*b.g,
		a*b.b
	);
}; 


/**
 * Divides a color by a float : ret = a / b
 */
inline Color3 operator/(Color3 const &a, float const &b)
{
	float inv = 1.f / b;
	return Color3
	(
		a.r*inv,
		a.g*inv,
		a.b*inv
	);
}; 


/**
 * color Equality, epsilon used due to numerical imprecision
 */
inline bool operator==(Color3 const &a,Color3 const &b)
{
	if(fastabs(a.r-b.r)<EPSILON)
		if(fastabs(a.g-b.g)<EPSILON)
			if(fastabs(a.b-b.b)<EPSILON)
				return true;
	return false;
};


/*-----------end of struct color3 overloads--------------------*/


#endif // _COLOR_3_H_
