#ifndef _COLOR_4_H_
#define _COLOR_4_H_

/********************************************************
*	color4.h											*
*	By Shady EL Mously									*
*	header file for the alpha based color data type		*
********************************************************/
#define ColorValue(X)	(*(D3DCOLORVALUE*)&X);
#include "color3.h"
/********************************************************
*	Color4 declaration									*
********************************************************/
struct Color4
{
	union 
	{
		struct 
		{
			float r, g, b, a; // Red, Green, and Blue color data
        };
		float c[4];
	};

	// empty constructor
	Color4(){}

	// construct using 4 floating point values
	Color4( float inR, float inG, float inB, float inA ) :
		r( inR ), g( inG ), b( inB ), a( inA )
	{
	}

	// construct using a color and a alpha value
	Color4( const Color3& in, float alpha = 1.f )
	{
		r = in.r;
		g = in.g;
		b = in.b;
		a = alpha;
	}

	// using a format like 0xff00ff00.. construct
	Color4( unsigned long color )
	{
		b = (float)(color&255) / 255.f;
		color >>= 8;
		g = (float)(color&255) / 255.f;
		color >>= 8;
		r = (float)(color&255) / 255.f;
		color >>= 8;
		a = (float)(color&255) / 255.f;
	}


	// set value of a color4 value
	void Assign( float inR, float inG, float inB, float inA )
	{
		r = inR;
		g = inG;
		b = inB;
		a = inA;
	}
	// give some calues if needed
	void AssignRGB( int inR, int inG, int inB ,int inA)
	{
		r = inR/255.0f;
		g = inG/255.0f;
		b = inB/255.0f;
		a=	inA/255.0f;
	}

	// same idea.. return the word 0xaarrggbb
	unsigned long MakeWord()
	{
		unsigned long iA = (int)(a * 255.f ) << 24;
		unsigned long iR = (int)(r * 255.f ) << 16;
		unsigned long iG = (int)(g * 255.f ) << 8;
		unsigned long iB = (int)(b * 255.f );
		return iA | iR | iG | iB;
	}

	// return 0xaarrggbb safely
	unsigned long MakeWordSafe()
	{
		Color4 temp = *this;
		temp.Sat();
		return temp.MakeWord();
	}

	// if any of the values are >1, cap them.
	void Sat()
	{
		if( r > 1 ) 
			r = 1.f;
		if( g > 1 ) 
			g = 1.f;
		if( b > 1 ) 
			b = 1.f;
		if( a > 1 ) 
			a = 1.f;
		if( r < 0 ) 
			r = 0.f;
		if( g < 0 ) 
			g = 0.f;
		if( b < 0 ) 
			b = 0.f;
		if( a < 0 ) 
			a = 0.f;
	}

	Color4& operator += (const Color4& in);
	Color4& operator -= (const Color4& in);
	Color4& operator *= (const Color4& in);
	Color4& operator /= (const Color4& in);
	Color4& operator *= (const float& in);
	Color4& operator /= (const float& in);

	// some basic colors.
	static const Color4 Black;
	static const Color4 Gray;
	static const Color4 White;
	static const Color4 Red;
	static const Color4 Green;
	static const Color4 Blue;
	static const Color4 Magenta;
	static const Color4 Cyan;
	static const Color4 Yellow;
};
/*-----------------end of Color4 declaration-----------*/

#endif // _COLOR_4_H_