#ifndef _MATRIX3_H_
#define _MATRIX3_H_

#include "vector3.h"
#include "vector4.h"
#include "segment.h"
//#include "matrix4.h"



struct Matrix3  
{
    //	3 different representations for the same area in memory
    union 
    {
        struct 
        {
            float	_11, _12, _13;
            float	_21, _22, _23;
            float	_31, _32, _33;
        };
        float	m[3][3];		//	can be accesses as a 2d array 3*3
        float   d[9];			//	can be accessed as a single 9 floats arrays
    };

    inline void MakeZero()
    {
        _11=0.0f;  _12=0.0f; _13=0.0f;
        _21=0.0f;  _22=0.0f; _23=0.0f;
        _31=0.0f;  _32=0.0f; _33=0.0f;
    }

    inline void MakeIdentity()
    {
        _11=1.0f;  _12=0.0f; _13=0.0f;
        _21=0.0f;  _22=1.0f; _23=0.0f;
        _31=0.0f;  _32=0.0f; _33=1.0f;
    }

    //$hack this can be optimized some(if s == 0)
    inline  void SetRotationFromQuaternion(Vector4& q1)
    {
        float n, s;
        float xs, ys, zs;
        float wx, wy, wz;
        float xx, xy, xz;
        float yy, yz, zz;

        n = (q1.x * q1.x) + (q1.y * q1.y) + (q1.z * q1.z) + (q1.w * q1.w);
        s = (n > 0.0f) ? (2.0f / n) : 0.0f;

        xs = q1.x * s;  ys = q1.y * s;  zs = q1.z * s;
        wx = q1.w * xs; wy = q1.w * ys; wz = q1.w * zs;
        xx = q1.x * xs; xy = q1.x * ys; xz = q1.x * zs;
        yy = q1.y * ys; yz = q1.y * zs; zz = q1.z * zs;

        _11 = 1.0f - (yy + zz); _21 =         xy - wz;  _31 =         xz + wy;
        _12 =         xy + wz;  _22 = 1.0f - (xx + zz); _32 =         yz - wx;
        _13 =         xz - wy;  _23 =         yz + wx;  _33 = 1.0f - (xx + yy);
    }

    inline void Multiply(Matrix3& a)
    {
        Matrix3& b=*this;
        Matrix3 Result; //safe not to initialize

        for (int j = 0; j < 3; j++)         // transform by columns first
            for (int i = 0; i < 3; i++)     // then by rows
                Result.m[i][j] = a.m[i][0] * b.m[0][j] + a.m[i][1] * b.m[1][j] + a.m[i][2] * b.m[2][j];

        //copy result back to this
        b = Result;
    }

    // get local X axis
    inline Vector3&	GetLocalX()
    {	return *(Vector3*)&_11;		}
    // get local Y axis
    inline Vector3&	GetLocalY()
    {	return *(Vector3*)&_21;		}
    // get local Z axis
    inline Vector3&	GetLocalZ()
    {	return *(Vector3*)&_31;		}
    // get any axis
    inline Vector3* GetAxis()
    {
        return (Vector3*)&_11;
    }

    // access as array of vector3
    inline Vector3& operator[](int index) 
    {
        return ((Vector3*)&_11)[index];
    }




    // make it an axis rotation matrix
    void IsAxisRotation(const Vector3& inAxis, float angle);

};


//
//inline void Matrix4SetRotationFromMatrix3(Matrix4& matrix, const Matrix3& source)
//{
//    matrix._11=source._11;matrix._21=source._21;matrix._31=source._31;
//    matrix._12=source._12;matrix._22=source._22;matrix._32=source._32;
//    matrix._13=source._13;matrix._23=source._23;matrix._33=source._33;
//}
//


#endif // _MATRIX3_H_