#ifndef _TWISTER_RAYTRACE_H_

#include <float.h>
#include "matrix4.h"
#include "matrix3.h"
#include "vector4.h"
#include "Plane.h"

// simple AABB
struct sAABB
{
    Vector3 Min;
    Vector3 Max;
};

// simple OBB
struct sOBB
{
    Matrix3 Rotation;
    Vector3 Center;
    Vector3 Extents;
};

// tests if a ray hits a quad [rect]
bool RayHitQuad(Vector3& raySource,Vector3& rayDirection,Matrix4& quadTransform,float quadWidth,float quadHeight);

// test if a ray hits a box
bool RayHitBox(Vector3& raySource,Vector3& rayDirection,Matrix4& boxTransform,float sizeX,float sizeY,float sizeZ);

// returns true if the ray hits a sphere
bool RayHitSphere(Vector3& raySource,Vector3& rayDirection,Vector3& spherePos,float sphereRadius);



// get the nearest point on a plane to a point, knowing in advance that the normal is normalized
inline void ClosestPointToPlane(Plane& p,Vector3& q,Vector3& result)
{
    float t=(p.n.Dot(q)-p.d);
    result.Subtract(q,t*p.n);
}

// if u wanna get the distance from a point to a plane
inline float GetDistanceToPlane(Plane& p,Vector3& q)
{
    // replace by return (p.n.Dot(q)-p.d) if the normal is unit
    return (p.n.Dot(q)-p.d)/p.n.Dot(p.n);
}

// given a line segment (2 points, this computes the closest point on this line to a given point
void ClosestPointInLineSegmentToPoint(Vector3& c,Vector3& a,Vector3& b,float& t,Vector3& d);

// square distance between point and segment
float SquareDistPointToSegment(Vector3& a,Vector3& b,Vector3& c);

// given an aabb (min and max)
// this gets the point on aabb closest to p
void ClosestPointToAABB(Vector3& p,sAABB& b,Vector3& q);

// if just the distance is needed
float SquareDistPointToAABB(Vector3& p,sAABB& b);

// given an OBB, and a point, this gets the closest point on the OBB
void ClosestPointToOBB(Vector3& p,sOBB b,Vector3& q);

// computes the square distance from a point to an OBB
float SquareDistPointToOBB(Vector3& p,sOBB& b);

// given a triangle a,b,c and a point p, this computes the nearest point on the triangle to p
void ClosestPointToTriangle(const Vector3& a,const Vector3& b,const Vector3& c,const Vector3& p,Vector3& q);

// test if a point is outside of a plane, regardless of the order of the poits, but given a 4th point
int PointOutsideOfPlane(const Vector3& p,const Vector3& a,const Vector3& b,const Vector3& c,const Vector3& d);

// get the closest point ina a tetrahedron to a point
Vector3 ClosestPointToTetrahedron(const Vector3& a,const Vector3& b,const Vector3& c,const Vector3& d,const Vector3& p);

// test the intersection of a ray and a sphere, and gets the intersection point too
int IntersectRaySphere(Vector3& p,Vector3& d,Vector3& sc,float radius,float& t,Vector3& q);

// intersect a ray with AABB, return 1 on success and computes the point of intersection
int IntersectRayAABB(Vector3& p,Vector3& d,sAABB& a,float& tmin,Vector3& q);

// same with an OBB
int IntersectRayOBB(Vector3& p,Vector3& d,Matrix4& transform,Vector3& boxSize,float& tmin,Vector3& q);


#endif // _TWISTER_RAYTRACE_H_