#include "vector3.h"
#include "vector2.h"
#include <vector>

#ifndef _SEGMENT_H_
#define _SEGMENT_H_

using std::vector;
/********************************************************
*	Segment struct										*
*	part of the twister engine							*
*	By Shady El Mously									*
*	This is a very simple line in 3d space...			*
*	made from 2 points									*
*	from type point3d, useful later						*
********************************************************/
struct Segment
{
	Vector3 v0,v1;

	Segment(const Vector3 &a, const Vector3 &b )
	{
		v0 = a; v1 = b;
	}
	Segment(){}

	//equality of segments
	bool operator==(Segment& other)
	{
		return ((v0==other.v0)&&(v1==other.v1)||(v0==other.v1)&&(v1==other.v0));
	}
};
/*--------------end of struct Segment------------------*/

// this function gets the intersection point of 2 2D line segments
Vector3 Get2DIntersectionPoint(const Segment& segment1,const Segment& segment2);


// a 2d segment structure
struct Segment2D
{
	Vector2 v0;
	Vector2 v1;

	// find the intersection point with another segment if existing
	Vector2 Intersect(Segment2D& other);
};

// given 2 points, create a parrlel segment with a certain distance from them
void ComputeParralelSegment2D(Segment2D& seg,Vector2& p1,Vector2& p2,float distance);

// given a set of points, we create athick outline of segent around them, this can be used to given
// thickness to any shape described in 2d.. like the sheeting or the purlin or whatever
void GenerateOutlineSegments(float thickness,vector<Segment2D>& segments, vector<Vector2>& points,int align=0,bool edgeThickness=true);

#endif // _SEGMENT_H_