#ifndef _BSPHERE3D_H_
#define _BSPHERE3D_H_
/********************************************************
*	sphere.h											*
*	header file for the bounding sphere data type		*
********************************************************/

#include "vector3.h"

/********************************************************
*	Sphere data type declaration						*
********************************************************/
struct Sphere
{
	// the data is a 3d point.. and a radius
	float m_radius;
	Vector3 m_loc;

	Sphere():m_radius(1.0f),m_loc(Vector3::Zero)
	{
			// all is set
	}

	// construct from a point3d and a float
	Sphere(float radius, Vector3 loc ):m_radius( radius ), m_loc( loc )
	{
			// all is set
	}


	// construct.. using a point as a center.. and a list of point ..( a polygon..
	// the algorithm will pass through al the points in the list and find the rauid needed
	// to represent this object with a sphere
	Sphere( Vector3 loc, int nVerts, Vector3* pList )
	{
		m_loc = loc;
		m_radius = 0.f;
		float currRad;
		for( int i=0; i< nVerts; i++ )
		{
			currRad = pList[i].Mag();
			if( currRad > m_radius )
			{
				m_radius = currRad;
			}
		}
	}

	// template constructor..
	// using this stuff
	// a bounding sphere is made.. from a point
	// and two other primitives.. point3d or point4d
	template< class iter >
	Sphere(Vector3 loc, iter& begin, iter& end )
	{
		iter i = begin;
		m_loc = loc;
		m_radius = 0.f;
		float currRad;
		while( i != end )
		{
			currRad = (*i).mag();
			if( currRad > m_radius )
			{
				m_radius = currRad;
			}
			i++;
		}
	}

	Sphere& operator += ( const Vector3& in );
	Sphere& operator -= ( const Vector3& in );

	// this will return true if two spheres intersect..
	static bool Intersect(Sphere& a,Sphere& b)
	{
		// avoid a square root by squaring both sides of the equation
		float magSqrd = a.m_radius * a.m_radius + b.m_radius * b.m_radius;
		if( (b.m_loc - a.m_loc).MagSquared() > magSqrd )
		{
			return false;
		}
		return true;
	}

	// same but to test with he current calling object
	bool Hit(Sphere& a)
	{
		return Intersect(*this,a);
	}
};
/*-------------end of Sphere declaration---------------*/
#endif // _BSPHERE3D_H_