/********************************************************
*	vector2.h											*
*	By Shady El Mously									*
*	part of the twister game engine						*
*	this is the declaration file fo the vector2 struct	*
*	the main 2d vector object used in the engine		*
*	it may be use to hold and animate texture			*
*	coordinates											*
********************************************************/

#ifndef _VECTOR2_H_
#define _VECTOR2_H_

#include "vector3.h"

/********************************************************
*	struct vector2 declaration							*
*	can be used for texure coordinates					*
********************************************************/
struct Vector2
{
	union
	{
		float v[2];
		struct
		{
			float x,y;
		};
		struct
		{
			float s,t;
		};
	};

	//	default constructor (empty)
	Vector2()		{}

	//	construct with 3 floats
	Vector2(float a,float b):x(a),y(b){}

	//	assigning values
	void Assign(float X,float Y)
	{
		x=X;
		y=Y;
	}

	//	treating vector3 as vector2
	operator Vector3()
	{
		return Vector3(x,y,0);
	}

	Vector2& operator=(Vector3& v)
	{
		x=v.x;
		y=v.y;
		return *this;
	}

	//	an add function which is better than the + operator (no constructor called)
	void Add(const Vector2& p1,const Vector2& p2)
	{
		x=p1.x+p2.x;
		y=p1.y+p2.y;
	}
	
	//	a subtract function which is better than the + operator (no constructor called)
	void Subtract(const Vector2& p1,const Vector2& p2)
	{
		x=p1.x-p2.x;
		y=p1.y-p2.y;
	}

	//	get the magnitude squared of the vector
	float MagSquared()
	{
		return x*x+y*y;
	}

	//	magnitude of the vector
	float Mag()
	{	return sqrtf(x*x+y*y);	}

	//	making the vector a normal vector
	void Normalize()
	{
		float M=1.0f/sqrtf(x*x+y*y);
		x=x*M;
		y=y*M;
	}

	//	dot product..return a scalar quantity
	float Dot(Vector2& a)
	{	return x*a.x+y*a.y;	}

	//	negation
	void Negate()
	{
		x=-x;
		y=-y;
	}


	//	some overloads (single operand)
	Vector2& operator += ( const Vector3& in );
	Vector2& operator -= ( const Vector3& in );
	Vector2& operator *= ( const float& in );
	Vector2& operator /= ( const float& in );
	Vector2  operator -  (); // this is plain old negation
};
/*-------------end of vector2 declaration--------------*/
#endif // _VECTOR2_H_
