#ifndef _VERTEX_FAMILY_H_
#define _VERTEX_FAMILY_H_

/************************************************************
*	vertex,h												*
*	Header declaration for all structures used				*
*	as vertices, incliding normal vertices, textured,		*
*	lit, and textulred and lit ones							*
*	according to the usage needed we may need a certain		*
*	type													*
************************************************************/


/************************************************************
*	struct sVertex											*
*	this is simply a vector..x,y,z							*
************************************************************/
typedef Vector3 sVertex;
/*-------------------end of struct sVertex-----------------*/

/************************************************************
*	struct sTVertex											*
*	this is a textured vertex, it just have additional u,v	*
*	coordinates for usage as a texture						*
************************************************************/

struct sTVertex
{
	union
	{
		struct
		{
			Vector3 p;	//	 the point (x,y,z) 
			Vector2 t;	// the texture coordinates (u,v)
		};

		struct 
		{
			float x,y,z,u,v;
		};

		struct 
		{
			float d[5];
		};
	};
};
/*------------------end of struct sTVertex-----------------*/


/************************************************************
*	struct sLVertex											*
*	this is a lit  vertex, it has extra 3 component for		*
*	the normal of lighting									*
************************************************************/
struct sLVertex
{
	union
	{
		struct
		{
			Vector3 p;	//	 the point (x,y,z) 
			Vector3 n;	// the normal
		};

		struct 
		{
			float x,y,z,nx,ny,nz;
		};

		struct 
		{
			float d[6];
		};
	};
};
/*------------------end of struct sLVertex-----------------*/

/************************************************************
*	struct sTLVertex										*
*	this is a lit and textured  vertex, it has extra 5		*
*	components for	light and texture coordinates			*
************************************************************/
struct sTLVertex
{
	union
	{
		struct
		{
			Vector3 p;	//	the point (x,y,z) 
			Vector3 n;	//	the normal (nx,ny,nz);
			Vector2 t;	//	the texture coordinate (u,v)
		};

		struct 
		{
			float x,y,z,nx,ny,nz,u,v;
		};

		struct 
		{
			float d[8];
		};
	};
};
/*------------------end of struct sTLVertex----------------*/


/************************************************************
*	struct sGenVertex										*
*	this is a lit and textured  vertex, it has extra 5		*
*	components for	light and texture coordinates			*
* has 2 texture coordinates, and even colors				*
* based on it, the dx8 vertex buffers may be created		*
************************************************************/
struct sGenVertex
{
	union
	{
		struct
		{
			Vector3 p;	//	the point (x,y,z) 
			Vector3 n;	//	the normal (nx,ny,nz);
			Vector2 t1;	//	the texture coordinate (u,v)
			Vector2 t2;	//	the texture coordinate (u,v)
			unsigned long   color;//	 a color?
		};

		struct 
		{
			float x,y,z,nx,ny,nz,u1,v1,u2,v2;
		};

		struct 
		{
			float d[10];
		};
	};
};



#endif // _VERTEX_FAMILY_H_


