#ifndef _MD3_H
#define _MD3_H

#include "md3loader.h"


class cMD3Base
{
public:


    // create a new model from a file, given the filename, and the skin file name
    bool CreateModel(sMD3Model* pModel,char* pFileName,char* pSkinFileName,char* pTextureFolder);


    // Member Variables

    // This stores the texture array for each of the textures assigned to this model
    sTexture m_Textures[MD3_MMAX_TEXTURES];	

    // This stores a list of all the names of the textures that have been loaded.  
    // This was created so that we could check to see if a texture that is assigned
    // to a mesh has already been loaded.  If so, then we don't need to load it again
    // and we can assign the textureID to the same textureID as the first loaded texture.
    // You could get rid of this variable by doing something tricky in the texture loading
    // function, but I didn't want to make it too confusing to load the textures.
    vector<string> strTextures;

    // This loads the models textures with a given path
    void LoadModelTextures(sMD3Model *pModel, LPSTR strPath);

    // assign the textures to all meshes which are inside the models
    void AssignTextures(sMD3Model *pModel);


};

// This is our model class that we use to load and draw and free the Quake3 characters
class CModelMD3 : public cMD3Base
{

public:

	// These our our init and deinit() C++ functions (Constructor/Deconstructor)
	CModelMD3();
	~CModelMD3();
	
	// This loads the model from a path and name prefix.   It takes the path and
	// model name prefix to be added to _upper.md3, _lower.md3 or _head.md3.
	bool LoadModel(LPSTR strPath, LPSTR strModel);

	// This loads the weapon and takes the same path and model name to be added to .md3
	bool LoadWeapon(LPSTR strPath, LPSTR strModel);

	// This render the character to the screen
	void Render()
    {
        m_Lower.Render();
    }


    // this will update the model if it is animated
    void UpdateModel();

    // This takes a string of an animation and sets the torso animation accordingly
    void SetTorsoAnimation(LPSTR strAnimation);

    // This takes a string of an animation and sets the legs animation accordingly
    void SetLegsAnimation(LPSTR strAnimation);

    // This returns a pointer to a .md3 model in the character (kLower, kUpper, kHead, kWeapon)
    sMD3Model *GetModel(int whichPart);

    // set the angles
    void SetAngles(float yaw,float pitch,float roll);
	
private:

    // This loads the animation config file (.cfg) for the character
    bool LoadAnimations(LPSTR strConfigFile);

	// These are are models for the character's head and upper and lower body parts
	sMD3Model m_Head;
	sMD3Model m_Upper;
	sMD3Model m_Lower;
	// This store the players weapon model (optional load)
	sMD3Model m_Weapon;
};

#endif
