#ifndef _TWISTER_MD3_LOADER_H_
#define _TWISTER_MD3_LOADER_H_


#include <windows.h>
#include <string>
using std::string;

#include "md3types.h"
#include "md3model.h"


bool IsInString(string strString, string strSubString);

// This class handles all of the main loading code
class CLoadMD3
{

public:

    // This inits the data members
    CLoadMD3();	


    // This is the function that you call to load the MD3 model
    bool ImportMD3(sMD3Model *pModel, char *strFileName);

    // This loads a model's .skin file
    bool LoadSkin(sMD3Model *pModel, LPSTR strSkin);

    // This loads a weapon's .shader file
    bool LoadShader(sMD3Model *pModel, LPSTR strShader);

private:


    // This reads in the data from the MD3 file and stores it in the member variables,
    // later to be converted to our cool structures so we don't depend on Quake3 stuff.
    void ReadMD3Data(sMD3Model *pModel);


    // This converts the member variables to our pModel structure, and takes the model
    // to be loaded and the mesh header to get the mesh info.
    void ConvertDataStructures(sMD3Model *pModel, tMd3MeshInfo meshHeader);

    // This frees memory and closes the file
    void CleanUp();

    // Member Variables		

    // The file pointer
    FILE *m_FilePointer;

    tMd3Header				m_Header;			// The header data
    tMd3Skin				*m_pSkins;			// The skin name data (not used)
    tMd3TexCoord			*m_pTexCoords;		// The texture coordinates
    tMd3Face				*m_pTriangles;		// Face/Triangle data
    tMd3Triangle			*m_pVertices;		// Vertex/UV indices
    tMd3Bone				*m_pBones;			// This stores the bone data (not used)
};


#endif // _TWISTER_MD3_LOADER_H_