#ifndef _TWISTER_MD3MODEL_H_
#define _TWISTER_MD3MODEL_H_

#include "md3mesh.h"


// This our model structure
struct sMD3Model 
{
    vector<tAnimationInfo>  m_Animations; // The list of animations
    vector<tMaterialInfo>   m_Materials;	// The list of material information (Textures and colors)
    vector<sMD3Mesh*>       m_Meshes;			// The object list for our model

    sMD3Model**             m_pLinks;         // This stores a list of pointers that are linked to this model
    tMd3Tag*                m_pTags;          // This stores all the tags for the model animations


    int                     m_CurrAnimation;					// The current index into pAnimations list 
    int                     m_CurrFrame;					// The current frame of the current animation 
    int                     m_NextFrame;						// The next frame of animation to interpolate too
    float                   m_CurrTime;                            // The ratio of 0.0f to 1.0f between each key frame
    float                   m_LastTime;                     // This stores the last time that was stored
    unsigned int            m_NumTags;						// This stores the number of tags in the model

    // the transformation matrix and 3 additional animation angles
    Matrix4                 m_Matrix;
    float                   m_Yaw;
    float                   m_Pitch;
    float                   m_Roll;

    sMD3Model();

    virtual ~sMD3Model();


    void UpdateTags();
    void Render();

    void RenderRec();
    
    void Link(sMD3Model* pChildToLink,char* tagName);
    void GetChildPosition(char* tagName,Vector3& pos);

    void SetCurrentTime();
    void Update();
};


#endif // _TWISTER_MD3MODEL_H_