#ifndef _FACES_MESH_H_
#define _FACES_MESH_H_

/************************************************************
*	facesmesh.h												*
*	By Shady El Mously										*
*	header declaration file for the faces mesh class		*
************************************************************/
#include "indexmesh.h"	//	for the Indexed mesh (base class)
#include "../3dtypes/polygon2d.h"
//#include "../../util/util.h"


#define FACESMESH_FLAGS_MAIN_FACES			0x0000000C
#define FACESMESH_FLAGS_SIDE_FACES			0x00000002
#define FACESMESH_FLAGS_FRONT_FACE			0x00000004
#define FACESMESH_FLAGS_BACK_FACE			0x00000008
#define FACESMESH_FLAGS_FRONTANDBACK_FACES	0x0000000C


// polygon 3d is simply a set of vectors in space
typedef vector<Vector3> Polygon3D;

/************************************************************
*	struct sFace, represents a single face inside the		*
*	faces mesh												*
************************************************************/

struct sFace
{
	unsigned int m_StartIndex;
	unsigned int m_NumIndices;
	unsigned int m_RenderMode;
	Vector3 m_Normal;
	// this compute the normals, given the original set of vertices
	void ComputeNormal(vector<Vector3>& Vertices,vector<unsigned short>& Indices);
};
/*------------end of struct sFace declaration--------------*/


/************************************************************
*	struct sFacesMesh declaration							*
*	an advnaced mesh used to represents the object as a set *
*	of coplanar faces, all vertices are defines just once	*
*	and the are a set of indices and a set of faces			*
************************************************************/
class sFacesMesh : public sIndexedMesh
{
private:
	
	// those are all the faces to be rendered
	vector<sFace*>	m_Faces;


	// this adds the faces of a single filled mesh
	void AddMesh(sMesh* pMesh,unsigned int faceFlags=FACESMESH_FLAGS_FRONT_FACE);

	// this would add the faces of a set of meshes..
	// probably wouold be called after tthe tesselation opertaion
	void AddMeshes(vector<sMesh*>& Meshes,unsigned int faceFlags=FACESMESH_FLAGS_FRONT_FACE);


public:

	// constructor...
	sFacesMesh();

	// copy constructor
	sFacesMesh(sFacesMesh& other);

	// the destructor
	~sFacesMesh()
	{		DestroyFaces();		}

	// the = operator.. we should create new faces similar to the ones in the other
	sFacesMesh& operator=(sFacesMesh& other);

	// this tarnsformas all vertices and faces using a certain matrix
	void Transform(Matrix4& matrix);
	// compute the normals for all the faces
	void ComputeNormals();
	// destroys the faces
	void DestroyFaces()

	{	
		for(unsigned int i=0;i<m_Faces.size();i++)
			delete m_Faces[i];
		m_Faces.clear();

	//	DestroyVector(m_Faces);		
	}
	// clean everything in here
	void Clean();
	// this renders the faces mesh, face by face 
	void Render();

	// compiling the faces mesh..add normals to each ones of the vertices
	void Compile();



	// add a single triangle
	void AddTriangle(Vector3& v1,Vector3& v2,Vector3& v3);

	// this creates a curves steel objects, given 9 control points
	// there is no plan projection in this case, used until now to create corner cuved pieces
	void AddPatch(Vector3* pControlPoints,int smoothness,unsigned int faceFlags=FACESMESH_FLAGS_FRONTANDBACK_FACES);

	// add he sides faces of an polygon extendes from 2 different z levels..
	// this is used to create objects with zero thickness
	// like girts or sheeting panels... or a flag.. or a 
	void AddSides(vector<Vector2>& polygon,float startZ,float endZ,unsigned int faceFlags=FACESMESH_FLAGS_FRONTANDBACK_FACES);

	// add a face, this is plain direct adding of 1 new face
	void AddFacePolygon(vector<Vector3>& polygon,Vector3& Normal);

	// add a coplanar polygon 
	// this is a more accurate version of the same function
	// this time all vertices are trnasformaed to be in the plane 0,0,1,0
	// so that the tesselator is accurate
	// after tesslating, they are trnasformed back again
	// the polygon sent can be convex, but the first 2 vertices are used to determine the 
	// winding drection, so it is required to not have the inner oportion in the first vertices
	// all plygon ponts must be coplanar, if not, they are brought to
	// the plane of the first 3
	void AddPolygon(vector<Vector3>& polygon,unsigned int flags=FACESMESH_FLAGS_FRONTANDBACK_FACES);

	// add faces resulting from a difference between two triangle
	// meshes...
	void AddDifference(Vector3& frontDirection,vector<Vector3>& Positive,vector<Vector3>& Negative,unsigned int faceFlags=FACESMESH_FLAGS_FRONTANDBACK_FACES);

	// add faces resulting from a intersection between two triangle
	// meshes...
	void AddIntersection(Vector3& frontDirection,vector<Vector3>& Positive,vector<Vector3>& Negative,unsigned int faceFlags=FACESMESH_FLAGS_FRONTANDBACK_FACES);

	// Add Faces for a projected polygon
	// this is useful if we have a 2d polygon we want to extrude
	void AddExtrusion(vector<Vector2>& Polygon,Matrix4& startMatrix,Matrix4& endMatrix,unsigned int flags=FACESMESH_FLAGS_SIDE_FACES|FACESMESH_FLAGS_FRONTANDBACK_FACES);

	// this makes the same using a path of matrices
	void AddExtrusion(vector<Vector2>& Polygon,vector<Matrix4>& matrixPath,unsigned int flags=FACESMESH_FLAGS_SIDE_FACES|FACESMESH_FLAGS_FRONTANDBACK_FACES);

	// Add Faces for a projected polygon
	// this is useful if we have a 2d polygon we want to extrude
	void AddExtrusion(vector<Vector2>& Polygon,vector<Vector3>& Path,Vector3& localX,unsigned int flags=FACESMESH_FLAGS_SIDE_FACES|FACESMESH_FLAGS_FRONTANDBACK_FACES);

	// convert the mesh into an all triangles mesh....
	void TriangulizeAllFaces();

	// convert the mesh into a set of outlined polygons
	// the direction if sent will be the camera direction, which will 
	// be used to remove the hidden faces
//	void PolygonizeAllFaces(Vector3& VisibleDirection=Vector3(0,0,0));

	// get the bounding box 2 points..
	void GetBoundingBox(Vector3& min,Vector3& max);

	// the 3 boolean functions betwwen 2 faces meshes

/*
	// compute the union of 2 faces meshes
	static sFacesMesh* ComputeUnion(sFacesMesh& mesh1,sFacesMesh& mesh2);

	// compute the intersection of 2 faces meshes
	static sFacesMesh* ComputeIntersection(sFacesMesh& mesh1,sFacesMesh& mesh2);
	
	// compute the difference between 2 faces meshes
	static sFacesMesh* ComputeDifference(sFacesMesh& positiveMesh,sFacesMesh& negativeMesh,float tolerance=0.0f);
*/


};
/*-----------end of struct sFacesMesh declaration----------*/
#endif // _FACES_MESH_H_
