#ifndef _INDEXED_MESH_H_
#define _INDEXED_MESH_H_

/************************************************************
*	indexmesh.h												*
*	header declaration file for the indexed mesh class		*
************************************************************/

#include "mesh.h"

/************************************************************
*	struct sIndexedMesh										*
*	encapsulates an aray of vertices and an array of inidces*
*	and enables rendering them								*
* could be used as a base class for any indexed mesh		*
************************************************************/
struct sIndexedMesh : public sMesh
{
protected :

    // the internal index buffer
    sIndexBuffer* m_pIndexBuffer;

    // number of allocated indices
    int m_NumAllocatedIndices;

public:

    // the indices used
    unsigned int * m_pIndices;

    // their count
    int m_NumIndices;

    // default constructor
    sIndexedMesh();

    // default destructor
    ~sIndexedMesh();

    // render the indexed mesh, using vertex arrays
    void Render()
    {
        m_pVertexBuffer->Render(m_pIndexBuffer);
    }


    // render a range of the indexes with offset
    void RenderIndexTrianglesRange(int startVertex,int startIndex,int numIndexes)
    {
        m_pVertexBuffer->Render(m_pIndexBuffer,startVertex,startIndex,numIndexes);
    }

    // render a range of the indexes
    void RenderIndexTrianglesRange(int startIndex,int numIndexes)
    {
        m_pVertexBuffer->Render(m_pIndexBuffer,startIndex,numIndexes);
    }


    // compile
    void Compile();

    // allocates more indices
    void ReserveIndices(int indexCount);

    // add one single new index
    void AddIndex(unsigned int newIndex);

    // create a box..
    void CreateTestBox();

    // create a wires box
    void CreateWireBox(float sizeX,float sizeY,float sizeZ);

    // this code will create the mesh of the box
    void CreateBox(float sizeX,float sizeY,float sizeZ);

    // this creates a grid..like a terrain will Y zeros
    void CreateGrid(float width,float depth,int numSegmentsX,int numSegmentsZ);

    // clean the mesh
    void Clear();


};
/*-------------end of Indexed mesh declaration-------------*/
#endif // _INDEXED_MESH_H_