#ifndef _MESH_PROCESSOR_H_
#define _MESH_PROCESSOR_H_

/****************************************************
*	meshprocessor.h									*
*	declaration file for the mesh processor class	*
*	this class is used to create and manipulate		*
*	meshes											*
****************************************************/

#include "../math/math.h"
#include "mesh.h"
#include "../../util/singleton.h"	// for the singletton..
#include <vector>
using std::vector;


/****************************************************
*	class CMeshProcessor declaration				*
*	this class can be used to create or manipulate	*
*	meshes, have methods like, extrude, tesselate	*
*	roatate, translate, invert, mirror				*
*	all functions are static and take an adress to a*
*	float and as a parameter						*
* this class is also used to generate sets of meshes*
*	using polygon tesselation techniques..			*
****************************************************/
class CMeshProcessor : public singleton<CMeshProcessor>
{
	public:
	// simple mesh manipulations..
	// all function will take a pointer to a float
	// and the stride steop between each vertex and the next (minimum=3)


	// all next function do not create new vertices, they just manipulate what is already in

	// translate a mesh
	static void TranslateMesh(float* pVertexData,int stride,int count,float x,float y,float z);

	// rotate a mesh
	static void RotateMesh(float* pVertexData,int stride,int count,float yaw,float pitch,float roll);

	// scale a mesh
	static void ScaleMesh(float* pVertexData,int stride,int count,float yaw,float pitch,float roll);

	// make any transformation, using a matrix
	static void TransformMesh(float* pVertexData,int stride,int count,const Matrix4& matrix);


	// given a set of vertices defined in the x-y plane, we ca extrude them using this function
	// the length is taken in the z diretion
	// i shouled send 2 vectors for vertices and for indices to this functio to be filled...
	static void Extrude(float* pVertexData,int stride,int count,vector<Vector3>& outputVerts,vector<unsigned short>& outputIndices,float length,bool useBackFace=false);



	// constructor and destructor
	CMeshProcessor();
	~CMeshProcessor();
	
};
/*-----end of class CMeshProcessor declaration-----*/

// global access to the mesh processor singletton
inline CMeshProcessor	*&	MeshProcessor		(void){return CMeshProcessor::GetSingletonPtr();}

#endif // _MESH_PROCESSOR_H_