#ifndef _AUDIO_H
#define _AUDIO_H

/****************************************************************
*	audio.h												*
*	By Shady el Mously											*
*	Main header for the declaration of sound subsystem classes	*
*	_AUDIO,music and sound										*
****************************************************************/

#include	"../util/util.h"	//	for singletton
#include	"../3d/math/math.h"
#include	<mmsystem.h>


//	DirectSound Includes
#include	<dsound.h>
#include	<dmusici.h>
#include	<dmusicc.h>
#include	<dmusicf.h>

// Convert from multibyte format to Unicode using the following macro:
#define MULTI_TO_WIDE( x,y )  MultiByteToWideChar( CP_ACP,MB_PRECOMPOSED, y,-1,x,_MAX_PATH);


/****************************************************************
*	The cAudioSubSystem class									*
*	By Shady el mawssely										*
*	This class encapsulates all the directsound/music			*
****************************************************************/
class cAudioSubSystem :	public singleton<cAudioSubSystem>
{
private:

	// direct music interface pointers
	IDirectMusicPerformance8*	m_Performance	;		// the directmusic performance manager 
	IDirectMusicLoader8*		m_Loader		;		// the directmusic loader
	IDirectMusicAudioPath8*		m_Path			;		//	direct music audio path for the sound
	IDirectSound3DBuffer*		m_Buffer		;		//	main sound buffer
	IDirectSound3DListener*		m_Listener		;		//	main sound listener

	DS3DBUFFER					m_SoundBufferParams	;	//	main buffer params
	DS3DLISTENER				m_ListenerParams	;	//	listener params



public:
	
	//	constructordestructor
	cAudioSubSystem();				//	start sound/music and report success
	virtual ~cAudioSubSystem();		//	shutdown sound/music and report success

	//	access functions :
	IDirectMusicPerformance8*	GetPerformance	(void)		{return m_Performance;	}
	IDirectMusicLoader8*		GetLoader		(void)		{return m_Loader;		}
	IDirectMusicAudioPath8*		GetPath			(void)		{return m_Path;			}
	IDirectSound3DBuffer*		GetMainBuff		(void)		{return m_Buffer;		}


	//	stuff for 3d
	void SetListenerPosition(Vector3& loc);
	void SetListenerOrientation(Vector3& loc,Vector3& Up);
};
/*----------end of class cAudioSubSystem declaration-----------*/


//	public access to the audio subsystem from here..
inline cAudioSubSystem*&	Audio()	{return cAudioSubSystem::GetSingletonPtr();}
/*-------------------------------------------------------------*/

/****************************************************************
*	class cSound												*
*	By Shady El Mously											*
*	This class is a bse class for a simple	2d sound			*
*	from it i will have 2 more classes , c3DSound and cMusic	*
*	Which is made from a midi file in the format				*
****************************************************************/		
class cSound
{
protected:

	IDirectMusicSegment8*		m_Segment;	// the directmusic segment

public:


	//	always make the sound with a file
	cSound(){}
	cSound(char * filename);			
	~cSound();
	
	void Load(char * filename)				;
	//	control functions
	void	Stop		(void)				;
	void	SetRepeat	(int)				;
	bool	IsPlaying	(void)				;

	//	the play function is virtual here
	virtual void	Play(void)				;
};

/*-------------end of class cSound declaration-------------------*/


/********************************************************************
*	class cMusic													*
*	used for the creation of music segments, the only difference	*
*	here is that they loop infinitely								*
********************************************************************/


class cMusic
{
private:

	IDirectMusicSegment*		m_dmsegment;	// the directmusic segment
	IDirectMusicSegmentState*	m_dmsegstate;	// the state of the segment
	
public:


	cMusic(char * filename);						//	must construct with a midifile
	~cMusic();

	void Load		(char*	filename);
	void Stop		(void);
	void Play		(void);
	
};
/*----------------end of class cMusic declaration------------------*/


/********************************************************************
*	class c3DSound													*
*	adds 3d capabilities to the sound								*
********************************************************************/
class c3DSound : public cSound
{
public:

	//	construct with a 3d wave file
	c3DSound(char* filename);

	//	play the 3d sound
	void Play();

	//	set the sound position
	void SetPosition(float x,float y,float z);
};
/*------------------end of class c3DSound declaration--------------*/

#endif //_AUDIO_H