/********************************************************
*	engine.h											*
*	part of the Twister 3d engine						*
*	this file is the declaration of the main engine		*
*	manager class , whic is the main system class		*
*	it is used everywhere								*
*  it starts and stops other secondary manegrs and wrap	*
*	the whole system									*
*	 can log events and report errors					*
********************************************************/

#ifndef _ENGINE_H
#define _ENGINE_H

#include "../util/singleton.h"		//	a manager class is a singleton
#include "../util/generic.h"		
#include "../util/screenmode.h"		//	for the screen mode
#include "profiler.h"

#include <fstream>
#include <windows.h>

using std::ofstream;

#define TWISTER_GRAHICSDRIVER_OPENGL    0x00000000
#define TWISTER_GRAHICSDRIVER_DIRECTX   0x00000001

/********************************************************
*	class cTwisterEngine declaration					*
********************************************************/
class cTwisterEngine : public singleton<cTwisterEngine>
{
private:
			
public:
	
	static bool		IsAlive							;	//	when this boolean is true.. its time to quit
	sScreenMode		DisplayMode						;	//	display settings for the engine
	FILE*			LogFile							;	//	a file for logging
	char			CurrentWorkingDirectory[256]	;	//	the current workding directory
	float			FrameRate						;	//	the frame rate 
    HINSTANCE       Instance                        ;
    bool            Active;
    unsigned        int m_GraphicsDriver;
  //  CSystemProfiler     Profiler;
	

	//	constructor of the engine
	cTwisterEngine(HINSTANCE AppInstance,void * pInitData=0);

	//	destructor
	~cTwisterEngine();

	//	this function will try to load the settings from a file
	bool LoadSettings();

	//	this function saves the settings
	void SaveSettings();

	//	this will Initialize the engine, i can use a certain handle to device context or not
	void Initialize(void * pInitData=0);
	
	//	this will shutdown the engine
	void ShutDown();

	//	the main function which will be the 3d application itself
	void Main();

	//	fatal errors always end the program
	void ReportError(char* ErrorString,...);

	//	output function inside the log file
	void print(char* ErrorString,...);
	void println(char* ErrorString,...);

	//	processing windows messages returns false when its over
	bool ProcessMessages();

	//	same windows message hanling.but this one waits..
	bool ProcessMessages2();


    // used to calculate the frame rate 
    int CalculateFPS();

    static LRESULT CALLBACK MessageHandler(	HWND	hWnd,	// Handle For This Window
        UINT	uMsg,					// Message For This Window
        WPARAM	wParam,					// Additional Message Information
        LPARAM	lParam)	;				// Additional Message Information

    static LRESULT CALLBACK DefaultMessageHandler(	HWND	hWnd,	// Handle For This Window
        UINT	uMsg,					// Message For This Window
        WPARAM	wParam,					// Additional Message Information
        LPARAM	lParam)	;				// Additional Message Information



};

//	the system class is accessed through the global function System
inline cTwisterEngine	*&	System		(void){return cTwisterEngine::GetSingletonPtr();}

/*------end of class cTwisterEngine declaration---------*/

#endif // _ENGINE_H