/********************************************************
*	generic.h											*
*	part of twister 3d engine							*
*	this is header declaration for generic data types	*
*	and functions and macros							*
*	most of the stuff declared here is used everywhere	*
********************************************************/
#ifndef _GENERIC_H_
#define _GENERIC_H_
#include <vector>
using std::vector;
/********************************************************
*	some useful macros to be used everywhere			*
********************************************************/
#define SAFE_DELETE(p)  if(p) {delete (p);(p)=0;	}
#define SAFE_FREE(p)	if(p) {delete [](p);(p)=0;	}
#define SAFE_RELEASE(x) if (x) {x->Release(); x=NULL;}
#define KEYDOWN(vk_code) ((GetAsyncKeyState(vk_code) & 0x8000) ? 1 : 0)
#define KEYUP(vk_code)   ((GetAsyncKeyState(vk_code) & 0x8000) ? 0 : 1)
#define Toggle(x)  x=!x


#ifdef _LIB
#define TWISTER_API __declspec(dllexport)
#endif

#ifndef _LIB
#define TWISTER_API   
#endif


//	gets the time and date when called as a c string
char * CurrentTime();


// removes an item from a vector
template <class T> void RemoveVectorItem(vector<T>& vec,T& item)
{
	int index=GetVectorIndex(vec,item);
	if(index!=-1)
	{
		vec.erase(&vec[index]);
	}
}

// generic tempalte vector based search function, gets the index to something 
// inside an stl vector, return 1 if not found
template <class T> int GetVectorIndex(vector<T>& vec,T& instance)
{
	for (unsigned int i=0;i<vec.size();i++)
	{
		if(vec[i]==instance)
			return i;
	}
	return -1;
}
// this destroy a vector of pointers..
template <typename T> void DestroyVector(vector<T>&	vec)
{
	for(unsigned int i=0;i<vec.size();i++)
	{
		delete vec[i];
	}
	vec.clear();
}

// this calls Render on a vector of pointers to anythings given they have the function Render
template <typename T> inline void RenderVectorItems(vector<T>&	vec)
{
	for(unsigned int i=0;i<vec.size();i++)
	{
		vec[i]->Render();
	}
}
// this calls Render on a vector of pointers to anythings given they have the function Render
template <typename T> inline void SilentRenderVectorItems(vector<T>&	vec)
{
	for(unsigned int i=0;i<vec.size();i++)
	{
		vec[i]->SilentRender();
	}
}



// find the first item in a vector satisfying a certain condition
template <class T,class F>	T	FindFirstVectorItem(vector<T>& Items,F func)
{
	for(unsigned int i=0;i<Items.size();i++)
	{
		if(func(Items[i]))
			return Items[i];
	}
	return NULL;
}


// find the last item in a vector satisfying a certain condition
template <class T,class F>	T	FindLastVectorItem(vector<T>& Items,F func)
{
	for(unsigned int i=0;i<Items.size();i++)
	{
		if(func(Items[Items.size()-1-i]))
			return Items[Items.size()-1-i];
	}
	return NULL;
}
/*-----------------end of screenmode structure---------------*/

#endif //_GENERIC_H_