#ifndef _RESOURCEMGR_H_
#define _RESOURCEMGR_H_
/********************************************************
*	ResourceMgr.h										*
*	By Shady EL Mously									*
*	part of TWISTER game engine							*
*	main header for resource management base stuff		*
********************************************************/
#include "singleton.h"
#include "generic.h"
#include <vector>

using std::vector;
#include "resourceentry.h"	//	the resource entries ...

/********************************************************
*	class cResourceManager								*
*	represents a resource manager for any resource type	*
*	i should first Create a class for the entries		*
*	(inherited from sResource Entry and then use this	*
*	class name and the name of the manager class i wish *
*	to make and	inherit from this class	here, will		*
*	be automaticly Created a database for the entries	*
*	and the manager keeps track of how many references	*
*	and	releases for each entry, it will free the		*
*	unreferenced entries automaticly and Add reference	*
*	upon referencing to	them	in each child class,	*
*	i should make a function which gets the				*
*	ID of a certain item								*
********************************************************/

template<typename T,typename X>
class cResourceManager : public singleton<X>
{
protected:

	//	the list of all Resource entries
	vector<T*>	dataBase;

	//	the base id, starts from one and always incrementing
	int	baseID;

public:


	//	constructor just set the base id to one
	cResourceManager()
	{	baseID	=	0;	}


	//	destructor free all resources in database
	virtual ~cResourceManager()
	{
		//	loop on all the m_Textures and deallocate the entry data
		for(unsigned int i=0;i<dataBase.size();i++)
		{
			dataBase[i]->FreeResource();
			SAFE_DELETE(dataBase[i]);
		}
	}


	//	get pointer to entry using the id
	T*	GetEntry(int id)
	{
		//	loop on all entries and try to find the texture requested
		for(int i=0;i<dataBase.size();i++)
		{
			//	if it found, return the id
			if(dataBase[i]->m_ID==id)
			{
				return dataBase[i];
			}
		}
		return NULL;
	}

	//	reference to a ceratin entry using its ID
	void Reference(int id)
	{
		//	loop on all entries and try to find the texture requested
		for(int i=0;i<dataBase.size();i++)
		{
			//	if it found, return the id
			if(dataBase[i]->m_ID==id)
			{
				dataBase[i]->RefCount++;
				break;
			}
		}
	}


	//	use this to derefence a texture from the database
	void Release(int id)
	{
		//	loop on all entries and try to find the texture requested
		for(int i=0;i<dataBase.size();i++)
		{
			if(dataBase[i]->m_ID==id)
			{
				dataBase[i]->RefCount--;

				//	now check if the count is Zero
				if(dataBase[i]->RefCount==0)
				{
					//	deallocate the bitmap data..
					dataBase[i]->FreeResource();

					//	and remove the entry
					dataBase.erase(dataBase.begin()+i);
				}
			}
		}
	}
};
/*------end of class cResourceManager declaration------*/

#endif // _RESOURCEMGR_H_
