/********************************************************
*	timing.h											*
*	By Shady El Mously									*
*	this is the header for the timing module			*
*	which include a timer class that may be used		*
*	for profiling										*
********************************************************/

#ifndef _TIMING_H_
#define _TIMING_H_
#include <windows.h>	//	we need QueryPerformanceFrequency()

#include <deque>
using std::deque;

/********************************************************
*	class cTimer										*
*	represents a timer object							*
*	which can be used in profiling and controlling		*
*	the frame rate										*
*	it can be used as a stop watch to see the time of	*
*	each function										*
********************************************************/
class cTimer
{
private	:

	//	union to reference the same data as multiple types
	union
	{
		struct
		{
			LARGE_INTEGER	m_iBeginTime	;
			LARGE_INTEGER	m_iEndTime		;
		};
		struct
		{
			LONGLONG		m_lBeginTime	;
			LONGLONG		m_lEndTime		;
		};
	};


	deque<LONGLONG>	m_TicksQueue;

public	:

	union
	{
		LARGE_INTEGER	m_PerfFrequency	;	//	performance frequency of the current system
		LONGLONG		m_lPerfFrequency;	
	};

	
	

	//	number of tick for the profile
	LONGLONG	m_NumTicks;

	//	constructor..just count the frequency
	cTimer(int QueueSize=20)
	{
		QueryPerformanceFrequency(&m_PerfFrequency);	
		//	fill the queue with 50 Zeros
		for(int i=0;i<QueueSize;i++)
			m_TicksQueue.push_back(0);
	}

	//	this will get the tick counter value
	LARGE_INTEGER	GetTicks()
	{
		DWORD dwLow,dwHigh;
		LARGE_INTEGER	Ticks	;
		__asm
		{
			rdtsc
			mov dwLow,eax
			mov dwHigh,edx
		}
		Ticks.QuadPart=((unsigned __int64)dwHigh<<32)|(unsigned __int64)dwLow;
		return Ticks;
	}
	//	start and end proficling
	void Begin()
	{	m_iBeginTime=GetTicks();	}

	void End();
	//	this will convert the last profiling time into milliseconds
	double ProfileTime()
	{	return m_NumTicks*1.0/m_lPerfFrequency;	}

	
	friend class cFrameController;
};
/*-------------end of class cTimer declaration---------*/

#endif // _TIMING_H_