#ifndef _TREE_H
#define _TREE_H
/****************************************************************
*	tree.h														*
*	By Shady El Mously											*
*	this is a header declaration and implementation for a		*
*	generic templatized tree class								*
*	this part can be used to construct any kind of trees		*
*	the tree nodes, do not own the	children					*
*	this template class will make it easier to construct tree	*
*	based structres												*
****************************************************************/
#include <vector>


/****************************************************************
*	class cTree													*
*	template class representing a generic tree					*
*	for object which have any number of child objects			*
*	and Zero or one parent objects								*
*	this class is templatized, to use it, using inheritance		*
*	i typethe new class name as template argumanet				*
*	all functions here are inline for speeding stuff and because*
*	it is templatized											*
****************************************************************/
template <typename T>
class cTree
{
	public:

	vector<T*>	m_Childs;		//	any number of children, (pointers to them)
	T*			m_Parent;		//	a single pointer to the parent

	//	construct set parent to NULL
	cTree()
	{	m_Parent	=	NULL;	}


	//	AttachTo()
	//	attach the tree to another  tree
	//	make the patent pointer here point to it
	//	and Add its adress to its childs
	void AttachTo(T& Other)
	{
		Other.m_Childs.push_back((T*)this);
		m_Parent	=	&Other;
	}


	//	GetRoot()
	//	return a pointer to the root node in the tree
	//	GetRoot()
	//	Goes up in the box tree parent by parent until it reaches the top part
	//	also very useful
	T*	GetRoot()
	{
		//	reach tree top and draw from there always
		//	the box top could be this same box itself
		T* BoxTop=((T*)this);
		while(BoxTop->m_Parent)
		{		BoxTop=BoxTop->m_Parent;	}
		return BoxTop;
	}


	//	GetPathFromRoot()
	//	retuns an array of pointers to nodes, sorted by the same path from root to
	//	the node here, this function will use recursion
	//	this path can be used to compute anything later
	void	GetPathFromRoot(vector<T*>& Path)
	{
		T*	pNode;		//	the node we start with
		pNode=(T*)this;			//	start from this Address

		//	clear the sent vector first
		Path.clear();
		Path.push_back(pNode);

		//	as long as there is a parent
		while(pNode->m_Parent)
		{
			pNode=pNode->m_Parent;			//	jump to parent
			Path.push_back(pNode);	//	Add the node found
		}

		//	now the vector has all the path, but reversed
		//	i should reverse it
		reverse(Path.begin(),Path.end());
	}

	//	DetachFromParent()
	//	the node removes its Address from the parent list of Addresses
	//	it SHOULD find its own adress of course
	//	the function does nothing if there is no parent
	void DetachFromParent()
	{
		//	if there is no parent, go back
		if(m_Parent==NULL)return;

		//	save your own Address
		T* myadress=(T*)this;

		//	find it in the parent list of Addresses
		for(unsigned int i=0;i<m_Parent->m_Childs.size();i++)
		{
			if(myadress==m_Parent->m_Childs[i])
				break;
		}

		//	now remove it from the parent data
		m_Parent->m_Childs.erase(m_Parent->m_Childs.begin()+i);

		//	make parent here a NULL 
		m_Parent=NULL;
	}


	//	GetAllTreeNodes()
	//	goes to the top of the tree
	//	and take all the tree node into a single array and return them
	void GetAllTreeNodes(vector<T*>& AllNodes)
	{
		//	go up the tree
		T*	RootNode	=	GetRoot();
		AllNodes.clear();
		RootNode->GetAllTreeRec(AllNodes);
	}

	//	GetAllChildren
	//	Gets all the child nodes in the tree
	void GetAllChilds(vector<T*> AllNodes)
	{
		AllNodes.clear();
		GetAllTreeRec(AllNodes);
	}


	//	GetAllTreeRec()
	//	recursive function that will be used to get all the nodes in the tree
	//	this is very useful for collision detection
	void GetAllTreeRec(vector<T*>& AllNodes)
	{
		AllNodes.push_back((T*)this);
		for(int i=0;i<m_Childs.size();i++)
		{
			m_Childs[i]->GetAllTreeRec(AllNodes);
		}
	}

	//	IsChildOf()
	//	return true if the sent node is anywhere up in the tree
	bool IsChildOf(T* pNode)
	{
		T* thisnode=(T*)this;

		while(thisnode)
		{
			if(thisnode==pNode)
				return true;
			thisnode=thisnode->m_Parent;
		}
		return false;
	}
};
/*--------------end of class cTree declaration & implementation------*/
#endif			//	_TREE_H